varying vec3 normal;

/**
 * Returns:
 *  direction - direction from the vertex to the light
 *  color - color of the light
 **/
void calculateLight(out vec3 direction, out vec3 color);

/**
 * Inputs:
 *  L - direction to the light
 *  Lcolor - color of the light
 *  N - surface normal
 * Returns:
 *  Final color of the surface
 **/
vec3 calculateMaterial(vec3 L, vec3 Lcolor, vec3 N);


vec3 calculateFog(vec3 incolor, float distance)
{
    const float fogDensity = 0.003;
    const vec3 fogColor = vec3(0.0);//vec3(0.10, 0.10, 0.10);
    float fog = exp2(distance * (-fogDensity * 1.442695));
    fog = clamp(fog, 0.0, 1.0);

    return mix(fogColor, incolor, fog);
}

void main()
{
    vec3 lightDir;
    vec3 lightColor;
    calculateLight(lightDir, lightColor);
    
    vec3 N = normalize(normal);
    vec3 color = calculateMaterial(lightDir, lightColor, N);

    gl_FragColor = vec4(color, 1.0);
}
