/*
 * Copyright 2009  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef DATASTORE_H
#define DATASTORE_H

#include <QObject>

#include <QHash>
#include <QVector>

class Model;
class ShaderManager;

namespace KGLLib
{
class Texture;
class Program;
class GeometryBuffer;
}

class DataStore : public QObject
{
    Q_OBJECT
    public:
        static void initInstance(const QString& datadir);
        static DataStore* instance()  { return mInstance; }

        QString dataDir() const  { return mDataDir; }

        Model* model(const QString& name);
        Model* randomAsteroidModel();
        KGLLib::Texture* texture(const QString& name);
        KGLLib::Program* program(const QString& name);
        ShaderManager* shaderManager() const  { return mShaderManager; }

        bool loadAsteroidModels();
        bool loadModel(const QString& name);

        const QHash<QString, Model*>& nonFinalizedModels() const  { return mNonFinalizedModels; }
        bool finalizeModels();

    signals:
        void loadingAsteroid(const QString& name, int i, int total);

    protected:
        DataStore(const QString& datadir);
        ~DataStore();

    private:
        static DataStore* mInstance;

        QString mDataDir;

        QHash<QString, Model*> mModels;
        QHash<QString, KGLLib::Texture*> mTextures;
        QHash<QString, KGLLib::Program*> mPrograms;
        QList<KGLLib::GeometryBuffer*> mGeometryBuffers;
        ShaderManager* mShaderManager;

        QList<Model*> mAsteroidModels;
        QHash<QString, Model*> mNonFinalizedModels;
};

#endif // DATASTORE_H
