/*
    Copyright 2008-2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "displaywidget.h"

#include "gameview.h"
#include "datastore.h"

#include <kgllib/camera.h>
#include <kgllib/widgetproxy.h>
#include <kgllib/textrenderer.h>
#include <kgllib/hdrglwidgetcontrol.h>

#include <KDebug>
#include <KStandardDirs>
#include <QTimer>
#include <QMouseEvent>
#include <QLayout>

using namespace KGLLib;


DisplayWidget::DisplayWidget(QWidget* parent) : HdrGLWidget(createGLFormat(), parent)
{
    mProxy = 0;

    setAutoExposureRange(0.4, 2.5);
    setAutoExposureTarget(0.15);
}

DisplayWidget::~DisplayWidget()
{
}

QGLFormat DisplayWidget::createGLFormat()
{
    QGLFormat format;
    format.setAlpha(true);
    format.setDepthBufferSize(24);
    // Other format options are left to default values

    return format;
}

WidgetProxy* DisplayWidget::proxy()
{
    if (!mProxy) {
        kDebug() << "Creating new Proxy";
        mProxy = new WidgetProxy(this, static_cast<QWidget*>(parent()));
    }
    return mProxy;
}

void DisplayWidget::appendView(GLView* view)
{
    mViews.append(view);

    if (glInitialized()) {
        view->init();
        view->resize(width(), height());
    }
}

void DisplayWidget::removeView(GLView* view)
{
    mViews.removeAll(view);
}

void DisplayWidget::mouseMoveEvent(QMouseEvent* e)
{
    if (e->buttons() & Qt::LeftButton) {
        emit signalRotateShip(e->pos().x() - mMousePos.x(), e->pos().y() - mMousePos.y());

        QWidget* w = static_cast<QWidget*>(parent());
        mMousePos = QPoint(w->width()/2, w->height()/2);
        QCursor::setPos(w->mapToGlobal(mMousePos));
    } else {
        mMousePos = e->pos();
    }
}

void DisplayWidget::initializeGL()
{
    kDebug();
    // Find where our data is
    setDataPath(DataStore::instance()->dataDir() + "shaders/");

    HdrGLWidget::initializeGL();
    setAutoExposure(true);
    setHdrRenderingActive(false);
    glClearDepth(1.0);

    camera()->setFoV(90);
    camera()->setDepthRange(1, 15000);

    foreach (GLView* v, mViews) {
        v->init();
    }
}

void DisplayWidget::resizeGL(int width, int height)
{
    KGLLib::HdrGLWidget::resizeGL(width, height);

    foreach (GLView* v, mViews) {
        v->resize(width, height);
    }
}

void DisplayWidget::render()
{
    KGLLib::HdrGLWidget::render();

    foreach (GLView* v, mViews) {
        if (v->viewType() == GLView::View_GUI) {
            v->render();
        }
    }
}

void DisplayWidget::renderScene()
{
    foreach (GLView* v, mViews) {
        if (v->viewType() != GLView::View_GUI) {
            v->render();
        }
    }
}

void DisplayWidget::showHdrConfig()
{
    QWidget* hdrControlWin = mProxy->createWindow("HDR Controls");
    hdrControlWin->layout()->addWidget(new HdrGLWidgetControl(this, hdrControlWin));
    hdrControlWin->move(10, 300);
    mProxy->addWidget(hdrControlWin);
}

