/*
    Copyright 2008-2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GAMEVIEW_H
#define GAMEVIEW_H


#include "glview.h"

class GameWorld;
class GameObject;
class ShaderManager;

namespace KGLLib
{
    class Texture;
    class Mesh;
}


class GameView : public GLView
{
    Q_OBJECT
    public:
        GameView(KGLLib::GLWidget* parent, GameWorld* world);
        ~GameView();

        virtual void init();
        virtual void render();

    public slots:
        void increaseDetailLevel();
        void decreaseDetailLevel();

    signals:
        void dataLoadingProgress(int done, int total, const QString& text);

    protected:
        void setupWorldView();

        void renderSpace();
        void renderObjects(const QList<GameObject*>* objects);

    private:
        GameWorld* mWorld;
        float mDetailLevel;
        KGLLib::Texture* mSpaceTexture;
        KGLLib::Mesh* mSpaceMesh;
        KGLLib::Texture* mEarthTex;
        KGLLib::Mesh* mEarthMesh;
        ShaderManager* mShaderManager;
};

#endif // GAMEVIEW_H
