/*
    Copyright 2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "glview.h"

#include <kgllib/glwidget.h>


GLView::GLView(KGLLib::GLWidget* parent): QObject(parent)
{
    mCamera = 0;
    mViewType = View_Scene;
}

GLView::~GLView()
{
    delete mCamera;
}

KGLLib::GLWidget* GLView::glWidget() const
{
    return static_cast<KGLLib::GLWidget*>(parent());
}

KGLLib::Camera* GLView::camera() const
{
    return mCamera ? mCamera : glWidget()->camera();
}

void GLView::setCamera(KGLLib::Camera* c)
{
    delete mCamera;
    mCamera = c;
}

KGLLib::FPSCounter* GLView::fpsCounter() const
{
    return glWidget()->fpsCounter();
}

KGLLib::TextRenderer* GLView::textRenderer() const
{
    return glWidget()->textRenderer();
}

void GLView::setViewType(GLView::ViewType v)
{
    mViewType = v;
}

void GLView::update()
{
    glWidget()->update();
}

void GLView::init()
{
}

void GLView::resize(int w, int h)
{
    Q_UNUSED(w);
    Q_UNUSED(h);
}

void GLView::render()
{
}

void GLView::setupOrthoView(int w, int h)
{
    if (w == -1) {
        w = glWidget()->width();
    }
    if (h == -1) {
        h = glWidget()->height();
    }

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // Same as  gluOrtho2D(0, w, h, 0);
    glOrtho(0, w, h, 0, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}
