/*
    Copyright 2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GLVIEW_H
#define GLVIEW_H

#include <QObject>

namespace KGLLib
{
    class GLWidget;
    class Camera;
    class FPSCounter;
    class TextRenderer;
}

class GLView : public QObject
{
    Q_OBJECT
    public:
        GLView(KGLLib::GLWidget* parent);
        ~GLView();

        KGLLib::GLWidget* glWidget() const;

        virtual void init();
        virtual void render();
        virtual void resize(int w, int h);

        enum ViewType {
            View_Scene = 0,
            View_GUI = 1
        };
        ViewType viewType() const  { return mViewType; }

    public slots:
        void update();

    protected:
        KGLLib::Camera* camera() const;
        KGLLib::FPSCounter* fpsCounter() const;
        KGLLib::TextRenderer* textRenderer() const;

        void setCamera(KGLLib::Camera* c);
        void setViewType(ViewType v);

        void setupOrthoView(int w = -1, int h = -1);

    private:
        KGLLib::Camera* mCamera;
        ViewType mViewType;
};

#endif // GLVIEW_H
