/*
    Copyright 2008-2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SHADERMANAGER_H
#define SHADERMANAGER_H

#include <QString>
#include <QHash>
#include <Eigen/Core>

namespace KGLLib
{
    class Program;
}


class ShaderManager
{
    public:
        ShaderManager(const QString& shaderDir);
        ~ShaderManager();

        KGLLib::Program* activateShader();
        void disableShader();

        bool precompileProgram(const QString& renderMode, const QString& lightType, const QString& material);

        void setNoLight();
        void setLight(const QString& type);
        void setLightColor(const Eigen::Vector3f& color);
        void setLightPosition(const Eigen::Vector3f& position);
        void setLightDirection(const Eigen::Vector3f& dir);
        void setLightRadius(float r);

        void setMaterial(const QString& type);

        void setRenderMode(const QString& mode);

        struct ShaderInfo
        {
            QString renderMode;
            QString lightType;
            QString materialType;

            bool operator==(const ShaderInfo& info) const
            {
                return renderMode == info.renderMode &&
                        lightType == info.lightType &&
                        materialType == info.materialType;
            }

            Eigen::Vector3f lightColor;
            Eigen::Vector3f lightPosition;
            Eigen::Vector3f lightDirection;
            float lightRadius;
        };

    private:
        KGLLib::Program* compileProgram(const ShaderInfo& info);

        QString mShaderDir;
        QHash<ShaderInfo, KGLLib::Program*> mPrograms;
        KGLLib::Program* mActiveProgram;
        ShaderInfo mActiveInfo;
};

#endif // SHADERMANAGER_H
