/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2011, Dmitry Chernov diman4ik.chernov@gmail.com
 */

#ifndef BACKGAMMON_FIBS
#define BACKGAMMON_FIBS

/*

  This is the FIBS backgammon engine. It handles all communication
  with the server.

*/

#include <QRegExp>
#include <QMap>
#include <QPair>

#include "own_info.h"
#include "move.h"
#include "backgammongame.h"

class BackgammonFibsConnect;


class BackgammonFibs : public BackgammonGame
{
    Q_OBJECT
public:
    enum PStatus { Ready, Away, Blind };
    
    enum state
    {
        connected,
        authorized,
        playing,
    };
    
    enum toggles
    {
        allowpip,
        autoboard,
        autodouble,
        automove,
        away,
        bell,
        crawford,
        edouble,
        greedy,
        moreboards,
        moves,
        notify,
        ratings,
        ready,
        report,
        silent 
    };
        
    explicit BackgammonFibs( QObject *parent = 0 );
    
    virtual ~BackgammonFibs();
    
    void processUIEvent( QGraphicsSceneMouseEvent* event, BackgammonScene* board );
    
    virtual void startNewGame();
    
public slots:
    /**
     * Undo the last move
     */
    //virtual void undo();

    /**
     * Redo the last move
     */
    //virtual void redo();

    /**
     * Roll dice for whoevers turn it is
     */
    virtual void roll();

    /**
     * Double the cube for whoevers can double right now
     */
    virtual void doubleCube();

    /**
     * Reload the board to the last known sane state
     */
    virtual void load();

    /**
     * Commit a move
     */
    virtual void done();

    //void handleCommand(const QString &cmd);

    void fibsRequestInvitation(const QString &player);
    
    void setupOk();
    
    void connectFIBS( const QString& serv, qint16 port );
    void login( const QString& login, const QString& password );
    void message( const QString& name, const QString& text );
    void tell( const QString& name, const QString& text );
    void invite( const QString& name, int length );
    void join( QString name );
    
    void say( const QString& say );
    void kibitz( const QString& kibitz );
    
    void sendLine( QString line );
    
    void toggle( int toggle, bool state );
    
    void accept();
    void reject();
    
    //void questionConfirmed( const QString& question );
    //void questionRejected( const QString& question );

signals:
        
    void connectionEstablished();

    void serverString(const QString &s);

    void fibsWhoInfo(QString *line);
    void fibsWhoEnd();
    void fibsLogout(QString *line);
    void fibsLogin(QString *line);

    void fibsConnectionClosed();

    void childrenReadConfig();
    void childrenSaveConfig();

    void changePlayerStatus(const QString &, int, bool);

    void fibsGamesInformation( QString msg );
    void fibsChatInformation( QString msg );
    void fibsMessage( QString str );
    void serverInfo( QString str );
    
    void board();       // Внутренний формат передачи доски надо еще продумать, необязательно строка

    void fibsStartNewGame(const QString &msg);
    void fibsGameOver();
    
    void togglesLoaded( const QMap<int, bool>& toggles );
    
    void newWhoInfo( WhoInfo info );
    
    void resumeInvite( QString );
    void playInvite( const QString& name, int length );
    
    void showRollOrDoubleQuestion();
    void showAcceptRejectQuestion();
    
    void opponentMoves( const QVector<Move> moves );
    
    //void roll( int, int );
    
    void yourTurn();
    
    void showYesNoDialog( const QString& , const QString& ,
                          const QString& , const QString&  );
                          
    void logouts( const QString& name );

protected slots:

    void invitationDone();

    void handleServerData( QString& line );

    void prepareMenu();

    void disconnectFIBS();

    void serverDisconnected();

    void newBoard(QString *line);

    void setAllowed(int cmd, bool f);
private:
    state m_state;
    
    QString getLine( QString& line_given );
    QString getAll( QString& line_given, QRegExp end_pattern );
    void getAll( QString& line_given, QRegExp end_pattern, QVector<QString>& vec );
    void skipAll( QString& line_given, QRegExp end_pattern );
    QString getFirstWord( const QString& line );
    QString getLastWord( const QString& line );
    
    BackgammonFibsConnect* m_connection;
    
    QRegExp m_patternWelcome;
    QRegExp m_patternOwnInfo;
    QRegExp m_patternBanner;
    QRegExp m_patternMotdBegin;
    QRegExp m_patternMotdEnd;
    QRegExp m_patternYourLogin;
    QRegExp m_patternServerTime;
    QRegExp m_patternWhoInfoBegin;
    QRegExp m_patternWhoInfoEnd;
    QRegExp m_patternShouts;
    QRegExp m_patternWhispers;
    QRegExp m_patternSays;
    QRegExp m_patternMessage;
    QRegExp m_patternMessDelivered; 
    QRegExp m_patternMessSaved;
    QRegExp m_patternKibitzes;
    QRegExp m_patternYouSays;
    QRegExp m_patternYouShouts;
    QRegExp m_patternYouWhispers;
    QRegExp m_patternYouKibitzes;
    QRegExp m_patternUnknown;
    QRegExp m_patternLogins;
    QRegExp m_patternLogouts;
    QRegExp m_patternInviteResume;
    QRegExp m_patternInvitePlay;
    QRegExp m_patternInviteEnd;
    
    QRegExp m_patternMatchBegin1, m_patternMatchBegin2, m_patternMatchBegin3, m_patternMatchBegin4;
    QRegExp m_patternGameReload1, m_patternGameReload2;
    
    QRegExp m_patternInviteConfirm;
        
    QRegExp m_patternYourTurn, m_patternPleaseMove;    
    
    QRegExp m_patternGameBegin;
    
    QRegExp m_patternBoard;
    
    QRegExp m_patternBoardBegin;
    QRegExp m_patternBoardEnd;
    
    QRegExp m_patternRoll;
    QRegExp m_patternOpponentRoll;
    
    QRegExp m_patternOpponentFirst;
    QRegExp m_patternOpponentMove;
    
    QRegExp m_patternOpponentDoubles;
    QRegExp m_patternAcceptDone;
    
    QRegExp m_patternInfo;
    
    QRegExp m_patternYouWin;
    QRegExp m_patternYouLost;
    
    QRegExp m_patternTurn;
    QRegExp m_patternMatchLength;
    QRegExp m_patternPointsFor;
    
    QRegExp m_patternCantMove;
    
    QRegExp m_patternMatchEndLost;
    QRegExp m_patternMatchEndWon;
    
    QRegExp m_patternBlind;
    QRegExp m_patternUnblind;
    QRegExp m_patternStartWatch;
    QRegExp m_patternStopWatch;

    QRegExp m_patternWave1;
    QRegExp m_patternWave2;
    QRegExp m_patternWave3;

    
    QMap< int, bool > m_toggles;
    
    QString m_name;
    //QString m_opponentName;
    QString m_password;
    
    //QString m_askedInvite;
    QVector<QString> m_invites;
    
    double m_rating;
    int m_experience;
    int m_redoubles;
    QString m_timezone;
    
    void getMyOwnInfo( const QString& line );
    void getWhoInfo( const QString& line );
    void getInvite( const QString& line );
    void getOpponentMove( const QString& line );
    Move getMoveFromStr( const QString& line_part );
    void getRoll( const QString& line );
    Rolls getRolls( const QString& line );
    void getGameBegin( QString& line );
    void readBoard( const QVector<QString>& line );
    
    bool m_yourTurn;
    
    IPlayer& getYou() const
    {   return *m_players[0];    }
    IPlayer& getOpponent() const
    {   return *m_players[1];    }
    
    /*IPlayer* getYou() const
    {   return *m_players[0];    }
    IPlayer* getOpponent() const
    {   return *m_players[1];    }*/
    
    enum pattern
    {
        patternWelcome,
        patternOwnInfo,
        patternBanner,
        patternMotdBegin,
        patternMotdEnd,
        patternYourLogin,
        patternServerTime,
        patternWhoInfoBegin,
        patternWhoInfoEnd,
        patternShouts,
        patternWhispers,
        patternSays,
        patternMessage,
        patternMessDelivered,
        patternMessSaved,
        patternKibitzes,
        patternYouSays,
        patternYouShouts,
        patternYouWhispers,
        patternYouKibitzes,
        patternUnknown,
        patternLogins,
        patternLogouts,
        patternInviteResume,
        patternInvitePlay,
        patternInviteEnd,
        //patternMatchBegin1, 
        //patternMatchBegin2,
        patternInGameYouInvited,
        //patternMatchBegin3, 
        //patternMatchBegin4;
        patternInGameOpponentInvited,
        //patternGameReload1,
        patternInGameOpponentInvitedReload,
        //patternGameReload2,
        patternInGameYouInvitedReload,
        patternInviteConfirm,
        patternYourTurn,
        patternPleaseMove,
        patternGameBegin,
        patternBoard,
        /*patternBoardBegin,
        patternBoardEnd,*/
        patternRoll,
        patternOpponentRoll,
        patternOpponentFirst,
        patternOpponentMove,
        patternOpponentDoubles,
        patternAcceptDone,
        patternInfo,
        patternYouWin,
        patternYouLost,
        patternTurn,
        patternMatchLength,
        patternPointsFor,
        patternCantMove,
        patternMatchEndLost,
        patternMatchEndWon,
        patternBlind,
        patternUnblind,
        patternStartWatch,
        patternStopWatch,
        patternWave1,
        patternWave2,
        patternWave3,
        noPattern,
    };
    
    pattern getPattern( const QString& line );
    
    bool m_needBoard;
    bool m_needReverse;

    void sendMove();
    
    QVector<Move> m_myLastMoves;
    
    int reverseIndex(int);
};

#endif
