/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   Copyright (C) 2011, Dmitry Chernov                                    *
 *   diman4ik.chernov@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/


#include "mainwindow.h"
#include "backgammonview.h"
#include "backgammonscene.h"
#include "backgammongame.h"
#include "boardrenderer.h"
#include "historywidget.h"
#include "settings.h"
//#include "game_widget.h"
#include "fibs/fibs_widget.h"
#include "fibs/backgammon_fibs.h"
#include "backgammon_local.h"

#include <QtGui/QDropEvent>
#include <QtGui/QPainter>
#include <QDesktopWidget>
#include <QGridLayout>
#include <KPasswordDialog>
#include <KMessageBox>

#include <kconfigdialog.h>
#include <kstatusbar.h>

#include <kaction.h>
#include <kactioncollection.h>
#include <kstandardaction.h>
#include <kstandardgameaction.h>
#include <ktoggleaction.h>

#include <KDE/KLocale>


static BackgammonFibs* cretateBackgammonFIBS( BackgammonMainWindow* window, FibsWidget* widget );


BackgammonMainWindow::BackgammonMainWindow()
    : KXmlGuiWindow(), m_backgammonFibs(0), m_backgammonLocal(0)
{
    QWidget *mainWid = new QWidget;
    // TODO: replace grid layout with something more simple/suitable
    QGridLayout *lay = new QGridLayout(mainWid);
    lay->setColumnStretch( 0, 1 );
    lay->setMargin(1);

    m_scene = new BackgammonScene();

    m_view = new BackgammonView(m_scene, mainWid);
    lay->addWidget(m_view, 0, 0, 2, 1);

    m_history = new HistoryWidget(i18n("Game History"), this);
    addDockWidget( Qt::RightDockWidgetArea, m_history);
    kDebug() << "History visible:"<< Settings::showHistory();
    m_history->setVisible(Settings::showHistory());

    // accept drag and drop
    setAcceptDrops(true);

    // tell the KXmlGuiWindow that this is indeed the main widget
    //setCentralWidget(mainWid);
    QTabWidget* tw = new QTabWidget(this);
    setCentralWidget(tw);
    
    tw->addTab( mainWid, "Game" );
    //GameWidget* gw = new GameWidget(this);
    //tw->addTab( gw, "Game" );
    
    /*QWidget* fibs = new QWidget(this);
    
    tw->addTab( fibs, "FIBS" );*/
    m_fibsWidget = new FibsWidget(this);
    tw->addTab( m_fibsWidget, "FIBS" );

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();
    statusBar()->insertItem( "Pip:" , 0 , 0 );
    statusBar()->insertItem( "", 1, 0 );

    // a call to KXmlGuiWindow::setupGUI() populates the GUI
    // with actions, using KXMLGUI.
    // It also applies the saved mainwindow settings, if any, and ask the
    // mainwindow to automatically save settings if changed: window size,
    // toolbar position, icon size, etc.
    setupGUI( BoardRenderer::defaultTableSize() );
    
    if( m_backgammonFibs == 0 )
        m_backgammonFibs = cretateBackgammonFIBS();
    
    if( m_backgammonLocal == 0 )
        m_backgammonLocal = new BackgammonLocal(this);
}

BackgammonMainWindow::~BackgammonMainWindow()
{
}

void BackgammonMainWindow::setupActions()
{
    KStandardGameAction::gameNew(this, SLOT(slotNewGame()), actionCollection());
    KStandardGameAction::quit(qApp, SLOT(quit()), actionCollection());
    
    KAction* new_fibs = new KAction(this);
    new_fibs->setText( i18n("Play at FIBS") );
    actionCollection()->addAction( "new_fibs", new_fibs );
    connect( new_fibs, SIGNAL( triggered(bool) ), SLOT( newFIBSGame() ) );

    m_undoAct = KStandardGameAction::undo(this, SLOT(slotUndo()), actionCollection());
    m_undoAct->setEnabled( false ); // nothing to undo at the start of the game

    m_demoAct = KStandardGameAction::demo(this, SLOT(slotToggleDemoMode()), actionCollection());

    m_showLegalMovesAct = new KToggleAction( KIcon("legalmoves"), i18n("Show Legal Moves"), this );
    m_showLegalMovesAct->setChecked(Settings::legalMoves());
    actionCollection()->addAction("show_legal_moves", m_showLegalMovesAct);
    connect( m_showLegalMovesAct, SIGNAL(triggered(bool)), SLOT(slotShowLegalMoves(bool)) );

    m_showHistoryAct = new KToggleAction( KIcon("view-history"), i18n("Show Game History"), this );
    m_showHistoryAct->setChecked(Settings::showHistory());
    m_showHistoryAct->setShortcut(Qt::CTRL + Qt::Key_H);
    actionCollection()->addAction("show_history", m_showHistoryAct);
    connect( m_showHistoryAct, SIGNAL(triggered(bool)), SLOT(slotHistoryVisible(bool)) );

    KStandardAction::preferences(this, SLOT(slotPreferences()), actionCollection());

    connect(m_history, SIGNAL(closed()),  SLOT(slotUncheckHistoryMenuItem()) );
}

void BackgammonMainWindow::slotShowLegalMoves(bool newValue)
{
//TODO:
}

void BackgammonMainWindow::slotUncheckHistoryMenuItem()
{
    slotHistoryVisible(false);
}

void BackgammonMainWindow::slotHistoryVisible(bool newValue)
{
    kDebug() << "Request: history visible: " << newValue << ", sender="<<sender();

    if (Settings::showHistory() != newValue)
    {
        Settings::setShowHistory(newValue);
        Settings::self()->writeConfig();

        m_history->setVisible(newValue);
        m_showHistoryAct->setChecked(newValue);
    }
}

IGameEngine* BackgammonMainWindow::selectedGame()
{
//TODO: when we have more games, return corresponding instance. Now it's reasonable to limit by backgammon only
// switch(Settings::gameType()){
// case Backgammon: return new BackgammonGame();
// case Nackgammon: return new BackgammonGame("-2:-2:0:0:0:4:0:3:0:0:0:-4:4:0:0:0:-3:0:-4:0:0:0:2:2:0:0:0:0");
// case Fevga: return new FevgaGame();
// case Nardy: return new NardyGame();
// so on...}

    //return new BackgammonGame(); 
    return m_backgammonLocal;
}

void BackgammonMainWindow::slotNewGame()
{
    IGameEngine* game = selectedGame(); 
    connect( game, SIGNAL(gameEvent(QString)), SLOT(slotAddHistoryLine(QString)));
    connect( game, SIGNAL( pipCountChanged( const QString& ) ), SLOT( slotShowPip( const QString& ) ) );
    connect( game, SIGNAL( engineMessage( const QString& ) ), SLOT( slotChangeStatus( const QString& ) ) );

    m_scene->setGame(game);
    m_history->clear();
    game->startNewGame();
}

void BackgammonMainWindow::slotChangeStatus( const QString& statusInfo )
{
    statusBar()->changeItem( statusInfo, 1 );
}

void BackgammonMainWindow::slotAddHistoryLine( const QString& msg )
{
    m_history->addHistoryLine(msg);
}

void BackgammonMainWindow::slotUndo()
{
    //TODO: stub
}

void BackgammonMainWindow::slotToggleDemoMode()
{
    //TODO: stub
}

void BackgammonMainWindow::slotPreferences()
{
    // The preference dialog is derived from prefs_base.ui
    //
    // compare the names of the widgets in the .ui file
    // to the names of the variables in the .kcfg file
    //avoid to have 2 dialogs shown
    if ( KConfigDialog::showDialog( "settings" ) )  {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
    QWidget *generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    ui_prefs_base.kcfg_animationSpeed->setEnabled(ui_prefs_base.kcfg_animationEnabled->isChecked());
    ui_prefs_base.animationSpeed2->setEnabled(ui_prefs_base.kcfg_animationEnabled->isChecked()); //FIXME: is it possible to move to ui/kcfg file?
    dialog->addPage(generalSettingsDlg, i18n("Board"), "games-config-board");
//     //connect(dialog, SIGNAL(settingsChanged(QString)), m_view, SLOT(settingsChanged()));
    dialog->setAttribute( Qt::WA_DeleteOnClose );
    dialog->show();
}

void BackgammonMainWindow::newFIBSGame()
{
    //TODO : check if we are already connected
    m_backgammonFibs->connectFIBS( "www.fibs.com", 4321 );
    
    //kDebug() << "new fibs game";
    
    KPasswordDialog dlg( this, KPasswordDialog::ShowUsernameLine );
    dlg.setPrompt( i18n( "Enter your FIBS login and password" ) );
    if( !dlg.exec() )
      return; //the user canceled
    
    m_backgammonFibs->login( dlg.username(), dlg.password() );
    
    connect( m_backgammonFibs, SIGNAL( gameEvent( const QString& ) ), SLOT( slotAddHistoryLine( const QString& ) ) );
    //connect( m_backgammonFibs, SIGNAL( pipCountChanged(QString) ), SLOT( slotChangeStatus(QString) ) );
    connect( m_backgammonFibs, SIGNAL( engineMessage( const QString& ) ), SLOT( slotChangeStatus( const QString& ) ) );

    m_scene->setGame( m_backgammonFibs );
    m_history->clear();
    //m_backgammonFibs->startNewGame();
}

BackgammonFibs* BackgammonMainWindow::cretateBackgammonFIBS()
{
    BackgammonFibs* ret = new BackgammonFibs(this);
            
    connect( m_fibsWidget, SIGNAL( sendLine( QString ) ), ret, SLOT( sendLine( QString ) ) );
    
    connect( m_fibsWidget, SIGNAL( message( const QString&, const QString& ) ), 
             ret, SLOT( message( const QString&, const QString& ) ) );
             
    connect( m_fibsWidget, SIGNAL( tell( const QString&, const QString& ) ), 
             ret, SLOT( tell( const QString&, const QString& ) ) );
            
    connect( m_fibsWidget, SIGNAL( invite( const QString&, int ) ), ret, SLOT( invite( const QString&, int ) ) );
    
    connect( m_fibsWidget, SIGNAL( toggle( int, bool ) ), ret, SLOT( toggle( int, bool ) ) );
    
    connect( ret, SIGNAL( serverInfo( QString ) ), m_fibsWidget, SLOT( info( QString ) ) );
    
    connect( ret, SIGNAL( fibsGamesInformation( QString ) ), m_fibsWidget, SLOT( game( QString ) ) );
    
    connect( ret, SIGNAL( fibsChatInformation( QString ) ), m_fibsWidget, SLOT( chat( QString ) ) );
    
    connect( ret, SIGNAL( togglesLoaded( const QMap<int, bool>& ) ), 
             m_fibsWidget, SLOT( setCheckedStates( const QMap<int, bool>& ) ) );
            
    connect( ret, SIGNAL( newWhoInfo( WhoInfo ) ), m_fibsWidget, SLOT( newWhoInfo( WhoInfo ) ) );
    
    connect( ret, SIGNAL( resumeInvite( QString ) ), m_fibsWidget, SLOT( resumeInvite( QString ) ) );
    
    connect( ret, SIGNAL( playInvite( QString, int ) ), m_fibsWidget, SLOT( playInvite( QString, int ) ) );
    
    connect( ret, SIGNAL( diceRolled( int, int ) ), m_fibsWidget, SLOT( roll( int, int ) ) );
    
    connect( ret, SIGNAL( opponentMoves( const QVector<Move>& ) ), 
            m_fibsWidget, SLOT( opponentMoves( const QVector<Move>& ) ) );
    
    connect( ret, SIGNAL( yourTurn() ), m_fibsWidget, SLOT( yourTurn() ) );
    
    connect( ret, SIGNAL( resumeInvite( const QString& ) ), 
             this, SLOT(  resumeInvite( const QString& ) ) );
             
    connect( ret, SIGNAL( playInvite( const QString& , int ) ), 
             this, SLOT( playInvite( const QString& , int ) ) );
             
    connect( ret, SIGNAL( showRollOrDoubleQuestion() ), SLOT( showRollOrDoubleQuestion() ) );
    connect( ret, SIGNAL( showAcceptRejectQuestion() ) , SLOT( showAcceptRejectQuestion() ) );
    
    connect( ret, SIGNAL( logouts( const QString& ) ), m_fibsWidget, SLOT( logouts( const QString& ) ) );
    
    return ret;
}

void BackgammonMainWindow::resumeInvite( const QString& name )
{
    int res = KMessageBox::questionYesNo( this,  i18n( "Player " ) + name + 
                                          i18n( " wants to resume a saved match with you" ),
                                          i18n( "Game Invitation" ), 
                                          KGuiItem( i18n( "Join" ) ), KGuiItem( i18n( "Reject" ) ) );
                                          
    if( res == KMessageBox::Yes )
    {
        m_backgammonFibs->join(name);
    }
}
    
void BackgammonMainWindow::playInvite( const QString& name, int length )
{
    int res = KMessageBox::questionYesNo( this,  i18n( "Player " ) + name + 
                                          i18n( " invites you for a " ) + QString::number(length) + i18n( " point match" ), 
                                          i18n( "Game Invitation" ), 
                                          KGuiItem( i18n( "Join" ) ), KGuiItem( i18n( "Reject" ) ) );
                                          
    if( res == KMessageBox::Yes )
    {
        m_backgammonFibs->join(name);
    }
}

void BackgammonMainWindow::showRollOrDoubleQuestion()
{
    int res = KMessageBox::questionYesNo( this,  i18n( "Please roll or double" ), 
                                          i18n( "Question" ), 
                                          KGuiItem( i18n( "Roll" ) ), KGuiItem( i18n( "Double" ) ) );
                                          
    if( res == KMessageBox::Yes )
    {
        m_backgammonFibs->roll();
    }
    else
        m_backgammonFibs->doubleCube();
}

void BackgammonMainWindow::showAcceptRejectQuestion()
{
    int res = KMessageBox::questionYesNo( this,  i18n( "Your opponent doubles please accept or reject" ), 
                                          i18n( "Question" ), 
                                          KGuiItem( i18n( "Accept" ) ), KGuiItem( i18n( "Reject" ) ) );
                                          
    if( res == KMessageBox::Yes )
    {
        m_backgammonFibs->accept();
    }
    else
        m_backgammonFibs->reject();
}

void BackgammonMainWindow::slotShowPip( const QString& info )
{
    statusBar()->changeItem( info, 0 );
}
