/***************************************************************************
 *   Copyright 2006-2007 Dmitry Suzdalev <dimsuz@gmail.com>                *
 *                  2009 Alexander Smirnov <alexander.v.smirnov@gmail.com> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef RENDERER_H
#define RENDERER_H

#include <QPainter>
#include <QRectF>

#include <kstandarddirs.h>


class KSvgRenderer;
class KPixmapCache;

class Renderer{

public:
    QRectF elementRect(const QString& elementId) const;
    /**
     * Returns "default" size of specified SVG element
     * @param elementId - name of SVG element
     * @return original size of SVG element
    */
    QSize elementSize(const QString& elementId) const;

    void renderElement ( QPainter *p, const QString& elementid, const QRectF& r ) const;

    virtual void onThemeLoadFailure();
    virtual void onThemeLoadSuccess();

protected:
    Renderer( int cacheSize = 3*1024, QString themeURL = KStandardDirs::locate( "appdata", "default_theme.svgz" ));
    ~Renderer();
private:
    /**
     *  Svg renderer instance
     */
    KSvgRenderer *m_renderer;
    KPixmapCache *m_cache;
};

#endif
