/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#ifndef ENGINEAMT_H
#define ENGINEAMT_H

/**
English American game rules, implemented as template.

	@author Piotr Banasiak <piotrbanasiak@onet.pl>
*/
template <class T, int bSize>
class EngineAmT : public AbstractEngineT<T,bSize>
{
public:
  enum gamePhase
  {
     earlyPhase=0,
     middlePhase,
     gambitPhase
  };
  enum Score
  {
    scoreMan=1000,
    scoreKing=1500,
    scoreKingInGambit=1500,
    scoreNextKing=1500
  };
  
  EngineAmT();
  ~EngineAmT();
    
    virtual int gamePhase(const BoardT<T>* const pBoard)const;
    void setHash( QHash<BoardT<T>,HashNode<LegalMoveT<T> > > *pHash ){pH=pHash;};
    virtual void fillBook( QList<QList<LegalMove> > &Book );

    virtual void genPossibleMoves( QList<LegalMoveT<T> > &MoveList ,const BoardT<T>* const pBoard );
    virtual void genPossibleMoves( QList<LegalMove > &MoveList ,const Board* const pBoard );
    virtual void reFillSQV( int size,int phase );
    
    QHash<BoardT<T>,HashNode<LegalMoveT<T> > > *pH;
private:
  virtual int materialScore( const BoardT<T> * const pBoard )const;
  
 void genNKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList);
 void genCapKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList=0)const;
 void genKCapPart(int src0, int srcNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B,QList<LegalMove>*pList=0, LegalMove *pPart=0 )const;
 inline bool genKCapPartDir(int src0, int srcNr, T opponent, int level,QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, const int* D,  QList<LegalMove> *pList, LegalMove *pPart)const;

 void genNManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList=0);
 inline void genManMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares, int* D ,QList<LegalMove> *pList);
 
 void genCapManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList=0);
 void genCapPart(int src0, int srcNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B,QList<LegalMove>*pList=0, LegalMove *pPart=0 );
 inline bool genCapPartDir(int src0, int srcNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, int* D,
                           QList<LegalMove>*pList, LegalMove *pPart );
 
 //void initTbyteNr();

 inline bool canOpponentCapture( const BoardT<T> &B )const;  
 inline bool checkDirection( int sqNr, const T& opponent, const T& squares, const int * D )const;
};



/////////////////////////////////////////////////////////////////////////////////////////////
// implementation 
/////////////////////////////////////////////////////////////////////////////////////////////

template <class T, int bSize>
EngineAmT<T, bSize>::EngineAmT():AbstractEngineT<T,bSize>()
{
   Q_ASSERT( bSize==8 or bSize==10 or bSize==12 );
  
}
//------------------------------------------------------------------
template <class T, int bSize>
EngineAmT<T, bSize>::~EngineAmT()
{
}
//------------------------------------------------------------------
/** normal moves of men FIXME 
  bytes are xxxx 31,30 .... 2,1 ,0
  where xxx are unused in case variable is longer then need
*/
template <class T, int bSize>
void EngineAmT<T, bSize>::genNManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList)
{
  Q_ASSERT( !B.captured );
  
  T myMen;
  T squares= B.blackK | B.blackM | B.whiteK | B.whiteM;
  
  if( B.isWhiteTurn())
    myMen=B.whiteM;
  else
    myMen=B.blackM;

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myMen;
  
  while( shift )
  {
    if(myMen & srcMask)
    {
      if( B.isWhiteTurn())
      {
        genManMovesDirection( sqNr, MoveList, squares, this->LU , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RU , pList);
      }
      else
      {
        genManMovesDirection( sqNr, MoveList, squares, this->LD , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RD , pList);
      }
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//-----------------------------------------------------------------------
template <class T, int bSize>
void EngineAmT<T, bSize>::genManMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares, int* D , QList<LegalMove> *pList)
{
  int d=D[s];
  if(d<0)return;//outside Board
  
  T M=0x01;
  M<<=d;
  
  bool noEmpty=squares & M;
  if( noEmpty )
    return;
    
    //add the move to the list
    LegalMoveT<T> lm;
    lm.src = s;
    lm.dst = d;
    MoveList.append( lm );
    
    if( pList )
    {
       QPoint start( this->Bx[s], this->By[s] );
       QPoint end( this->Bx[d], this->By[d] );
       LegalMove lm(start,end);
       
       pList->append( lm );
    }
}

//------------------------------------------------------------------
/** generates all possible capture  man moves 
    pLM is LegalMove with intermediate squares used in Gui
*/
template <class T, int bSize>
void EngineAmT<T, bSize>::genCapManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, QList<LegalMove> *pList )
{
  T myMen;

  Q_ASSERT( !B.captured );
  T opponent;
  
  if( B.isWhiteTurn())
  {
    myMen=B.whiteM;
    opponent=( B.blackM | B.blackK );
  }
  else
  {
    myMen=B.blackM;
    opponent=( B.whiteM | B.whiteK );
  }

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myMen;

  while( shift )
  {
    bool candidate=myMen & srcMask;
    if(candidate)
    {
      if( pList )
      {
         LegalMove part;
         QPoint start( this->Bx[sqNr], this->By[sqNr] );
         part.insPath( start,0 );
         genCapPart(sqNr, sqNr, opponent, 0, MoveList, B, pList, &part);
      }
      else
         genCapPart(sqNr, sqNr, opponent, 0, MoveList, B, 0, 0);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//-------------------------------------------------------------
// src0 very first position of man
// sqNr is where last multicapture is finished, we start checking from that position
template <class T, int bSize>
void EngineAmT<T, bSize>::genKCapPart(int src0, int sqNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, 
                                      QList<LegalMove> *pList, LegalMove *pPart)const
{
   bool atLeastOne=false;
   bool ok=false;
   
      ok=genKCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LU, pList, pPart);
      if(ok) atLeastOne=true;
      
      ok=genKCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RU, pList, pPart);
      if(ok) atLeastOne=true;

      ok=genKCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LD, pList, pPart);
      if(ok) atLeastOne=true;
   
      ok=genKCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RD, pList, pPart);
      if(ok) atLeastOne=true;
   
   if( !atLeastOne and level>0 )// or B.captured
   {
      LegalMoveT<T> lm;
      lm.src = src0;
      lm.dst = sqNr;
      lm.captured=B.captured;
      
      MoveList.append( lm );
      
      // and pPart represent finished/full multicapture move
      // so add it to move list
      
      if( pList )
      {
         pList->append( *pPart );
      }
   }
}
//-------------------------------------------------------------
// src0 very first position of man
// sqNr is where last multicapture is finished, we start checking from that position
template <class T, int bSize>
void EngineAmT<T, bSize>::genCapPart(int src0, int sqNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, 
                                      QList<LegalMove> *pList, LegalMove *pPart)
{
   bool atLeastOne=false;
   bool ok=false;
   
   if( B.isWhiteTurn())
   {
      ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LU, pList, pPart);
      if(ok) atLeastOne=true;
      
      ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RU, pList, pPart);
      if(ok) atLeastOne=true;
   }
   else
   {
      ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LD, pList, pPart);
      if(ok) atLeastOne=true;
   
      ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RD, pList, pPart);
      if(ok) atLeastOne=true;
   }
   
   if( !atLeastOne and level>0 )// or B.captured
   {
      LegalMoveT<T> lm;
      lm.src = src0;
      lm.dst = sqNr;
      lm.captured=B.captured;
      
      MoveList.append( lm );
      
      // and pPart represent finished/full multicapture move
      // so add it to move list
      
      if( pList )
      {
         pList->append( *pPart );
      }
   }
}

//------------------------------------------------------------------
// check/generate next captures in multicapture move for King
// D direction table 
// pList and pPart are to generate LegalMoves with intermediate steps.
template <class T, int bSize>
bool EngineAmT<T, bSize>::genKCapPartDir(int src0, int srcNr, T opponent, int level, 
     QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, const int* D,  QList<LegalMove> *pList, LegalMove *pPart)const
{
  bool captured=false;
  
  T squares= ( B.whiteM | B.whiteK | B.blackM | B.blackK | B.captured );
  
      int d1=D[srcNr];
      int d2=-1;
      if( d1>=0)d2=D[d1];
      if( d1>=0 and d2>=0 )//both on board
      {
        T d1Mask= (T(1) << d1);
        T d2Mask= (T(1) << d2);
        
        if( !(squares & d2Mask) and( d1Mask & opponent ) )
        {
          // generating intermediate information for Gui, building part of the move
          if( pPart )
          {
             QPoint start( this->Bx[srcNr], this->By[srcNr] );
             QPoint inter( this->Bx[d1], this->By[d1] );
             QPoint end( this->Bx[d2], this->By[d2] );
             
             pPart->insPath(end,level+1);
             pPart->insTaken(inter,level);
          }
          
          T inv = ~d1Mask;
          BoardT<T> b(B);
          
          //capture opponent
          b.captured |= d1Mask;
          b.whiteM &= inv;
          b.whiteK &= inv;
          b.blackM &= inv;
          b.blackK &= inv;
          
          //move my own
          T srcMask= 1;
          srcMask <<= srcNr;

          T opponent;
          const bool wTurn=b.isWhiteTurn();
          if( wTurn )
          {
            b.whiteM &= (~srcMask);
            b.whiteM |= d2Mask;
            opponent=( b.blackM | b.blackK );
          }
          else
          {
            b.blackM &= (~srcMask);
            b.blackM |= d2Mask;
            opponent=( b.whiteM | b.whiteK );
          }

          genKCapPart( src0, d2 , opponent, level+1, MoveList, b, pList, pPart );
          captured=true;
        }
      }

  return captured;
}


//------------------------------------------------------------------
// check/generate next captures in multicapture move
// D direction table 
// pLm and pPart are to generate LegalMoves with intermediate steps.
template <class T, int bSize>
bool EngineAmT<T, bSize>::genCapPartDir(int src0, int srcNr, T opponent, int level, 
     QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, int* D,  QList<LegalMove> *pList, LegalMove *pPart)
{
  bool captured=false;
  
  T squares= ( B.whiteM | B.whiteK | B.blackM | B.blackK | B.captured );
  
      int d1=D[srcNr];
      int d2=-1;
      if( d1>=0)d2=D[d1];
      if( d1>=0 and d2>=0 )//both on board
      {
        T d1Mask= (T(1) << d1);
        T d2Mask= (T(1) << d2);
        
        if( !(squares & d2Mask) and( d1Mask & opponent ) )
        {
          // generating intermediate information for Gui, building part of the move
          if( pPart )
          {
             QPoint start( this->Bx[srcNr], this->By[srcNr] );
             QPoint inter( this->Bx[d1], this->By[d1] );
             QPoint end( this->Bx[d2], this->By[d2] );
             
             pPart->insPath(end,level+1);
             pPart->insTaken(inter,level);
          }
          
          T inv = ~d1Mask;
          BoardT<T> b(B);
          
          //capture opponent
          b.captured |= d1Mask;
          b.whiteM &= inv;
          b.whiteK &= inv;
          b.blackM &= inv;
          b.blackK &= inv;
          
          //move my own
          T srcMask= 1;
          srcMask <<= srcNr;

          T opponent;
          const bool wTurn=b.isWhiteTurn();
          if( wTurn )
          {
            b.whiteM &= (~srcMask);
            b.whiteM |= d2Mask;
            opponent=( b.blackM | b.blackK );
          }
          else
          {
            b.blackM &= (~srcMask);
            b.blackM |= d2Mask;
            opponent=( b.whiteM | b.whiteK );
          }

          genCapPart( src0, d2 , opponent, level+1, MoveList, b, pList, pPart );
          captured=true;
        }
      }

  return captured;
}


//--------------------------------------------------------------
/** No descriptions */
template <class T, int bSize>
void EngineAmT<T, bSize>::genNKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList=0)
{
  T myKings;
  if( B.isWhiteTurn() )
    myKings=B.whiteK;
  else
    myKings=B.blackK;

  T squares= B.blackK | B.blackM | B.whiteK | B.whiteM;
  
  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myKings;

  while( shift )
  {
    if(myKings & srcMask)
    {
        genManMovesDirection( sqNr, MoveList, squares, this->LU , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RU , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->LD , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RD , pList);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//----------------------------------------------------------------------------------------------
// generates fast 
template <class T, int bSize>
void EngineAmT<T, bSize>::genPossibleMoves( QList<LegalMoveT<T> > &MoveList ,const BoardT<T>* const pB )
{
  Q_ASSERT( pB );

  MoveList.clear();
  
  genCapManMove( MoveList, *pB );
  genCapKingMove( MoveList, *pB );
  
  int howMany=MoveList.count();
  /// @todo remove moves that has the same src,dst and captures

  //captures are mandatory
  if( howMany>0 )
  {
    return ;
  }

  //no captures, gen normal moves
  genNManMove( MoveList, *pB );
  genNKingMove( MoveList, *pB );
 
   howMany=MoveList.count();
}
//----------------------------------------------------------------------------------------------
// LegalMove used in GUI needs intermediate results.
template <class T, int bSize>
void EngineAmT<T, bSize>::genPossibleMoves( QList<LegalMove > &MoveList ,const Board* const pB )
{
  Q_ASSERT( pB );

  BoardT<T> B( pB );
  
  MoveList.clear();
  QList<LegalMoveT<T> > MoveListNotUsed;
  
  genCapManMove( MoveListNotUsed, B, &MoveList );
  genCapKingMove( MoveListNotUsed, B, &MoveList );
  
  int howMany=MoveList.count();
  /// @todo remove moves that has the same src,dst and captures

  //captures are mandatory
  if( howMany>0 )
  {
    return ;
  }

  //no captures, gen normal moves
  genNManMove( MoveListNotUsed, B , &MoveList);
  genNKingMove( MoveListNotUsed, B, &MoveList);
 
   howMany=MoveListNotUsed.count();
}
//---------------------------------------------------------------
/** No descriptions */
template <class T, int bSize>
void EngineAmT<T, bSize>::genCapKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList)const
{
  Q_ASSERT( !B.captured );
  T myKings;

  Q_ASSERT( !B.captured );
  T opponent;
  
  if( B.isWhiteTurn())
  {
    myKings=B.whiteK;
    opponent=( B.blackM | B.blackK );
  }
  else
  {
    myKings=B.blackK;
    opponent=( B.whiteM | B.whiteK );
  }

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myKings;

  while( shift )
  {
    bool candidate=myKings & srcMask;
    if(candidate)
    {
      if( pList )
      {
         LegalMove part;
         QPoint start( this->Bx[sqNr], this->By[sqNr] );
         part.insPath( start,0 );
         genKCapPart(sqNr, sqNr, opponent, 0, MoveList, B, pList, &part);
      }
      else
         genKCapPart(sqNr, sqNr, opponent, 0, MoveList, B, 0, 0);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
  
}

//-----------------------------------------------------------
template <class T, int bSize>
int EngineAmT<T, bSize>::gamePhase(const BoardT<T>* const /* pB */ )const
{
    ///@todo implement that is specyfic to Engish draughts
    return earlyPhase;
}
//------------------------------------------------------------------------
// fill values for all squares suitable for current phase of game
// this is engine specyfic.
// current immplementation fills it completely random
// I am not expert in draughts strategy
// so let somebody else implement
template <class T, int bSize>
void EngineAmT<T, bSize>::reFillSQV( int /*size*/ ,int /*phase*/ )
{

  
  // just for safety
  for(int j=0;j< bSize*bSize/2 ; j++)
  {
     this->SqvWm[j]=0;
     this->SqvBm[j]=0;
     this->SqvK[j]=0;
  }
 

}


//------------------------------------------------------------
template <class T, int bSize>
void EngineAmT<T, bSize>::fillBook( QList<QList<LegalMove> > & )
{
   ///@todo implement simple book
   return;
}
//-------------------------------------------
template <class T, int bSize>
int EngineAmT<T, bSize>::materialScore( const BoardT<T> * const pBoard )const 
{
   int v=0;
   
   //------------ white Man
   T shift=pBoard->whiteM;
   T M=0x01;
   int sqNr=0;
   
   while( shift )
   {
     if( pBoard->whiteM & M )
     {
       v += this->SqvWm[sqNr];
       v += scoreMan;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   //------------ black Man
   shift=pBoard->blackM;
   M=0x01;
   sqNr=0;
   while( shift )
   {
     if( pBoard->blackM & M )
     {
       v -= this->SqvBm[sqNr];// minus because scores are calculated always for white
       v -= scoreMan;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   
   //------------ white Kings
   int WhiteKings=0;
   shift=pBoard->whiteK;
   M=0x01;
   sqNr=0;
   
   while( shift )
   {
     if( pBoard->whiteK & M )
     {
       v += this->SqvK[sqNr];
       if(++WhiteKings == 1)
         v += scoreKing;
       else
         v += scoreNextKing;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   //------------ black Kings
   int BlackKings=0;
   shift=pBoard->blackK;
   M=0x01;
   sqNr=0;
   
   while( shift )
   {
     if( pBoard->blackK & M )
     {
       v -= this->SqvK[sqNr];
       if(++BlackKings == 1)
         v -= scoreKing;
       else
         v -= scoreNextKing;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }

  if(! pBoard->isWhiteTurn() )
    v=-v;

  return v;
}
//----------------------------------------------------------------------------------------------
// only check if opponent has at least one capturing move
template <class T, int bSize>
bool EngineAmT<T, bSize>::canOpponentCapture( const BoardT<T> &B )const
{
  Q_ASSERT( !B.captured );
  T myPieces;
  T opponentMen;
  T opponentKings;
  T squares = B.blackM | B.blackK | B.whiteM | B.whiteK;
  if( B.isWhiteTurn())
  {
    myPieces= ( B.whiteM | B.whiteK);
    opponentMen=B.blackM ;
    opponentKings=B.blackK;
  }
  else
  {
    myPieces= (B.blackM | B.blackK);
    opponentMen=B.whiteM;
    opponentKings=B.whiteK;
  }

  int sqNr=0;// debug
  T M=0x01;
  T shift=opponentMen;

  while( shift )
  {
    bool candidate=opponentMen & M;
    bool ok=false;
    if(candidate)
    {
      ok=checkDirection( sqNr, myPieces, squares, this->LU);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->RU);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->LD);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->RD);
      if(ok) return true;
    }

    M <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
  
  if( opponentKings )
  {
    BoardT<T> newB(B);
    QList<LegalMoveT<T> > MoveList;
    newB.moveNr++; // pretend is opponent move
    
    genCapKingMove( MoveList, newB,0 );
    if( MoveList.count() )
       return true;
  }
  return false;
}
//----------------------------------
template <class T, int bSize>
bool EngineAmT<T, bSize>::checkDirection( int sqNr, const T& opponent, const T& squares, const int * D )const
{
  int d1=D[sqNr];
  int d2=-1;
  if( d1>=0)d2=D[d1];
  if( d1>=0 and d2>=0 )//both on board
  {
    T d1Mask= (T(1) << d1);
    T d2Mask= (T(1) << d2);
    
    if( !(squares & d2Mask) and( d1Mask & opponent ) )
      return true;
  }
  return false;
}

#endif
