/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "config.h"
#include "settingsdialog.h"
#include <QMessageBox>
#include <QtDebug> 

//Settings * Settings::instance=0;



SettingsDialog::SettingsDialog(QWidget *parent): QDialog(parent)
{
	ui.setupUi(this);
	

    updateDialog();
}


SettingsDialog::~SettingsDialog()
{
}


/*!
    \fn Settings::accept()
 */
void SettingsDialog::accept()
{
    hide();
    updateConfig();
    //QMessageBox::information( this, "accept slot","received accept");
}




/*!
    \fn Settings::reject
 */
void SettingsDialog::reject()
{
    //QMessageBox::information( this, "reject slot","received reject");
    hide();
}


/*!
    \fn Settings::updateDialog()
 */
void SettingsDialog::updateDialog()
{
    Conf().read(); // read the settings or set them to the default values

    //  ------  size of the board
    int boardSize=Conf().m_boardSize;
    switch( boardSize )
    {
    case 8:
        ui.cmbBoardSize->setCurrentIndex(0);
        break;
    case 10:
        ui.cmbBoardSize->setCurrentIndex(1);
        break;
    case 12:
        ui.cmbBoardSize->setCurrentIndex(2);
        break;        
    default:
        QMessageBox::critical(this, "error", "board size not supported using 8" );
        Conf().m_boardSize=8;
        ui.cmbBoardSize->setCurrentIndex(0);
        Conf().write();
    }

    
    //  ------  rules
    QString rules=Conf().m_Rules;
    if( rules=="International")
       ui.cmbRules->setCurrentIndex( 0 );
    else if( rules=="American")
       ui.cmbRules->setCurrentIndex( 1 );
    else
    {
       ui.cmbRules->setCurrentIndex( 0 );
       QMessageBox::critical(this, "error", "rules not supported using International" );
    }
}


/*!
    \fn Settings::updateConfig
 */
void SettingsDialog::updateConfig()
{
    int idx=ui.cmbBoardSize->currentIndex();
    int oldSize=Conf().m_boardSize;
    switch( idx )
    {
    case 0:
        Conf().m_boardSize=8;
        if( Conf().m_boardSize!=oldSize ) emit sizeChanged( oldSize );
        break;
    case 1:
        Conf().m_boardSize=10;
        if( Conf().m_boardSize!=oldSize ) emit sizeChanged( oldSize );
        break;
    case 2:
        Conf().m_boardSize=12;
        if( Conf().m_boardSize!=oldSize ) emit sizeChanged( oldSize );
        break;
    default:
        QMessageBox::critical(this, "error", "board size not supported using 8" );
        Conf().m_boardSize=8;
    }
    
    QString oldRules=Conf().m_Rules;
    Conf().m_Rules=ui.cmbRules->currentText();
    
    Conf().write();
    if( oldRules != Conf().m_Rules)
       emit sizeChanged( oldSize );
}

#include "settingsdialog.moc"
