/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "timemanager.h"

TimeManager::TimeManager()
{
    totalTime=0;
    savedTime=0;
    lastElapsed=0;
}


TimeManager::~TimeManager()
{
}


void TimeManager::setReqTime( int req )
{
    //Q_ASSERT( req>0 );
    msecPerMove=req*1000;
}

bool TimeManager::canContinue(int /*level*/ )
{
    int elapsed=time.elapsed();//from start
    if( elapsed>= 2*msecPerMove ) 
        return false;
    
    int currLevelTime=elapsed - lastElapsed;

    float coeff=float(currLevelTime)/float(lastLevelTime);
    if( coeff < 2 )
    { 
        //qDebug("coeff=%f",coeff);
        coeff=2;
    }
    
    int expected = int( (currLevelTime*coeff)+elapsed );

    lastElapsed=elapsed;
    lastLevelTime=currLevelTime;
    
    int canWait=msecPerMove+savedTime;
    if( canWait < msecPerMove/2 )// dont think too fast
        canWait = msecPerMove/2;
        
    //qDebug( "elapsed %d lastElapsed %d lastLevelTime %d currLevelTime %d coeff %f canWait %d",
    //         elapsed, lastElapsed, lastLevelTime, currLevelTime, coeff , canWait);
    
    
    if( expected >=canWait ) return false;
    return true;
}

void TimeManager::start()
{
    time.start();
    lastElapsed=0;
    lastLevelTime=0;
}

void TimeManager::stop()
{
    savedTime += ( msecPerMove - time.elapsed() );
    //qDebug( "savedTime %d", savedTime );
    
    savedTime = qMax( savedTime,-2*msecPerMove );
}

