/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _EXCEPTIONS_H
#define _EXCEPTIONS_H

#include <string>

class Exception {
    private:
        std::string file;
        int line;
        std::string message;
    public:
        Exception(std::string file, int line, std::string message);
        Exception(std::string message);
        virtual ~Exception();
        void printException() const;
    protected:
        virtual const char* getType() const { return "Exception"; }
};

#define DEFINE_EXCEPTION(ExceptionName, SuperClass) class ExceptionName : public SuperClass { \
    public: \
        ExceptionName(std::string file, int line, std::string message) : SuperClass(file, line, message) {} \
        ExceptionName(std::string message) : SuperClass(message) {} \
    protected: \
        virtual const char* getType() const { return #ExceptionName; } \
}
#define DEFINE_MAINEXCEPTION(ExceptionName) DEFINE_EXCEPTION(ExceptionName, Exception)

DEFINE_MAINEXCEPTION(IllegalArgumentException);

class CError : public Exception {
    private:
        int code;
    public:
        CError(std::string file, int line, std::string message, int errcode)
            : Exception(file, line, getDesc(errcode) + message) {}
        CError(std::string message, int errcode)
            : Exception(getDesc(errcode) + message) {}
        static const char* getDesc(int code);
    protected:
        virtual const char* getType() const { return "CError"; }
};

int checkErrorCode(std::string file, int line, int result, std::string message);
#define checkErrorCode(result, message) checkErrorCode(__FILE__, __LINE__, result, message)

#endif // _EXCEPTIONS_H
