/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _GAMEMATH_H
#define _GAMEMATH_H

#include <Eigen/Core>
#include <Eigen/Geometry>

class Plane {
public:
    float a, b, c, d;

    inline Plane() : a(0), b(0), c(0), d(0) {}
    inline Plane(float a, float b, float c, float d) : a(a), b(b), c(c), d(d) {}
    Plane normalize() const;
};

class AABB {
public:
    Eigen::Vector3f leftBottom, rightTop;

    inline AABB() : leftBottom(), rightTop() {}
    inline AABB(const Eigen::Vector3f &lb, const Eigen::Vector3f &rt) : leftBottom(lb), rightTop(rt) {}
    void draw() const;
};

class OBB {
public:
    Eigen::Vector3f points[8];

    OBB(const AABB &bb, const Eigen::Vector3f &pos, const Eigen::Vector3f &scale, const Eigen::Quaternionf &rot);
    OBB(const AABB &bb);
};

class BS {
public:
    Eigen::Vector3f pos;
    float radius;

    inline BS(const Eigen::Vector3f &pos, float radius) : pos(pos), radius(radius) {}
    inline BS() : pos(), radius(0) {}
    void draw() const;
};

class Frustum {
public:
    Plane plane[6];

    Frustum();
    static Frustum calc();
};

float distToPlane(const Plane &plane, const Eigen::Vector3f &pos);
bool bsInFrustum(const Frustum &frustum, const BS &bs);
bool aabbInFrustum(const Frustum &frustum, const AABB &bb);
bool obbInFrustum(const Frustum &frustum, const OBB &obb);
bool pointInFrustum(const Frustum &frustum, const Eigen::Vector3f &point);

#endif // _GAMEMATH_H
