/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "Globals.h"

// FruitWars headers
#include "Camera.h"
#include "Instance.h"
#include "Textures.h"

// KDE headers
#include <KGlobal>
#include <KStandardDirs>

// Qt headers
#include <QTime>

USING_PART_OF_NAMESPACE_EIGEN
using Eigen::Transform3f;
using Eigen::Translation3f;
using Eigen::Scaling3f;

unsigned int nplayers = 8;
bool gl_13_available;
bool pointgame = true;
bool timegame = true;
int maxPoints = 35;
long long int maxTime = 120000;
long long int theTime = 0;
bool gameEnded = false;
bool showText = true;
bool sessionLost = false;
bool gameQuit = false;
int thaPerm[8];
Transform3f fruitAttachMat;
int hitEffect = 0;
float playerStunTime = 4.0;
int diversityBonus = 5;
bool oneFruitType = false;
FruitType theFruitType = ftStraw;
int minFruit = 2;
int maxFruit = 8;
float minWait = 10.0;
float maxWait = 60.0;
int teamMode = 2;
int nTeams;
int nBaskets;
float spawnRadius = 60.0;
bool spawnRandom = false;
bool friendlyFire = false;
int fruitAppleScore = 4;
int fruitOrangeScore = 3;
int fruitLemonScore = 1;
int fruitAnanasScore = 2;
int fruitStrawScore = 5;
int screenWidth = 800;
int screenHeight = 600;
float mouseSpeed = 0.75 * M_PI / 180;
bool fullScreen = false;
int extraHitPoints = 1;
int syncRate = 80;
float throwInfluence = 0.0;
int holdMode = 0;
bool drunkMode = true;
int drunkSize = 256;
bool nprMode = false;
QGLContext* glcontext = 0;
QTime* globaltime = 0;

void basicSetup()
{
    glClearColor(0.0, 0.0, 0.0, 1.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glShadeModel(GL_SMOOTH);
    glFrontFace(GL_CCW);
}

void lightSetup()
{
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_RESCALE_NORMAL);
}

void varInit()
{
    halfWidth = screenWidth / 2;
    halfHeight = screenHeight / 2;
    // setCursorPos XXX
    mouseX = -1;
    mouseY = -1;
    moveAmount = 0.0;
    pastTime = 0;
    frameCount = 0;
    oldTime = time();
    syncTime = time();
    for (int i = 0; i < 512; i++) {
        keyState[i] = false;
        dKeyState[i] = false;
        dLastTime[i] = time();
    }

    // Set Colors
    myPlayerColor[0] = Vector3f(1.0, 0.0, 0.0);
    myPlayerColor[1] = Vector3f(0.0, 0.0, 1.0);
    myPlayerColor[2] = Vector3f(1.0, 1.0, 0.0);
    myPlayerColor[3] = Vector3f(0.6, 0.0, 1.0);
    myPlayerColor[4] = Vector3f(0.0, 0.75, 0.0);
    myPlayerColor[5] = Vector3f(1.0, 0.5, 0.0);
    myPlayerColor[6] = Vector3f(0.0, 0.95, 1.0);
    myPlayerColor[7] = Vector3f(0.35, 0.13, 0.03);

    // Create Fruit attach matrix
    Matrix4f m;
    m << FRUITATTACH[0], FRUITATTACH[1], FRUITATTACH[2], FRUITATTACH[3],
                FRUITATTACH[4], FRUITATTACH[5], FRUITATTACH[6], FRUITATTACH[7],
                FRUITATTACH[8], FRUITATTACH[9], FRUITATTACH[10], FRUITATTACH[11],
                FRUITATTACH[12], FRUITATTACH[13], FRUITATTACH[14], FRUITATTACH[15];
    fruitAttachMat = m;
    fruitAttachMat *= Translation3f(0.0, 0.0, 64.0);
    fruitAttachMat *= Scaling3f(100.0, 100.0, 100.0);

    nTeams = 2 << teamMode;
    nBaskets = nTeams;
    
    curTex = 0;
    if (screenHeight >= 512) drunkSize = 512;
    createEmptyTexture(emptyTex);

    nprMode = nprMode && gl_13_available;
}

void camInit()
{
    myCam = ThirdPersonCamera(Vector3f(5.0, 5.0, 20.0), SEAHEIGHT, &myTer);
    // if spectator XXX, dist = 0
    myCam.setDist(8.0);
    myCam.setSmooth(0.80);
    myCam.setAngSmooth(0.75);
}

void envInit()
{
    // Island
    myTer = Terrain(64, 64, 128.0, 128.0, 0.0, 17.50);
    myTer.loadHeightMap(KGlobal::dirs()->findResource("appdata", "data/hmapisle5.bmp"));
    if (gl_13_available) {
        myTer.setTexMethod(DetailTex);
        myTer.loadTex(0, KGlobal::dirs()->findResource("appdata", "data/grass.bmp"));
        myTer.loadTex(1, KGlobal::dirs()->findResource("appdata", "data/landscapetrial5.bmp"));
        myTer.setTexScale(0, Vector2f(16.0, 16.0));
    } else {
        myTer.setTexMethod(ColorTex);
        myTer.loadColorMap(KGlobal::dirs()->findResource("appdata", "data/landscapetrial5.bmp"));
        myTer.loadTex(0, KGlobal::dirs()->findResource("appdata", "data/grass.bmp"));
        myTer.setTexScale(0, Vector2f(16.0, 16.0));
    }
    myTer.position(Vector3f(-64.0, 0.0, 64.0));
    myTer.setPatchSize(4);
    myTer.updateVertBuf();

    // Island finish
    Color bruin1 = Color( 256.0 / 255.0, 210.0 / 255.0, 34.0 / 255.0, 1.0 );
    Color bruin2 = Color( 157.0 / 255.0, 177.0 / 255.0, 21.0 / 255.0, 1.0 );
    myTerFinish = TerrainFinish(&myTer, 0.0, 2, 4, 5, 5, bruin1, bruin2);
    myTerFinish.setEdgeColor(KGlobal::dirs()->findResource("appdata", "data/landscapetrial5.bmp"));

    // Sea
    mySea = Terrain(2, 2, 128.0, 128.0, SEAHEIGHT, SEAHEIGHT);
    mySea.position(Vector3f(-64.0, 0.0, 64.0));
    mySea.setPatchSize(8);
    mySea.calcSeaTex(0, &myTer, 256, 6.0);
    mySea.updateVertBuf();

    // Sea finish
    Color sea1 = Color( 0.368, 0.60, 0.88, 0.65 );
    Color sea2 = Color( 0.293, 0.50, 0.81, 0.60 );
    mySeaFinish = TerrainFinish(&mySea, SEAHEIGHT, 4, 3, 1, 1, sea1, sea2);
    mySeaFinish.calcEdgeSea(&myTer, 6.0);
    sea1= Color( 0.22, 0.28, 0.43, 1.00 );
    mySeaFinish.setOuterColor(1, sea1);
    sea1 = Color( 0.13, 0.20, 0.43, 0.98 );
    mySeaFinish.setOuterColor(2, sea1);
    sea1 = Color( 0.24, 0.30, 0.45, 1.00 );
    mySeaFinish.setCornerColor(1, sea1);

    // Sky
    mySky = SkyCube(256.0);
    mySky.loadTex(SKY_TOP, KGlobal::dirs()->findResource("appdata", "data/skybox0005.bmp"));
    mySky.loadTex(SKY_FRONT, KGlobal::dirs()->findResource("appdata", "data/skybox0001.bmp"));
    mySky.loadTex(SKY_BACK, KGlobal::dirs()->findResource("appdata", "data/skybox0003.bmp"));
    mySky.loadTex(SKY_LEFT, KGlobal::dirs()->findResource("appdata", "data/skybox0004.bmp"));
    mySky.loadTex(SKY_RIGHT, KGlobal::dirs()->findResource("appdata", "data/skybox0002.bmp"));
    mySky.position(Vector3f(0, 8.5, 0));
}

void modelLoading()
{
    int invPerm[8];
    for (int i = 0; i < 8; i++) {
        invPerm[thaPerm[i]] = i;
    }

    // Create special textures
    createWarpTex();
    if (nprMode) {
        createCartoonTex();
        createCartoonShadowTex();
    } else {
        createShadowTex();
    }
    createAimTex();
    createStarTex();
    createStarShowTex();
    createTrailTex();
    createIndicTex();

    // Load teams
    if (teamMode < 2) {
        myTeams.resize(nTeams);
        playerInTeam.resize(nplayers);
        for (unsigned int i = 0; i < nplayers; i++) {
            playerInTeam[i] = i % nTeams;
        }
        for (int i = 0; i < nTeams; i++) {
            myTeams[i] = Team(i);
            for (unsigned int j = 0; j < nplayers; j++) {
                if (playerInTeam[j] == i) myTeams[i].addPlayer(j);
            }
            myTeams[i].setBaskId(i);
            myTeams[i].setColorId((4 >> teamMode) * i);
        }
    }

    // Load players
    playerModel = loadModel(KGlobal::dirs()->findResource("appdata", "data/KOS2.ogl"));
    bananaModel = loadModel(KGlobal::dirs()->findResource("appdata", "data/banaan.ogl"));
    myPlayers.resize(nplayers);
    myPlayerTex.resize(nplayers);
//	int lastId = -1;
    bool idused[8];
    for (unsigned int i = 0; i < nplayers; i++) idused[i] = false;
    for (unsigned int i = 0; i < nplayers; i++) {
        myPlayers[i] = HumanPlayer(playerModel, i + 1);
        myPlayers[i].scale(Vector3f(PLAYERSCALE, PLAYERSCALE, PLAYERSCALE));
        myPlayers[i].position(Vector3f(i * 3.0, myTer.getGroundHeight(Vector2f(0.0, 0.0)), 0.0));
        myPlayers[i].setIsAI(true);
        myPlayers[i].setColorId(thaPerm[i]);
        if (teamMode == 2) {
            myPlayers[i].setBaskId(thaPerm[i]);
        } else {
			myPlayers[i].setTeamId(playerInTeam[i]);
			myPlayers[i].setBaskId(myTeams[myPlayers[i].getTeamId()].getBaskId());
			myPlayers[i].setRelTeamId(myTeams[myPlayers[i].getTeamId()].getRelTeamId(i));
        }
        myPlayers[i].getBanana().getIt();
        loadTexture(KGlobal::dirs()->findResource("appdata", QString("data/kostex%1.bmp").arg(thaPerm[i]+1)), myPlayerTex[i], glcontext);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
        myPlayers[i].getInst()->setTexture(0, myPlayerTex[i]);
    }

    // Load Fruit
    fruitSpawner = FruitSpawner(minFruit, maxFruit, minWait, maxWait);

    // Load Fruitbasket
    basketModel = loadModel(KGlobal::dirs()->findResource("appdata", "data/basket.ogl"));
    myBasketTex.resize(nBaskets);
    for (int i = 0; i < nBaskets; i++) {
        myBaskets.push_back(FruitBasket(basketModel));
        myBaskets[i].scale(Vector3f(FRUITBASKETSCALE, FRUITBASKETSCALE, FRUITBASKETSCALE));
        if (teamMode == 2) {
            Vector2f pos(sin(i * M_PI * 0.25 + BASKETROTSHIFT) * BASKETPOSRADIUS, cos(i * M_PI * 0.25 + BASKETROTSHIFT) * BASKETPOSRADIUS);
            myBaskets[i].position(Vector3f(pos[0],
                            BASKETHEIGHTSHIFT[i] + myTer.getGroundHeight(pos) + FRUITBASKETHEIGHT * FRUITBASKETSCALE,
                            pos[1]));
        } else {
            Vector2f pos(sin((4 >> teamMode) * i * M_PI * 0.25 + BASKETROTSHIFT) * BASKETPOSRADIUS,
                         cos((4 >> teamMode) * i * M_PI * 0.25 + BASKETROTSHIFT) * BASKETPOSRADIUS);
            myBaskets[i].position(Vector3f(pos[0],
                            BASKETHEIGHTSHIFT[(4 >> teamMode) * i] + myTer.getGroundHeight(pos) + FRUITBASKETHEIGHT * FRUITBASKETSCALE,
                            pos[1]));
        }
        myBaskets[i].setPlayId(invPerm[i]);
        if (teamMode < 2) {
            myBaskets[i].setTeamId(i);
        }
        if (teamMode == 2) {
            loadTexture(KGlobal::dirs()->findResource("appdata", QString("data/basket%1.bmp").arg(i+1)), myBasketTex[i], glcontext);
        } else {
            loadTexture(KGlobal::dirs()->findResource("appdata", QString("data/basket%1.bmp").arg((4 >> teamMode) * i + 1)), myBasketTex[i], glcontext);
        }
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
        myBaskets[i].getInst()->setTexture(0, myBasketTex[i]);
    }

    // Crosshair
    theCrossHair = Aim(0);

    // Indicator
    theIndicator = Indicator(0.40, 0, myPlayers[0].getBaskId());

    // hud rect
    loadTexture(KGlobal::dirs()->findResource("appdata", "data/hudrect.tga"), HUDRectTex, glcontext);

    // Respawn all players
    for (unsigned int i = 0; i < nplayers; i++) {
        myPlayers[i].respawn();
    }
}

void createEmptyTexture(GLuint& tex)
{
    GLuint text;
    glGenTextures(1, &text);
    glBindTexture(GL_TEXTURE_2D, text);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    void* data = malloc(drunkSize * drunkSize * 3);
    memset(data, 0, drunkSize * drunkSize * 3);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, drunkSize, drunkSize, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
    tex = text;
    free(data);
}

ThirdPersonCamera myCam;
bool keyState[514];
bool dKeyState[512];
bool dLastTime[512];
int mouseX, mouseY;
float moveAmount;
int halfWidth, halfHeight;
GLuint HUDRectTex;
float glblRotate;
Terrain myTer, mySea;
TerrainFinish myTerFinish, mySeaFinish;
SkyCube mySky;
float lightPos[4] = {5.0, 50.0, 12.0, 1.0};
float lightDir[4] = {-0.49, 0.81, -0.32, 0.0};
float ambient[4] = {0.05, 0.05, 0.05, 1.0};
float diffuse[4] = {0.9, 0.9, 0.9, 1.0};
int frameCount, pastTime;
unsigned int oldTime, syncTime;
QString fpsString;
Model3D* playerModel;
Array<HumanPlayer> myPlayers;
Array<GLuint> myPlayerTex;
Vector3f myPlayerColor[8];
FruitSpawner fruitSpawner;
Model3D* basketModel;
std::vector<FruitBasket> myBaskets;
std::vector<GLuint> myBasketTex;
Model3D* bananaModel;
Aim theCrossHair;
std::vector<Team> myTeams;
std::vector<int> playerInTeam;
Indicator theIndicator;
GLuint emptyTex;
GLuint curTex;
bool drunkActive;

unsigned int time()
{
    return globaltime->elapsed();
}

float frandom()
{
    double d = random();
    d /= 2147483647.0;
//	printf("frandom() = %.5f\n", (float)d);
    return d;
}
