/*
    gamefuemulatorconfig.cpp - Emulator configuration

    Copyright (c) 2005      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefuemulatorconfig.h"

// Qt includes
#include <qlayout.h>
#include <qlabel.h>
#include <qvaluelist.h>

// KDE includes
#include <kdebug.h>
#include <kgenericfactory.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kglobal.h>

// Gamefu includes
#include <systemmanager.h>
#include <system.h>

// Own includes
#include "emulatorconfigwidget.h"

class GamefuEmulatorConfig::Private
{
public:
	QValueList<EmulatorConfigWidget*> emulatorConfigList;
};

typedef KGenericFactory<GamefuEmulatorConfig, QWidget> GamefuEmulatorConfigFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_gamefu_emulatorconfig, GamefuEmulatorConfigFactory( "kcm_gamefu_emulatorconfig" ) )

GamefuEmulatorConfig::GamefuEmulatorConfig(QWidget* parent, const char* /*name*/, const QStringList& args)
	: KCModule( GamefuEmulatorConfigFactory::instance(), parent, args ), d( new Private() )
{
// 	( new QVBoxLayout( this ) )->setAutoAdd( true );
	QVBoxLayout *mainLayout = new QVBoxLayout(this);

	// Iterator over all loaded system and create a EmulatorConfigWidget
	Gamefu::SystemList systemList = Gamefu::SystemManager::self()->loadedSystems();
	Gamefu::SystemList::const_iterator systemIt;
	Gamefu::SystemList::const_iterator systemItEnd = systemList.end();
	for(systemIt = systemList.begin(); systemIt != systemItEnd; ++systemIt)
	{
		Gamefu::System *system = *systemIt;
		if(system)
		{
			EmulatorConfigWidget *emuConfig = new EmulatorConfigWidget(system->systemName(), this);
			// Dispatch settings changed.
			connect(emuConfig, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
			d->emulatorConfigList.append(emuConfig);
			mainLayout->addWidget(emuConfig);
		}
	}

	// Add a spacer item
	mainLayout->addItem( new QSpacerItem( 20, 40, QSizePolicy::Expanding, QSizePolicy::Minimum ) );

	load();
}

GamefuEmulatorConfig::~GamefuEmulatorConfig()
{
	delete d;
}

void GamefuEmulatorConfig::load()
{
	QValueList<EmulatorConfigWidget*>::iterator configIt;
	QValueList<EmulatorConfigWidget*>::iterator configItEnd = d->emulatorConfigList.end();
	for(configIt = d->emulatorConfigList.begin(); configIt != configItEnd; ++configIt)
	{
		((EmulatorConfigWidget*)*configIt)->loadSettings();
	}
	KCModule::load();
}

void GamefuEmulatorConfig::save()
{
	QValueList<EmulatorConfigWidget*>::iterator configIt;
	QValueList<EmulatorConfigWidget*>::iterator configItEnd = d->emulatorConfigList.end();
	for(configIt = d->emulatorConfigList.begin(); configIt != configItEnd; ++configIt)
	{
		((EmulatorConfigWidget*)*configIt)->saveSettings();
	}
	KCModule::save();
}

void GamefuEmulatorConfig::settingsChanged()
{
	emit changed(true);
}

#include "gamefuemulatorconfig.moc"
