/*
    gamefuconfigplugins.cpp - Plugins configuration

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefuconfigplugins.h"

// Qt includes
#include <qlayout.h>

// KDE includes
#include <kgenericfactory.h>
#include <kpluginselector.h>
#include <kplugininfo.h>
#include <kconfig.h>

// Gamefu includes
#include <gamefupluginmanager.h>

typedef KGenericFactory<GamefuConfigPlugins, QWidget> GamefuConfigPluginsFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_gamefu_configplugins, GamefuConfigPluginsFactory( "kcm_gamefu_configplugins" ) )

class GamefuConfigPlugins::Private
{
public:
	Private()
	 : pluginSelector(0)
	{}

	KPluginSelector *pluginSelector;
};

GamefuConfigPlugins::GamefuConfigPlugins(QWidget *parent, const char *name, const QStringList &args)
 : KCModule(parent, name, args), d(new Private)
{
	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	d->pluginSelector = new KPluginSelector(this);
	
	mainLayout->addWidget(d->pluginSelector);

	connect(d->pluginSelector, SIGNAL(changed(bool)), this, SIGNAL(changed(bool)));

	d->pluginSelector->addPlugins( Gamefu::PluginManager::self()->availablePlugins( QString::fromUtf8("Plugins") ), i18n("Generic Plugins"), QString::fromUtf8("Plugins") );
}

GamefuConfigPlugins::~GamefuConfigPlugins()
{
	delete d;
}

void GamefuConfigPlugins::save()
{
	d->pluginSelector->save();
	// Load/unload changed plugins.
	Gamefu::PluginManager::self()->load();
}

void GamefuConfigPlugins::load()
{
	d->pluginSelector->load();
}

#include "gamefuconfigplugins.moc"
