/*
    Gamefu, KDE All-Machine Emulator Frontend for UNIX
    Also a collection manager in its spare time ;)

    Copyright (c) 2005      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefuapplication.h"

#include <qstringlist.h>

// KDE includes
#include <kdebug.h>

// Gamefu includes
#include "gamefumainwindow.h"
#include <systemmanager.h>
#include <gamefupluginmanager.h>
#include <collectionaccess.h>
#include <threadweaver.h>

GamefuApplication::GamefuApplication()
  : KUniqueApplication(true, true, true)
{	
	Gamefu::PluginManager::self(); // Look for available plugins
	// SystemManager must be created after PluginManager because it listen to a signal of PluginManager
	Gamefu::SystemManager::self();

	mainWin = new GamefuMainWindow();

	Gamefu::PluginManager::self()->load();

	connect(Gamefu::PluginManager::self(), SIGNAL(loadingFinished()), this, SLOT(slotAllSystemsLoaded()));	
}

GamefuApplication::~GamefuApplication()
{
	ThreadWeaver::deleteInstance();
	kdDebug() << endl;
}

void GamefuApplication::slotAllSystemsLoaded()
{
	QStringList mimeList = Gamefu::SystemManager::self()->getRegistredMimeTypes();
	QStringList::const_iterator it;
	for(it = mimeList.begin(); it != mimeList.end(); ++it)
	{
		kdDebug() << "MIME type: " << *it << endl;
	}
	kdDebug() << "All systems loaded !" << endl;
}

int GamefuApplication::newInstance()
{
	setMainWidget(mainWin);
	return KUniqueApplication::newInstance();
}

#include "gamefuapplication.moc"
