/*
    gamefumainwindow.h - Gamefu Application Main Window

    Copyright (c) 2005-2006 by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef __GAMEFU_H__
#define __GAMEFU_H__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kmainwindow.h>

namespace Gamefu
{
	class Plugin;
}
/**
 * @short Application Main Window
 * @author Michaël Larouche <shock@shockdev.ca.tc>
 */
class GamefuMainWindow : public KMainWindow
{
    Q_OBJECT
public:
	/**
	 * Default Constructor
	 */
	GamefuMainWindow();

	/**
	 * Default Destructor
	 */
	virtual ~GamefuMainWindow();

private slots:
	/**
	 * This function is called at loading, checking if 
	 * it's the first time Gamefu is launched.
	 * Launch the first time wizard if needed.
	 */ 
	void checkFirstTime();
	/**
	 * Show the first time wizard.
	 */
	void showFirstTimeWizard();
	/**
	 * Called when wizard is accepted. Build the initial collection.
	 */
	void wizardAccepted();
	/**
	 * Fill the view with all the ROM in the collection.
	 */
	void fillRomList();
	/**
	 * Build the initial collection.
	 */
	void buildCollection();

	/**
	 * Error handler for database errors.
	 * @param errorMessage Error message to display.
	 */ 
	void displayError(const QString &errorMessage);
	/**
	 * Execute the required emulator from ROM url.
	 */
	void executeRom(const QString &romUrl);

	/**
	 * Open a ROM without adding it to the "play list" and the collection.
	 */
	void fileOpen();
	/**
	 * Show the preferences(configuration) dialog.
	 */ 
	void showPreferences();

	/**
	 * Read the size settings
	 */
	void readSettings();
	/**
	 * Save the size settings
	 */
	void saveSettings();

	/**
	 * @brief Add plugin GUI item to main window
	 *
	 * Called when PluginManager has loaded a plugin.
	 * @param plugin Loaded plugin
	 */
	void slotPluginLoaded(Gamefu::Plugin *plugin);

	/**
	 * @brief Remove plugin GUI item from main window.
	 *
	 * Called when PluginManager will unload a plugin.
	 * @param plugin Unloaded plugin.
	 */
	void slotPluginUnloaded(Gamefu::Plugin *plugin);

private:
	/**
	 * Create and setup all actions.
	 */
	void setupActions();

	/**
	 * Setup Qt signals/slots connections.
	 */
	void setupConnections();

	/**
	 * When exiting the application, do cleanup.
	 */	
	virtual bool queryExit();

private:
	class Private;
	Private *d;
	
};

#endif // _KEMUL_H_
