#include "statusbarnotify.h"

// Qt includes
#include <qguardedptr.h>

#include "gamefustatusbar.h"
#include "notifymanager.h"

class StatusBarNotify::Private
{
public:
	Private()
	{}

	QGuardedPtr<GamefuStatusBar> statusBar;
};

StatusBarNotify::StatusBarNotify(GamefuStatusBar *statusBar, QObject *parent, const char *name)
 : Gamefu::AbstractNotify(parent, name), d(new Private)
{
	d->statusBar = statusBar;
}

StatusBarNotify::~StatusBarNotify()
{
	delete d;
}

void StatusBarNotify::createNewProgressOperation(KIO::Job* job, const QString& description, int totalSteps)
{
	KDE::ProgressBar &progressBar = d->statusBar->newProgressOperation(job);
	progressBar.setDescription(description);
	progressBar.setTotalSteps(totalSteps);
}

void StatusBarNotify::createNewProgressOperation(QObject* owner, const QString& description, int totalSteps, QObject* receiver, const char* abortSlot)
{
	KDE::ProgressBar &progressBar = d->statusBar->newProgressOperation(owner);
	progressBar.setDescription(description);
	progressBar.setTotalSteps(totalSteps);
	progressBar.setAbortSlot(receiver, abortSlot);
}

void StatusBarNotify::incrementProgress(QObject* owner)
{
	d->statusBar->incrementProgress(owner);
}

void StatusBarNotify::longMessage(const QString& message, int type)
{
	d->statusBar->longMessage(message, type);
}

void StatusBarNotify::setProgress(QObject* owner, int steps)
{
	d->statusBar->setProgress(owner, steps);
}

void StatusBarNotify::setProgressStatus(QObject* owner, const QString& status)
{
	d->statusBar->setProgressStatus(owner, status);
}

void StatusBarNotify::shortLongMessage(const QString& shortMessage, const QString& longMessage, int type)
{
	d->statusBar->shortLongMessage(shortMessage, longMessage, type);
}

void StatusBarNotify::shortMessage(const QString &shortMessage, bool longDelay)
{
	d->statusBar->shortMessage(shortMessage, longDelay);
}

#include "statusbarnotify.moc"
