/*
    kfile_genesis.cpp - Sega Genesis/Megadrive Rom image KFile Plugin

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

     *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "kfile_genesis.h"

#include <kgenericfactory.h>
#include <kdebug.h>
#include <kmdcodec.h>

#include <qfile.h>
#include <qdatastream.h>

#include <nintendomakerlist.h>

typedef KGenericFactory<KGenesisFilePlugin> kgenesisfileFactory;

K_EXPORT_COMPONENT_FACTORY(kfile_genesis, kgenesisfileFactory( "kfile_genesis" ))

KGenesisFilePlugin::KGenesisFilePlugin(QObject *parent, const char *name, const QStringList& args)
	: KFilePlugin(parent, name, args)
{
	//add the mimetype here:
	KFileMimeTypeInfo *info = addMimeTypeInfo( "application/x-rom-genesis" );

	KFileMimeTypeInfo::GroupInfo *groupGeneral = 0L;
	groupGeneral = addGroupInfo(info, "romGeneralInfo", i18n("General"));
	
	// General Group
// 	addItemInfo(groupGeneral, "internalName", i18n("Internal Name"), QVariant::String);
// 	addItemInfo(groupGeneral, "country", i18n("Country"), QVariant::String);
// 	addItemInfo(groupGeneral, "editor", i18n("Developer"), QVariant::String);
	addItemInfo(groupGeneral, "md5Hash", i18n("MD5 Hash"), QVariant::String);
}

bool KGenesisFilePlugin::readInfo( KFileMetaInfo& info, uint /*what*/)
{
	// TODO: Check if it's really a Genesis ROM file (.bin could not be a Genesis ROM file)
	if( info.path().isEmpty() )
		return false;

	KFileMetaInfoGroup groupGeneral = appendGroup(info, "romGeneralInfo");

	QString md5Hash;

	QFile genesis_file( info.path() );

	if( genesis_file.open(IO_ReadOnly) )
	{
		// Generate the MD5 Hash
		KMD5 context(0L);
		context.update(genesis_file);
		md5Hash = context.hexDigest();

		genesis_file.close();
	}

	// General group
// 	appendItem(groupGeneral, "internalName", QString(internalName));
// 	appendItem(groupGeneral, "country", countryName);
// 	appendItem(groupGeneral, "editor", editorName);
	appendItem(groupGeneral, "md5Hash", md5Hash);

	return true;
}

#include "kfile_genesis.moc"
