/*
    gamefuplugin.h - Base class for plugins.

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#ifndef GAMEFUPLUGIN_H
#define GAMEFUPLUGIN_H

// Qt includes
#include <qobject.h>

// KDE includes
#include <kxmlguiclient.h>
#include <kdemacros.h>
#include <kinstance.h>

class KPluginInfo;

namespace Gamefu
{

class KDE_EXPORT Plugin : public QObject, public KXMLGUIClient
{
	Q_OBJECT
public:
	Plugin(KInstance *instance, QObject *object, const char *name);
	virtual ~Plugin();

	/**
	 * @brief Set the plugin info associated with that system.
 	 *
	 * Called by Gamefu::PluginManager.
	 * @param info plugin info
	 */
	void setPluginInfo( KPluginInfo *info );
	
	/**
	 * @brief Get the user visibile name of the plugin
	 * @return the plugin name.
	 */
	QString displayName() const;

protected:
	/**
	 * @brief Get acces to plugin info.
	 * 
	 * Plugins can use this method to get additional information
	 * from their desktop file.
	 *
	 * @return Current plugin info. 
	 */
	KPluginInfo *pluginInfo();
	KPluginInfo *pluginInfo() const;

private:
	class Private;
	Private *d;
};

}

#endif
