/*
    notifymanager.cpp - Gamefu Global Notify manager.

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "notifymanager.h"

// Qt includes
#include <qguardedptr.h>

// KDE includes
#include <kstaticdeleter.h>
#include <kdebug.h>

#include "abstractnotify.h"

namespace Gamefu 
{
static KStaticDeleter<NotifyManager> notifyManagerDeleter;

NotifyManager *NotifyManager::s_self = 0;
NotifyManager *NotifyManager::self()
{
	if( !s_self )
	{
		notifyManagerDeleter.setObject(s_self, new NotifyManager);
	}
	
	return s_self;
}

class NotifyManager::Private
{
public:
	Private()
	{}

	QGuardedPtr<AbstractNotify> notifyBackend;
};

NotifyManager::NotifyManager()
 : QObject(0, "GamefuNotifyManager"), d(new Private)
{}


NotifyManager::~NotifyManager()
{
	kdDebug() << endl;
	delete d;
}

void NotifyManager::setNotifyBackend(AbstractNotify *backend)
{
	d->notifyBackend = backend;
}

void NotifyManager::longMessage(const QString &message, int type)
{
	d->notifyBackend->longMessage(message, type);
}

void NotifyManager::shortLongMessage(const QString &shortMessage, const QString &longMessage, int type)
{
	d->notifyBackend->shortLongMessage(shortMessage, longMessage, type);	
}

void NotifyManager::shortMessage(const QString &shortMessage, bool longDelay)
{
	d->notifyBackend->shortMessage(shortMessage, longDelay);
}

void NotifyManager::createNewProgressOperation(QObject *owner, const QString &description, int totalSteps, QObject *receiver, const char *abortSlot)
{
	d->notifyBackend->createNewProgressOperation(owner, description, totalSteps, receiver, abortSlot);
}

void NotifyManager::createNewProgressOperation(KIO::Job *job, const QString &description, int totalSteps)
{
	d->notifyBackend->createNewProgressOperation(job, description, totalSteps);
}

void NotifyManager::incrementProgress(QObject *owner)
{
	d->notifyBackend->incrementProgress(owner);
}

void NotifyManager::setProgress(QObject *owner, int steps)
{
	d->notifyBackend->setProgress(owner, steps);
}

void NotifyManager::setProgressStatus(QObject *owner, const QString &status)
{
	d->notifyBackend->setProgressStatus(owner, status);
}

}
#include "notifymanager.moc"
