/*
    imagefillerdialog.cpp - Image Filler plugin.

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "imagefillerdialog.h"

// Qt includes
#include <qradiobutton.h>

// KDE includes
#include <kdebug.h>
#include <klocale.h>
#include <kurlrequester.h>

// Local includes
#include "imagefillerwidget.h"
#include "imagetypeenum.h"
#include "imagefillerstrategy.h"
#include "imagefillerjob.h"

class ImageFillerDialog::Private
{
public:
	Private()
	 : mainWidget(0)
	{}

	ImageFillerWidget *mainWidget;
};

ImageFillerDialog::ImageFillerDialog(QWidget *parent)
 : KDialogBase(parent, "ImageFillerDialog", false, i18n("Image Filler plugin"), KDialogBase::Apply | KDialogBase::Close, KDialogBase::Apply ), d(new Private)
{
	// Create main widget
	d->mainWidget = new ImageFillerWidget(this);
	setMainWidget( d->mainWidget );

	// Show directory select dialog
	d->mainWidget->directorySelector->setMode( KFile::Directory );

	// Delete dialog on close
	connect(this, SIGNAL(finished()), this, SLOT(delayedDestruct()));
}


ImageFillerDialog::~ImageFillerDialog()
{
	kdDebug() << endl;
	delete d;
}

void ImageFillerDialog::slotApply()
{
	ImageFillerJob *imageJob = new ImageFillerJob(this);

	// Get image type
	ImageEnum::ImageType imageType;
	if( d->mainWidget->radioTitlescreen->isChecked() )
		imageType = ImageEnum::Titlescreen;
	else if( d->mainWidget->radioScreenshot->isChecked() )
		imageType = ImageEnum::Screenshot;

	// Get image strategy
	ImageFillerStrategy *imageStrategy = 0;
	if( d->mainWidget->radioImageFilename->isChecked() )
		imageStrategy = new FilenameImageFillerStrategy;
	else if( d->mainWidget->radioImageRelease->isChecked() )
		imageStrategy = new ReleaseImageFillerStrategy;

	// Setup job
	imageJob->setImageDirectory( KURL(d->mainWidget->directorySelector->url()) );
	imageJob->setImageType( imageType );
	imageJob->setImageFillerStrategy( imageStrategy );

	// Start job
	ThreadWeaver::instance()->queueJob( imageJob );

	KDialogBase::slotApply();
}

#include "imagefillerdialog.moc"
