

#ifndef _HIQVIEW_H_
#define _HIQVIEW_H_

#include <qwidget.h>
#include <kparts/part.h>
#include "peg.h"
#include <qptrstack.h>
#include "historypart.h"
#include <kapplication.h>

class QPainter;
class KURL;

/**
 * This is the main view class for HiQ.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * This hiq uses an HTML component as an example.
 *
 * @short Main view
 * @author Samuel Hoffstaetter <sfh23@cornell.edu>
 * @version 0.1
 */

extern const QString defaultStyle;

class HiQView : public QCanvasView {
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    HiQView(QCanvas *canv, QWidget *parent, const char * name, bool anim,
            QString &styleName, KApplication*, int, int);

    QString& toString();
    static QString& boardToString(bool b[]);

    /**
     * Destructor
     */
    virtual ~HiQView();

    HistoryPtr* getHistory();

    virtual void contentsMousePressEvent( QMouseEvent* e );
    virtual void contentsMouseMoveEvent ( QMouseEvent *e );
    virtual void contentsMouseReleaseEvent ( QMouseEvent * e );


    /**
     * Print this view to any medium -- paper or not
     */
    void print(QPainter *, int height, int width);

signals:
    /**
     * Use this signal to change the content of the statusbar
     */
    void signalChangeStatusbar(const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
    void signalChangeCaption(const QString& text);

    // emit undoIsLegal(false) means undoing is illegal
    void undoIsLegal(bool);
    void redoIsLegal(bool);
    
public slots:
    bool undo();
    bool redo();
    void reset();
    void setAnimated(bool yes);
    void setEditable(bool yes);
    void setStyle(QString style);
    void setUpdateDelay(int, int);


private:
    void loadStyle();
    void createInitialBoard();
    Peg *pegs[33];
    QCanvasPixmapArray *img;
    QCanvasPixmapArray *mov;
    QCanvas *canvas;
    int mousePressX;
    int mousePressY;
    int mousePressNum;
    bool animated;
    bool editable;
    bool initialBoard[33];
    HistoryPtr *history;
    HistoryPtr *redoHistory;
    QString imageDir;
    QCanvasSprite * bgSprite;
    QString styleName;
    int pegHeight;
    int pegWidth;
    int findNumFrames();
    int advancePeriod;
    int advancePeriodFast;
public:
    bool enableMouse;
    QString style() {
        return styleName;
    }
};
#endif // _HIQVIEW_H_
