

#include "pref.h"

#include <klocale.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qslider.h>


HiQPreferences::HiQPreferences(bool anim, int animUpdateDelay, int noAnimUpdateDelay, QString style)
: KDialogBase(TreeList, i18n("HiQ Preferences"),
              Help|Default|Ok|Cancel, Ok) {
    // this is the base class for your preferences dialog.  it is now
    // a Treelist dialog.. but there are a number of other
    // possibilities (including Tab, Swallow, and just Plain)


    // adding page "General options"
    QFrame *frame = addPage(i18n("General"), i18n("General options"),
                            KGlobal::iconLoader()->loadIcon("kfm",KIcon::Panel,0,false) );
    QVBoxLayout *frameLayout = new QVBoxLayout( frame, 0, 0 );
    m_prefGeneral = new HiQGenPref(frame, anim, animUpdateDelay, noAnimUpdateDelay);
    frameLayout->addWidget(m_prefGeneral);

    QFrame *frame2 = addPage(i18n("Theme"), i18n("Theme options"),
                             KGlobal::iconLoader()->loadIcon("kfm",KIcon::Panel,0,false) );
    QVBoxLayout *frameLayout2 = new QVBoxLayout( frame2, 0, 0 );
    m_prefTheme = new HiQThemePref(frame2, style);
    frameLayout2->addWidget(m_prefTheme);

}

HiQGenPref::HiQGenPref(QWidget *parent, bool animate, int animUpdateDelay, int noAnimUpdateDelay) :QWidget(parent) {
    QVBoxLayout *GenPrefLayout = new QVBoxLayout( this);

    anim = new QRadioButton( i18n("Animate All Pegs (High CPU Usage)"), this, "AnimateAllPegs" );
    GenPrefLayout->addWidget( anim );
    anim->setChecked(animate);
    
    QHBox *sliderPanel = new QHBox(this);
    GenPrefLayout->addWidget( sliderPanel );
    QLabel *animLabel = new QLabel(i18n("Screen Update Delay:"), sliderPanel);
    animSlider = new QSlider( maxUpdateSpeed, minUpdateSpeed, 1, animUpdateDelay, Qt::Horizontal, sliderPanel);
    animLabel->setEnabled(animate);
    animSlider->setEnabled(animate);
    animSlider->setTickmarks(QSlider::Below);
    animSlider->setTickInterval(minUpdateSpeed/10);
    animSlider->setTracking(false);
    connect(anim, SIGNAL(toggled(bool)), animSlider, SLOT(setEnabled(bool)));
    connect(anim, SIGNAL(toggled(bool)), animLabel,  SLOT(setEnabled(bool)));
    
    noAnim = new QRadioButton( i18n("Animate One Peg"), this, "DontAnimateAllPegs" );
    GenPrefLayout->addWidget( noAnim );
    noAnim->setChecked(!animate);
    
    QHBox *sliderPanel2 = new QHBox(this);
    GenPrefLayout->addWidget( sliderPanel2 );
    QLabel *noAnimLabel = new QLabel(i18n("Screen Update Delay:"), sliderPanel2);
    noAnimLabel->setEnabled(!animate);
    noAnimSlider = new QSlider( maxUpdateSpeed, minUpdateSpeed, 1, noAnimUpdateDelay, Qt::Horizontal, sliderPanel2);
    noAnimSlider->setEnabled(!animate);
    noAnimSlider->setTickmarks(QSlider::Below);
    noAnimSlider->setTickInterval(minUpdateSpeed/10);
    noAnimSlider->setTracking(false);
    connect(noAnim, SIGNAL(toggled(bool)), noAnimSlider, SLOT(setEnabled(bool)));
    connect(noAnim, SIGNAL(toggled(bool)), noAnimLabel,  SLOT(setEnabled(bool)));

    QButtonGroup *g = new QButtonGroup();
    g->insert(anim, 0);
    g->insert(noAnim, 1);
    g->setRadioButtonExclusive(true);

    GenPrefLayout->addItem(new QSpacerItem(QSizePolicy::Minimum, QSizePolicy::Expanding));
}

HiQThemePref::HiQThemePref(QWidget *parent, QString style) :QWidget(parent) {

    QButtonGroup *g = new QButtonGroup();
    QVBoxLayout *GenPrefLayout = new QVBoxLayout(this);
    marble = new QRadioButton( i18n("Marble Theme"), this);
    GenPrefLayout->addWidget(marble);
    wood = new QRadioButton( i18n("Wood Theme"), this);
    GenPrefLayout->addWidget(wood);
    metal = new QRadioButton( i18n("Metal Theme"), this);
    GenPrefLayout->addWidget(metal);
    g->insert(marble, 0);
    g->insert(wood, 1);
    g->insert(metal, 2);
    g->setRadioButtonExclusive(true);

    if (style == "wood") {
    wood->setChecked(true);
    } else if (style == "metal") {
    metal->setChecked(true);
    } else {
    marble->setChecked(true);
    }

    GenPrefLayout->addItem(new QSpacerItem(QSizePolicy::Minimum, QSizePolicy::Expanding));

}

#include "pref.moc"
