/*
    Copyright 2009  Stanislas Marquis <smarquis@chaosorigin.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ZODIACORE_COMMON_H
#define ZODIACORE_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

/* Standard C headers */
#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

/* Sqlite 3 */
#include <sqlite3.h>

/* Swisseph */
#include <swephexp.h>

/* Debug */
#ifndef NDEBUG
#   define Debug( x )       x
#else
#   define Debug( x )
#endif /* !NDEBUG */

/* Trace */
#ifdef TRACE
#   define Trace( x )       x
#else
#   define Trace( x )
#endif /* TRACE */

/* Verbosity */
#ifndef SILENT
#   define Verbose( z, x )     if ( z->verboseFlag ) { x }
#else
#   define Verbose( z, x )
#endif /* !SILENT */

/* Fatal error */
#define Fatal( x )      { x exit( -1 ); }

/* Null */
#ifndef NULL
#   define NULL         ((void*)0)
#endif /* !NULL */

/* Inline */
#ifdef __cplusplus
#   define INLINE       inline
#elif defined __STDC_VERSION__ && __STDC_VERSION__ < 199901L
#   define INLINE
#else
#   define INLINE       extern inline
#endif /* INLINE */

/* Gcc */
#ifdef __GNUC__
#   define inline       __inline__
#endif /* __GNUC__ */

/* Index type */
typedef unsigned int idx_t;

/* Boolean */
typedef enum { False, True } Bool;

#ifdef __cplusplus
}
#endif

#endif /* !ZODIACORE_COMMON_H */
