/*
    Copyright 2009  Stanislas Marquis <smarquis@chaosorigin.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ZODIACORE_CORE_H
#define ZODIACORE_CORE_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _zodiacore_t ZodiaCore;

#include "btree.h"
#include "prefs.h"
#include "vector.h"
#include <sqlite3.h>

struct _zodiacore_t
{
#ifndef SILENT
    Bool        verboseFlag;    /* output to stdout */
#endif /* !SILENT */

    char*       dataDir;        /* user data dir */
    char*       shareDir;       /* system shared dir */

    sqlite3*    dataCnx;        /* personal database */
    sqlite3*    atlasCnx;       /* atlas database */

    Prefs*      prefs;          /* preferences */

    BTree*      aspectCollection;           /* aspects */
    BTree*      planetCollection;           /* planets */
    BTree*      aspectFilterCollection;     /* aspects filters */
    BTree*      planetFilterCollection;     /* planets filters */
    BTree*      orbFilterCollection;        /* orbs filters */
    BTree*      aspectRestrCollection;      /* aspects restrictions */
    BTree*      orbRestrCollection;         /* orbs restrictions */
    BTree*      profileCollection;          /* profiles */
    BTree*      contextCollection;          /* contexts */
    BTree*      horoscopeCollection;        /* horoscopes, by indexes */

    Vector*     chartCollection;            /* charts, by opening order */
    Vector*     sortedHoroscopes;           /* horoscopes, sorted by name */

};

void zodiacore_new( ZodiaCore** );

void zodiacore_delete( ZodiaCore**, Bool freeSwe );

void zodiacore_init_data_dir( ZodiaCore* );

void zodiacore_init_share_dir( ZodiaCore* );

#define zcINIT_HomeDir      0x01
#define zcINIT_ShareDir     0x02
#define zcINIT_UseDb        0x04
#define zcINIT_UseAtlas     0x08
#define zcINIT_LoadDb       0x10

#define zcINIT_AllOptions   zcINIT_HomeDir|zcINIT_ShareDir|zcINIT_UseDb|\
                            zcINIT_UseAtlas|zcINIT_LoadDb

void zodiacore_init( ZodiaCore*, int options );

#ifndef NDEBUG

void zodiacore_unittest( ZodiaCore* );

#endif /* !NDEBUG */

#ifdef __cplusplus
}
#endif

#endif /* !ZODIACORE_CORE_H */
