/***************************************************************************
 *   Copyright 2009 Casper van Donderen <casper.vandonderen@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_RENDERSTATE_MANAGER_H
#define KOLF_RENDERSTATE_MANAGER_H

class QColor;

namespace Kolf
{
	class RenderStateManager
	{
		public:
			RenderStateManager();
			void setDefaultRenderState();
			void setGlobalBackgroundColor(float r, float g, float b, float a);
			void setGlobalBackgroundColor(const QColor& color);
			void setGlobalDrawingColor(float r, float g, float b);
			void setGlobalDrawingColor(const QColor& color); ///< \warning Does not use the alpha component specified in \a color.
			void setGlobalLight();

			void enableBlendingState();
			void enableDepthState();
			void enableLightState();
			void enableWireframeState();
			void enableTextureState();

			void disableBlendingState();
			void disableDepthState();
			void disableLightState();
			void disableWireframeState();
			void disableTextureState();

			enum Direction{top, front, back, left, right};
			void setTopViewport();
			void setFrontPerspectiveViewport();
			void setRightPerspectiveViewport();
			void setBackPerspectiveViewport();
			void setLeftPerspectiveViewport();
			void setViewport(int direction);
			void changeViewport(int w, int h, int direction);
			int currentViewport() {return m_currentViewport;}
		private:
			int m_currentViewport;

	};
}

#endif
