/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-itemeditor.h"

#include <QCloseEvent>
#include <QHBoxLayout>
#include <QPushButton>
#include <KLocalizedString>

Kolf::BlenderItemEditor::BlenderItemEditor(QWidget *parent)
	: QWidget(parent)
	, m_min(new QSpinBox(this))
	, m_opt(new QSpinBox(this))
	, m_max(new QSpinBox(this))
	, m_button(new QPushButton(i18n("OK"), this))
{
	m_min->setRange(0, 255);
	m_opt->setRange(0, 255);
	m_max->setRange(0, 255);
	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget(m_min);
	layout->addWidget(m_opt);
	layout->addWidget(m_max);
	layout->addWidget(m_button);
	connect(m_button, SIGNAL(clicked()), this, SLOT(close()));
	connect(m_min, SIGNAL(valueChanged(int)), this, SLOT(minValueChanged(int)));
	connect(m_opt, SIGNAL(valueChanged(int)), this, SLOT(optValueChanged(int)));
	connect(m_max, SIGNAL(valueChanged(int)), this, SLOT(maxValueChanged(int)));
	setLayout(layout);
}

void Kolf::BlenderItemEditor::minValueChanged(int value)
{
	m_opt->setMinimum(value);
}

void Kolf::BlenderItemEditor::optValueChanged(int value)
{
	m_min->setMaximum(value);
	m_max->setMinimum(value);
}

void Kolf::BlenderItemEditor::maxValueChanged(int value)
{
	m_opt->setMaximum(value);
}

void Kolf::BlenderItemEditor::setValue(int min, int opt, int max)
{
	m_min->setValue(min);
	m_max->setValue(max);
	m_opt->setValue(opt);
}

void Kolf::BlenderItemEditor::closeEvent(QCloseEvent* event)
{
	event->accept();
	emit closed();
}

#include "blender-itemeditor.moc"
