/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_BLENDER_MANAGER_H
#define KOLF_BLENDER_MANAGER_H

#include "../elements/utils-heightmap.h"
#include <QMutex>
#include <QImage>
#include <QObject>

class QListView;
class KAction;

namespace Kolf
{
	class BlendThread;
	class TextureModel;
	class TextureListDelegate;

	class BlenderManager : public QObject
	{
		Q_OBJECT
		public:
			explicit BlenderManager(QObject* parent = 0);
			~BlenderManager();
			Heightmap heightmap() const {return m_heightmap;}
			Kolf::TextureModel* model() const {return m_model;}
			QListView* view() const {return m_view;}
			QMutex& mutex() {return m_mutex;}
			QImage blendResult() const {return m_blendResult;}

		public Q_SLOTS:
			void setHeightmap(const Kolf::Heightmap& heightmap);
			void setTexturesModified() {m_texturesModified = true;}
		Q_SIGNALS:
			void blendingFinished();
		private Q_SLOTS:
			void addTextures();
			void delTextures();
			void startBlending(bool force = false);
			void processBlending();

			friend class BlendThread;

		private:
			Kolf::TextureModel* m_model;
			Kolf::TextureListDelegate* m_delegate;
			QListView* m_view;
			Kolf::Heightmap m_heightmap;

			QImage m_blendResult;
			QSize m_outputSize;
			bool m_texturesModified;
			BlendThread* m_blendThread;
			QMutex m_mutex;
	};
}

#endif // KOLF_BLENDER_MANAGER_H 
