/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-model.h"
#include "blender-texture.h"
#include "blender-texturepool.h"

Kolf::TextureModel::TextureModel(QObject* parent)
	: QAbstractListModel(parent)
{
	//default rock texture
	QModelIndex index = Kolf::BlenderTexturePool::instance()->findKey("default-rock");
	Kolf::BlenderTexture* texture = new Kolf::BlenderTexture(index.data(Kolf::BlenderTexturePool::ImageRole).value<QImage>());
	texture->setRange(150, 255, 255);
	m_textures << texture;
	m_thumbnails << index.data(Kolf::BlenderTexturePool::ThumbnailRole).value<QPixmap>();
	//default grass texture
	index = Kolf::BlenderTexturePool::instance()->findKey("default-grass");
	texture = new Kolf::BlenderTexture(index.data(Kolf::BlenderTexturePool::ImageRole).value<QImage>());
	texture->setRange(60, 140, 220);
	m_textures << texture;
	m_thumbnails << index.data(Kolf::BlenderTexturePool::ThumbnailRole).value<QPixmap>();
	//default water texture
	index = Kolf::BlenderTexturePool::instance()->findKey("default-water");
	texture = new Kolf::BlenderTexture(index.data(Kolf::BlenderTexturePool::ImageRole).value<QImage>());
	texture->setRange(0, 0, 80);
	m_textures << texture;
	m_thumbnails << index.data(Kolf::BlenderTexturePool::ThumbnailRole).value<QPixmap>();
}

Kolf::TextureModel::~TextureModel()
{
	qDeleteAll(m_textures);
}

int Kolf::TextureModel::rowCount(const QModelIndex& parent) const
{
	return parent.isValid() ? 0 : m_textures.count();
}

QVariant Kolf::TextureModel::data(const QModelIndex& index, int role) const
{
	if (!index.isValid())
		return QVariant();

	BlenderTexture* texture = m_textures.value(index.row());
	if (!texture) //index.row() out of range
		return QVariant();
	int* range = texture->range();

	switch (role)
	{
		case Qt::DisplayRole:
			return QString("(%1,%2,%3)").arg(range[0]).arg(range[1]).arg(range[2]);
		case Qt::DecorationRole:
			return m_thumbnails.value(index.row());
		case Qt::EditRole:
			return QVariantList() << range[0] << range[1] << range[2];
		default:
			return QVariant();
	}
}

Qt::ItemFlags Kolf::TextureModel::flags(const QModelIndex& index) const
{
	return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
}

bool Kolf::TextureModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
	if (!index.isValid() || role != Qt::EditRole)
		return false;
	Kolf::BlenderTexture* texture = m_textures.value(index.row());
	if (!texture) //index.row() out of range
		return false;

	const QVariantList range = value.toList();
	texture->setRange(range[0].toInt(), range[1].toInt(), range[2].toInt());
	emit dataChanged(index, index);
	emit texturesModified();
	return true;
}

QList<Kolf::BlenderTexture*> Kolf::TextureModel::textures() const
{
	return m_textures;
}

void Kolf::TextureModel::addTexture(const QModelIndex& poolIndex)
{
	int newRow = m_textures.count();
	beginInsertRows(QModelIndex(), newRow, newRow);
	m_textures << new Kolf::BlenderTexture(poolIndex.data(Kolf::BlenderTexturePool::ImageRole).value<QImage>());
	m_thumbnails << poolIndex.data(Kolf::BlenderTexturePool::ThumbnailRole).value<QPixmap>();
	endInsertRows();
	emit texturesModified();
}

void Kolf::TextureModel::removeTexture(int index)
{
	Kolf::BlenderTexture* texture = m_textures.value(index);
	if (!texture) //index out of range
		return;
	beginRemoveRows(QModelIndex(), index, index);
	m_textures.removeAt(index);
	m_thumbnails.removeAt(index);
	delete texture;
	endRemoveRows();
	emit texturesModified();
}

#include "blender-model.moc"
