/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-stacker-texturebar.h"
#include "blender-texturepool.h"

#include <QGraphicsSceneMouseEvent>

//BEGIN Kolf::BsTextureItem

Kolf::BsTextureItem::BsTextureItem(const QString& key)
	: m_outline(new QGraphicsRectItem(this))
{
	setAcceptedMouseButtons(Qt::LeftButton);
	setKey(key);
}

QString Kolf::BsTextureItem::key() const
{
	return m_key;
}

void Kolf::BsTextureItem::setKey(const QString& key)
{
	if (m_key == key)
		return;
	const QModelIndex index = Kolf::BlenderTexturePool::instance()->findKey(key);
	const QPixmap thumbnail = index.data(Kolf::BlenderTexturePool::ThumbnailRole).value<QPixmap>();
	setPixmap(thumbnail);
	setOffset(-thumbnail.width() / 2, -thumbnail.height() / 2);
	m_outline->setRect(QRectF(offset(), thumbnail.size()));
}

void Kolf::BsTextureItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
}

void Kolf::BsTextureItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
	const QPointF diff = event->pos() - event->lastPos();
	emit moveRequest(this, diff.y());
	event->accept();
}

//END Kolf::BsTextureItem

//BEGIN Kolf::BsTextureBar

Kolf::BsTextureBar::BsTextureBar()
	: m_height(0)
{
}

void Kolf::BsTextureBar::setHeight(qreal height)
{
	m_height = height;
	setLine(0.0, 0.0, 0.0, height);
	doLayout();
}

void Kolf::BsTextureBar::addKey(const QString& key)
{
	Kolf::BsTextureItem* newItem = new Kolf::BsTextureItem(key);
	newItem->setParentItem(this);
	m_items << newItem;
	connect(newItem, SIGNAL(moveRequest(Kolf::BsTextureItem*, qreal)), this, SLOT(moveRequest(Kolf::BsTextureItem*, qreal)));
	doLayout();
}

void Kolf::BsTextureBar::removeKey(const QString& key)
{
	//TODO
}

void Kolf::BsTextureBar::doLayout()
{
	const int count = m_items.count();
	for (int i = 0; i < count; ++i)
	{
		const qreal relativePos = (i + 0.5) / count;
		m_items[i]->setPos(0, relativePos * m_height);
	}
}

void Kolf::BsTextureBar::moveRequest(Kolf::BsTextureItem* item, qreal dy)
{
	qreal y = item->pos().y() + dy;
	const qreal topMargin = qAbs(item->boundingRect().top());
	const qreal bottomMargin = qAbs(item->boundingRect().bottom());
	//image has to stay on the line
	if (y < topMargin)
		y = topMargin;
	else if (y > m_height - bottomMargin)
		y = m_height - bottomMargin;
	//move item to destination
	item->setPos(0, y);
}

//END Kolf::BsTextureBar

#include "blender-stacker-texturebar.moc"
