/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-widget.h"
#include "blender-manager.h"
#include "blender-stacker.h"

#include <QGridLayout>
#include <QLabel>
#include <QListView>
#include <QPushButton>
#include <KLocalizedString>

Kolf::BlenderWidget::BlenderWidget()
	: Kolf::GameUIPageWidget(i18n("Texture blender"))
	, m_manager(new Kolf::BlenderManager(this))
	, m_textureViewer(new QLabel)
	, m_stacker(new Kolf::BlenderStacker)
{
	m_textureViewer->setPixmap(QPixmap(512, 512)); //FIXME: hack (the Manager should give an empty QImage with the correct size before the first blending)
	setupUI();
	connect(this, SIGNAL(shown()), m_manager, SLOT(startBlending()));
	connect(m_manager, SIGNAL(blendingFinished()), this, SLOT(updateTexture()));
}

void Kolf::BlenderWidget::setupUI()
{
	QGridLayout* layout = new QGridLayout;
	QPushButton* button = new QPushButton(i18n("Regenerate texture"));
	layout->addWidget(m_stacker, 0, 0);
	layout->addWidget(m_manager->view(), 0, 1);
	layout->addWidget(button, 1, 0, 1, 2);
	layout->addWidget(m_textureViewer, 0, 2, 2, 1);
	connect(button, SIGNAL(clicked()), m_manager, SLOT(startBlending()));
	setLayout(layout);
}

Kolf::BlenderWidget::~BlenderWidget()
{
}

Kolf::BlenderManager* Kolf::BlenderWidget::manager() const
{
	return m_manager;
}

void Kolf::BlenderWidget::updateTexture()
{
	m_textureViewer->setPixmap(QPixmap::fromImage(m_manager->blendResult()));
}

#include "blender-widget.moc"
