/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-fillbutton.h"

#include <QPainter>
#include <QPen>
#include <KIcon>
#include <KLocalizedString>

Kolf::FillButton::FillButton(QWidget* parent)
	: ToolButton(parent)
	, m_gradientSpread(QGradient::PadSpread)
	, m_gradientType(Qt::RadialGradientPattern)
{
	setIcon(KIcon("fill-color"));
	setText(i18n("Fill selection"));
	setToolTip(i18n("Fills the selected area (or everything if nothing has been selected) with a smooth gradient."));
	setObjectName("FillButton");
}

Kolf::FillButton::~FillButton()
{
}

void Kolf::FillButton::handleMouseMoved(DrawingBoard* board, QMouseEvent* event)
{
	board->restoreImage();
	QPainter painter(&board->image());
	painter.save();
	painter.setPen(Kolf::DrawingBoard::HelperLinesPen);
	painter.drawLine(board->lastPoint(), event->posF());
	painter.restore();
	drawPoint(&painter, board->lastPoint(), 4);
	drawPoint(&painter, event->posF(), 4);
}

void Kolf::FillButton::handleMouseReleased(DrawingBoard* board, QMouseEvent* event)
{
	board->restoreImage();
	m_gradientPoints.first = board->lastPoint();
	m_gradientPoints.second = event->posF();
	QPainter painter(&board->image());
	//gradient
	QGradient gradient;
	if(m_gradientType == Qt::LinearGradientPattern)
	{
		gradient = QLinearGradient(m_gradientPoints.first,
				m_gradientPoints.second);
	}
	else if(m_gradientType == Qt::RadialGradientPattern)
	{
		gradient = QRadialGradient(m_gradientPoints.first,
				qMin(board->selection().boundingRect().width(),
				     board->selection().boundingRect().height()),
				m_gradientPoints.second);
	}
	else if(m_gradientType == Qt::ConicalGradientPattern)
	{
		QLineF line(m_gradientPoints.first, m_gradientPoints.second);
        	qreal angle = line.angle(QLineF(0, 0, 1, 0));
		if(line.dy() > 0)
            		angle = 360 - angle;
		gradient = QConicalGradient(m_gradientPoints.first, angle);
	}

	gradient.setSpread(m_gradientSpread);
	gradient.setStops(m_gradientStops);
	//brush
	QBrush brush(gradient);
	//painterPath && draw
	painter.fillPath(board->selection(), brush);
	board->storeImage();
}

void Kolf::FillButton::drawPoint(QPainter* painter, const QPointF& pos, qreal radius)
{
	painter->save();
	painter->setPen(Kolf::DrawingBoard::HelperLinesPen);
	qreal x = pos.x() - radius;
	qreal y = pos.y() - radius;
	painter->drawEllipse(QRectF(x, y, 2 * radius, 2 * radius));
	painter->restore();
}
#include "hmcreator-fillbutton.moc"
