/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-penconfwidget.h"
#include "hmcreator-penbutton.h"
#include <QGroupBox>
#include <QLabel>
#include <QPainter>
#include <QPushButton>
#include <QSlider>
#include <QSpinBox>
#include <QVBoxLayout>

Kolf::PenConfWidget::PenConfWidget(ToolButton* button, QWidget* parent)
	: QWidget(parent)
	, m_button(button)
{
	connect(this, SIGNAL(penWidthChanged(int)), m_button, SLOT(setPenWidth(int)));
	connect(this, SIGNAL(penColorChanged(const QColor&)), m_button, SLOT(setPenColor(const QColor&)));
	setupUI();
}

void Kolf::PenConfWidget::changePenColor(int value)
{
	m_penColor = QColor(value, value, value);
	emit penColorChanged(m_penColor);
}

void Kolf::PenConfWidget::setupUI()
{
	QGroupBox* groupBox = new QGroupBox(this);
	QVBoxLayout* layout = new QVBoxLayout;
	QLabel* title = new QLabel("Pen Color");
	layout->addWidget(title);
	QLabel* label = new QLabel(this);
	createPixmap();
	label->setPixmap(m_pixmap);
	layout->addWidget(label);
	QSlider* slider = new QSlider;
	connect(slider, SIGNAL(valueChanged(int)), this, SLOT(changePenColor(int)));
	slider->setRange(0, 255);
	slider->setValue(255);
	slider->setOrientation(Qt::Horizontal);
	slider->setInvertedAppearance(true);
	layout->addWidget(slider);
	title = new QLabel("Pen Width");
	layout->addWidget(title);
	QSpinBox* spinbox = new QSpinBox;
	connect(spinbox, SIGNAL(valueChanged(int)), this, SIGNAL(penWidthChanged(int)));
	spinbox->setRange(1, 100);
	spinbox->setValue(4);
	layout->addWidget(spinbox);
	QPushButton* okButton = new QPushButton(QString("OK"));
	layout->addWidget(okButton);
	groupBox->setLayout(layout);
}

void Kolf::PenConfWidget::createPixmap()
{
	m_pixmap = QPixmap(QSize(150, 30));
	QLinearGradient gradient(0, 0, m_pixmap.width(), 0);
	gradient.setColorAt(0, Qt::white);
	gradient.setColorAt(1, Qt::black);
	QPainter painter(&m_pixmap);
	painter.fillRect(m_pixmap.rect(), gradient);
	painter.setPen(QColor(146, 146, 146));
	painter.drawRect(m_pixmap.rect());
}
#include "hmcreator-penconfwidget.moc"
