/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-shadeconfwidget.h"
#include "hmcreator-fillbutton.h"
#include "hmcreator-hoverpoints.h"

Kolf::ShadeConfWidget::ShadeConfWidget(ToolButton* button, QWidget* parent)
	: QWidget(parent) 
	, m_button(button)
	, m_hoverPoints(new HoverPoints(this, HoverPoints::CircleShape))
{
    	setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    	connect(m_hoverPoints, SIGNAL(pointsChanged(const QPolygonF &)), this, SLOT(changePoints(const QPolygonF &)));
	connect(this, SIGNAL(stopsChanged(const QGradientStops& )), m_button, SLOT(changeGradientStops(const QGradientStops&)));

}

void Kolf::ShadeConfWidget::paintEvent(QPaintEvent* event)
{
	Q_UNUSED(event);
	createShade();
	QPainter painter(this);
	painter.drawPixmap(0, 0, m_shade);
	painter.setPen(QColor(146, 146, 146));
	painter.drawRect(0, 0, width() - 1, height() - 1);
}

void Kolf::ShadeConfWidget::showEvent(QShowEvent* event)
{
	Q_UNUSED(event);
	reloadPoints();
}

void Kolf::ShadeConfWidget::changePoints(const QPolygonF& points)
{
	m_points.clear();
	m_points = points;
	changeStops();
}

void Kolf::ShadeConfWidget::changeStops()
{
	m_gradientStops.clear();
	qreal w = width();
	qreal h = height();
	foreach(const QPointF& point, m_points)
	{
		qreal x_factor = point.x() / w;
		qreal y_factor = 1 - point.y() / h;
		QColor color = QColor::fromRgbF(y_factor, y_factor, y_factor);
		m_gradientStops << QGradientStop(x_factor, color); 
	}
	emit stopsChanged(m_gradientStops);
}

void Kolf::ShadeConfWidget::reset()
{
	m_points.clear();
	m_points << QPointF(0, 0) << QPointF(width(), height());
	reloadPoints();
	update();
}

void Kolf::ShadeConfWidget::createShade()
{
	if(m_shade.isNull() || m_shade.size() != size())
	{
		m_shade = QPixmap(size());
		QLinearGradient gradient(0, 0, 0, height());
		gradient.setColorAt(0, Qt::white);
		gradient.setColorAt(1, Qt::black);
        	QPainter painter(&m_shade);
        	painter.fillRect(rect(), gradient);
	}
}

void Kolf::ShadeConfWidget::reloadPoints()
{
	if(m_points.size() == 0)
		m_points << QPointF(0, 0) << QPointF(width(), height());
	m_hoverPoints->setPoints(m_points);
	m_hoverPoints->setPointLock(0, HoverPoints::LockToLeft);
	m_hoverPoints->setPointLock(1, HoverPoints::LockToRight);
	m_hoverPoints->setSortType(HoverPoints::XSort);
	changeStops();
}

#include "hmcreator-shadeconfwidget.moc"
