/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-toolbox.h"
#include "hmcreator-allselectbutton.h"
#include "hmcreator-ellipseselectbutton.h"
#include "hmcreator-eraserbutton.h"
#include "hmcreator-fillbutton.h"
#include "hmcreator-penbutton.h"
#include "hmcreator-rectselectbutton.h"
#include "hmcreator-toolboxconfwidget.h"
#include "hmcreator-toolbutton.h"

#include <QVBoxLayout>
#include <KIcon>
#include <KLocalizedString>
#include <KSeparator>

Kolf::ToolBox::ToolBox(DrawingBoard* board)
	: GameUIDockWidget(i18n("Heightmap toolbox"))
	, m_drawingBoard(board)
{
	setObjectName("ToolBox");
	setPreferredArea(Qt::RightDockWidgetArea);
	setupUI();
}

Kolf::ToolBox::~ToolBox()
{
}

Kolf::ToolButton* Kolf::ToolBox::button(const QString& buttonName)
{
	foreach(ToolButton* item, m_buttonList)
	{
		if(item->objectName() == buttonName)
			return item;
	}
	return 0;
}

void Kolf::ToolBox::setupUI()
{
	ToolButton* allButton = new AllSelectButton(this);
	connect(allButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << allButton;

	ToolButton* ellipseButton = new EllipseSelectButton(this);
	connect(ellipseButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << ellipseButton;

	ToolButton* rectButton = new RectSelectButton(this);
	connect(rectButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << rectButton;

	ToolButton* fillButton = new FillButton(this);
	connect(fillButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << fillButton;

	ToolButton* penButton = new PenButton(this);
	connect(penButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << penButton;

	ToolButton* eraserButton = new EraserButton(this);
	connect(eraserButton, SIGNAL(toggled(bool)), this, SLOT(buttonChanged()));
	m_buttonList << eraserButton;

	m_confWidget = new ToolBoxConfWidget(this);
	connect(this, SIGNAL(setCurrentButton(ToolButton*)), m_drawingBoard, SLOT(currentToolButtonChanged(ToolButton*)));
	connect(this, SIGNAL(setCurrentButton(ToolButton*)), m_confWidget, SLOT(buttonChanged(ToolButton*)));

	QVBoxLayout *vLayout = new QVBoxLayout;
	vLayout->addWidget(allButton);
	vLayout->addWidget(ellipseButton);
	vLayout->addWidget(rectButton);
	vLayout->addWidget(new KSeparator(Qt::Horizontal));
	vLayout->addWidget(fillButton);
	vLayout->addWidget(penButton);
	vLayout->addWidget(eraserButton);
	vLayout->addWidget(m_confWidget);

	QWidget* widget = new QWidget;
	widget->setLayout(vLayout);
	setWidget(widget);
}

void Kolf::ToolBox::buttonChanged()
{
	ToolButton* button = dynamic_cast<ToolButton*>(QObject::sender());
	if(button && button->isChecked())
	{
		foreach(ToolButton* item, m_buttonList)
			if(item != button)
				item->setChecked(false);
		emit setCurrentButton(button);
	}
	else
		emit setCurrentButton(0);
}

#include "hmcreator-toolbox.moc"
