/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-toolboxconfwidget.h"
#include "hmcreator-eraserconfwidget.h"
#include "hmcreator-fillconfwidget.h"
#include "hmcreator-penconfwidget.h"
#include "hmcreator-toolbox.h"
#include "hmcreator-toolbutton.h"

#include <QVBoxLayout>

Kolf::ToolBoxConfWidget::ToolBoxConfWidget(ToolBox* toolBox, QWidget* parent)
	: QWidget(parent)
	, m_toolBox(toolBox)
{
	foreach(QWidget* item, m_widgetList)
		item->hide();
	setupUI();
	this->show();
	showWidget(0);
}

void Kolf::ToolBoxConfWidget::buttonChanged(ToolButton* button)
{
	if(!button)
	{
		showWidget(0);
		return;
	}

	if(button->objectName() == QString("FillButton"))
	{
		showWidget(m_fillWidget);
		return;
       	}

	if(button->objectName() == QString("PenButton"))
	{
		showWidget(m_penWidget);
		return;
	}

	if(button->objectName() == QString("EraserButton"))
	{
		showWidget(m_eraserWidget);
		return;
	}

}

void Kolf::ToolBoxConfWidget::setupUI()
{
	m_fillWidget = new FillConfWidget(m_toolBox->button("FillButton"), this);
	m_widgetList << m_fillWidget;
	m_penWidget = new PenConfWidget(m_toolBox->button("PenButton"), this);
	m_widgetList << m_penWidget;
	m_eraserWidget= new EraserConfWidget(m_toolBox->button("EraserButton"), this);
	m_widgetList << m_eraserWidget;

	QVBoxLayout* layout = new QVBoxLayout;
	layout->addWidget(m_fillWidget);
	layout->addWidget(m_penWidget);
	layout->addWidget(m_eraserWidget);
	setLayout(layout);
}

void Kolf::ToolBoxConfWidget::showWidget(QWidget* widget)
{
	if(!widget)
	{
		foreach(QWidget* item, m_widgetList)
			item->hide();
		return;
	}

	foreach(QWidget* item, m_widgetList)
	{
		if(item == widget)
			item->show();
		else
			item->hide();
	}
}

#include "hmcreator-toolboxconfwidget.moc"
