/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "ball.h"
#include "geometry-sphere.h"
#include "objectregistry.h"
#include "overlay-empty.h"

#include <QBrush>
#include <QGraphicsEllipseItem>
#include <QGraphicsScene>
#include <QList>
#include <QPair>

const int Kolf::Ball::BallRadius = 10;

REGISTER_KOLF_OBJECT(Kolf::Ball, ki18n("Ball (start position for all players)"), KIcon())

namespace Kolf
{
	typedef QPair<const Kolf::Ball*, const Kolf::Ball*> BallPair;
	QList<Kolf::BallPair> recentCollisions;
	//The problem that this list solves is that colliding balls may try to resolve collisions multiple times. This list records recent collisions and keeps the balls from colliding again until they have moved away.
	//TODO: Is this still necessary?
}

Kolf::Ball::Ball(dWorldID worldID, dSpaceID spaceID)
	: Kolf::Object(worldID, spaceID)
	, m_geometry(new Kolf::SphereGeometry(spaceID, this))
	, m_ballItem(new QGraphicsEllipseItem(QRectF(-BallRadius, -BallRadius, 2 * BallRadius, 2 * BallRadius), this))
	, m_inHole(false)
{
	m_geometry->setRadius(BallRadius);
	m_ballItem->setBrush(color());
}

Kolf::Ball::~Ball()
{
	delete m_ballItem;
}

bool Kolf::Ball::isInHole() const
{
	return m_inHole;
}

void Kolf::Ball::setInHole(bool inHole)
{
	m_inHole = inHole;
}

Kolf::Object::SimulationFlags Kolf::Ball::simulationFlags() const
{
	if (m_inHole)
		return Kolf::Object::NoSimulation;
	else
		return Kolf::Object::KinematicSimulation | Kolf::Object::DynamicSimulation;
}

void Kolf::Ball::objectEvent(int type, const QVariant& data)
{
	Kolf::Object::objectEvent(type, data);
	switch (type)
	{
		case Kolf::Object::PositionChangeEvent:
			emit positionChanged(data.value<Kolf::PointF>());
			break;
		case Kolf::Object::ColorChangeEvent:
			m_ballItem->setBrush(data.value<QColor>());
			break;
	}
}

Kolf::Overlay* Kolf::Ball::createOverlay()
{
	return new Kolf::EmptyOverlay(this);
}

#include "ball.moc"
