/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "geometry-cuboid.h"

#include <ode/ode.h>

Kolf::CuboidGeometry::CuboidGeometry(dSpaceID spaceID, Kolf::Object* object)
	: Kolf::Geometry(dCreateBox(spaceID, 1.0, 1.0, 1.0), object)
	//set radius to 1 because I do not know how ODE behaves when given a null extent
{
	dMass mass;
	dMassSetSphere(&mass, 1.0, 1.0); //density is (for now) fixed to 1.0
	setMass(mass);
}

Kolf::SizeF Kolf::CuboidGeometry::size() const
{
	dVector3 sizeComponents;
	dGeomBoxGetLengths(id(), sizeComponents);
	return Kolf::SizeF(sizeComponents[0], sizeComponents[1], sizeComponents[2]);
}

void Kolf::CuboidGeometry::setSize(const Kolf::SizeF& size)
{
	dGeomBoxSetLengths(id(), size.xSpread(), size.ySpread(), size.zSpread());
	Kolf::Geometry::update();
}

void Kolf::CuboidGeometry::createOutlines()
{
	const Kolf::SizeF size = this->size();
	const qreal width = size.xSpread(), height = size.ySpread();
	//interaction outline
	QRectF rect(-width / 2, -height / 2, width, height);
	QPainterPath interactionOutline;
	interactionOutline.addRect(rect);
	setInteractionOutline(interactionOutline);
	//activation outline
	rect.setLeft(rect.left() - Kolf::Geometry::ActivationOutlinePadding);
	rect.setTop(rect.top() - Kolf::Geometry::ActivationOutlinePadding);
	rect.setRight(rect.right() + Kolf::Geometry::ActivationOutlinePadding);
	rect.setBottom(rect.bottom() + Kolf::Geometry::ActivationOutlinePadding);
	QPainterPath activationOutline;
	activationOutline.addRect(rect);
	setActivationOutline(activationOutline);
}
