/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "geometry-sphere.h"

#include <ode/ode.h>

Kolf::SphereGeometry::SphereGeometry(dSpaceID spaceID, Kolf::Object* object)
	: Kolf::Geometry(dCreateSphere(spaceID, 1.0), object)
	//set radius to 1 because I do not know how ODE behaves when given radius 0
{
	dMass mass;
	dMassSetSphere(&mass, 1.0, 1.0); //density is (for now) fixed to 1.0
	setMass(mass);
}

qreal Kolf::SphereGeometry::radius() const
{
	return dGeomSphereGetRadius(id());
}

void Kolf::SphereGeometry::setRadius(qreal radius)
{
	dGeomSphereSetRadius(id(), radius);
	dMass mass;
	dMassSetSphere(&mass, 1.0, radius);
	setMass(mass);
	Kolf::Geometry::update();
}

void Kolf::SphereGeometry::createOutlines()
{
	const qreal radius = this->radius();
	const qreal activationRadius = radius + Kolf::Geometry::ActivationOutlinePadding;
	const QPointF center(0.0, 0.0);
	//interaction outline
	QPainterPath interactionOutline;
	interactionOutline.addEllipse(center, radius, radius);
	setInteractionOutline(interactionOutline);
	//interaction outline
	QPainterPath activationOutline;
	activationOutline.addEllipse(center, activationRadius, activationRadius);
	setActivationOutline(activationOutline);
}
