/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OBJECT_BLACKHOLE_H
#define KOLF_OBJECT_BLACKHOLE_H

#include "object.h"

class QGraphicsEllipseItem;

namespace Kolf
{
	//The black hole's reference point is the center of its entry point.
	class BlackHoleObject : public Kolf::Object
	{
		Q_OBJECT
		//note that Kolf::VectorF is a typedef for Kolf::PointF
		Q_PROPERTY(Kolf::PointF ExitOffset READ exitOffset WRITE setExitOffset) ///< the distance of the exit from the entrye
		Q_PROPERTY(int ExitDelay READ exitDelay WRITE setExitDelay) ///< \warning in milliseconds!
		Q_PROPERTY(Kolf::PointF ExitVelocity READ exitVelocity WRITE setExitVelocity)
		public:
			BlackHoleObject(dWorldID worldID, dSpaceID spaceID);
			virtual ~BlackHoleObject();

			virtual bool isRotateable() const { return false; }

			int exitDelay() const;
			Kolf::PointF exitOffset() const;
			Kolf::VectorF exitVelocity() const;
			void setExitDelay(int exitDelay);
			void setExitOffset(const Kolf::PointF& exitOffset);
			void setExitVelocity(const Kolf::VectorF& exitVelocity);

			virtual Kolf::Object::SimulationFlags simulationFlags() const;
			virtual void doCustomInteraction(Kolf::Object* object);

			static const qreal ExitHalfLength;
		protected:
			virtual Kolf::Overlay* createOverlay();
		private:
			QGraphicsEllipseItem* m_entryItem;
			QGraphicsLineItem* m_exitItem;
			int m_exitDelay;
			Kolf::PointF m_exitOffset;
			Kolf::VectorF m_exitVelocity;
	};
}

#endif // KOLF_OBJECT_HOLE_H
