/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-coursemeta.h"
#include "objectregistry.h"
#include "object-wall.h"
#include "overlay-empty.h"
#include "scene.h"

REGISTER_KOLF_OBJECT(Kolf::CourseMetaObject, ki18n("Internal course representation"), KIcon())

Kolf::CourseMetaObject::CourseMetaObject(dWorldID worldID, dSpaceID spaceID)
	: Kolf::Object(worldID, spaceID)
	, m_hasCourseBoundary(false)
{
}

Kolf::CourseMetaObject::~CourseMetaObject()
{
}

bool Kolf::CourseMetaObject::hasCourseBoundary() const
{
	return m_hasCourseBoundary;
}

void Kolf::CourseMetaObject::setCourseBoundary(bool hasCourseBoundary)
{
	if (m_hasCourseBoundary == hasCourseBoundary)
		return;
	m_hasCourseBoundary = hasCourseBoundary;
	//if necessary, create course boundary
	if (hasCourseBoundary)
	{
		const QRectF sr = Kolf::Scene::SceneRect;
		//upper wall
		Kolf::WallObject* wall = reinterpret_cast<Kolf::WallObject*>(Kolf::ObjectRegistry::createObject("Kolf::WallObject", this)); //FIXME: ugly
		wall->setExtent(sr.topLeft(), sr.topRight());
		m_courseBoundaryComponents << wall;
		addSubObject(wall);
		//lower wall
		wall = reinterpret_cast<Kolf::WallObject*>(Kolf::ObjectRegistry::createObject("Kolf::WallObject", this));
		wall->setExtent(sr.bottomLeft(), sr.bottomRight());
		m_courseBoundaryComponents << wall;
		addSubObject(wall);
		//left wall
		wall = reinterpret_cast<Kolf::WallObject*>(Kolf::ObjectRegistry::createObject("Kolf::WallObject", this));
		wall->setExtent(sr.topLeft(), sr.bottomLeft());
		m_courseBoundaryComponents << wall;
		addSubObject(wall);
		//right wall
		wall = reinterpret_cast<Kolf::WallObject*>(Kolf::ObjectRegistry::createObject("Kolf::WallObject", this));
		wall->setExtent(sr.topRight(), sr.bottomRight());
		m_courseBoundaryComponents << wall;
		addSubObject(wall);
	}
	//if necessary, delete course boundary
	else
	{
		foreach (Kolf::Object* wallObject, m_courseBoundaryComponents)
			removeSubObject(wallObject);
		qDeleteAll(m_courseBoundaryComponents);
		m_courseBoundaryComponents.clear();
	}
}

Kolf::Overlay* Kolf::CourseMetaObject::createOverlay()
{
	return new Kolf::EmptyOverlay(this);
}

#include "object-coursemeta.moc"
