/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OBJECT_WALL_H
#define KOLF_OBJECT_WALL_H

#include "object.h"

class QGraphicsLineItem;

namespace Kolf
{
	class WallGeometry;

	//A wall's reference point is its center. The extent is the distance from one of the wall's corners to its center.
	//Note that the wall is always vertical! (Everything else would be very hard to display in the 2D view.)
	class WallObject : public Kolf::Object
	{
		Q_OBJECT
		Q_PROPERTY(Kolf::PointF Extent READ extent WRITE setExtent) //Kolf::VectorF is a typedef for Kolf::PointF
		public:
			WallObject(dWorldID worldID, dSpaceID spaceID);
			~WallObject();

			virtual bool isRotateable() const { return false; }

			Kolf::VectorF extent() const;
			void setExtent(const Kolf::VectorF& extent);
			void setExtent(const QPointF& point1, const QPointF& point2); //This overload is used by Kolf::EnviromentObject.
		protected:
			virtual Overlay* createOverlay();
		private:
			Kolf::WallGeometry* m_geometry;
			QGraphicsLineItem* m_line;
	};
}

#endif // KOLF_OBJECT_WALL_H
