/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_PHYSICSENGINE_H
#define KOLF_PHYSICSENGINE_H

#include <ode/common.h>
#include <QObject>
#include <QPair>

namespace Kolf
{
	class Geometry;
	class Object;
	class Scene;

	typedef QPair<Kolf::Object*, Kolf::Object*> InteractionPair;

	class PhysicsEngine : public QObject
	{
		Q_OBJECT
		public:
			PhysicsEngine(Kolf::Scene* scene);
			~PhysicsEngine();

			Kolf::Scene* scene() const;
		Q_SIGNALS:
			void nextFrameAvailable();
			void turnEnded();
		public Q_SLOTS:
			void startSimulation();
			void stopSimulation();
		protected:
			virtual void timerEvent(QTimerEvent* event);
		private:
			static void resolveCollisionCallback(void* data, dGeomID o1, dGeomID o2);
			void resolveCollision(Kolf::Geometry* geometry1, Kolf::Geometry* geometry2);

			int m_timerID;
			Kolf::Scene* m_scene;

			dJointGroupID m_contactJointGroupID;
			QList<Kolf::InteractionPair> m_pendingCustomInteractions;
			bool m_ballsMoving; //TODO: This is not a clean design.
	};
}

#endif // KOLF_PHYSICSENGINE_H
