/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_PUTTER_H
#define KOLF_PUTTER_H

#include <QGraphicsItem>
class QGraphicsLineItem;
#include <QObject>

namespace Kolf
{
	class Ball;

	/**
	 * \class Putter
	 * \since 2.0
	 *
	 * Provides the putting interface (esp. the helper line showing the velocity vector), and sets the ball velocity accordingly.
	 */
	class Putter : public QObject, public QGraphicsItem
	{
		Q_OBJECT
		public:
			///Creates a new putter which is not visible until you assign a ball to it.
			///\see show()
			Putter(QGraphicsScene* scene);
			~Putter();

			///Returns the ball that this putter is currently attached to.
			Kolf::Ball* ball() const;

			virtual QRectF boundingRect() const;
			virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0);
		public Q_SLOTS:
			///Attaches the putter to the given \a ball.
			void show(Kolf::Ball* ball);
			///Removes the putter from the screen, and detaches it from its ball.
			void hide();
			///Tells the putter that the mouse has left one of the views (to be called by the view classes).
			void mouseLeaveEvent();
		Q_SIGNALS:
			///This signal is emitted when the user clicks to put the ball, in order to notify the game engine that the score has to be incremented.
			void putted(Kolf::Ball* ball);
		protected:
			virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* event);
			virtual void mousePressEvent(QGraphicsSceneMouseEvent* event);
			///Moves the line to start at the ball's position and stop at the \a cursorPosition. If the \a cursorPosition is outside the scene rect, the line is hidden.
			void changeLine(const QPointF& cursorPosition);
			///
			QPointF findCursorScenePos() const;
		private:
			QGraphicsLineItem* m_line;
			Kolf::Ball* m_ball;
			int m_direction; ///< 0 means that putter points in direction of mouse. Each +1 adds a difference of 90° (clockwise) between ball-cursor vector and velocity vector.
	};
}

#endif // KOLF_PUTTER_H
