/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_SCENE_H
#define KOLF_SCENE_H

#include "objectlist.h"

#include <ode/common.h>
#include <QGraphicsScene>

namespace Kolf
{
	class Ball;
	class ManagedObject;
	class ManagedObjectGroup;
	class PhysicsEngine;
	class Putter;

	/**
	 * \class Scene
	 * \since 2.0
	 *
	 * This QGraphicsScene subclass provides the ODE world used for collision.
	 */
	class Scene : public QGraphicsScene
	{
		Q_OBJECT
		public:
			Scene(QObject* parent = 0);
			virtual ~Scene();

			dWorldID worldID() const;
			dSpaceID spaceID() const;
			static QRectF SceneRect;

			QList<Kolf::Ball*> balls() const;
			Kolf::ObjectList objects() const;

			Kolf::PhysicsEngine* physicsEngine() const;
			Kolf::ManagedObjectGroup* objectGroup() const;
			Kolf::Putter* putter() const;

			Kolf::Ball* addBall();
			void removeBall(Kolf::Ball* ball);
		Q_SIGNALS:
			void turnEnded();
		private Q_SLOTS:
			void objectCreated(Kolf::ManagedObject* managedObject);
			void objectAboutToBeDeleted(Kolf::ManagedObject* managedObject);
			void addObject(Kolf::Object* object);
			void removeObject(Kolf::Object* object);
		private:
			dWorldID m_worldID;
			dSpaceID m_spaceID;
			Kolf::ObjectList m_objects;
			QList<Kolf::Ball*> m_balls;
			//scene managers
			Kolf::PhysicsEngine* m_physicsEngine;
			Kolf::ManagedObjectGroup* m_objectGroup;
			Kolf::Putter* m_putter;
	};
}

#endif // KOLF_SCENE_H
