/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GAME_EDITOR_H
#define KOLF_GAME_EDITOR_H

class KAction;

#include "game.h"
#include "gameui.h"

namespace Kolf
{
	class ManagedObject;

	/**
	 * \class EditorGame
	 * \since 2.0
	 *
	 * This game does not allow to add players, but it provides an editor interface.
	 */
	class EditorGame : public Kolf::Game
	{
		Q_OBJECT
		public:
			///Constructs a new Kolf::EditorGame instance. Note that the game takes ownership of the \a course.
			EditorGame(Kolf::Course* course);
			virtual ~EditorGame();

			virtual bool startGame();
		protected:
			virtual void currentHoleAboutToBeChanged();
			virtual void currentHoleChanged();
		private Q_SLOTS:
			void overlayStateChanged();

			void actionGoToPreviousHole();
			void actionGoToNextHole();

			void objectCreated(Kolf::ManagedObject* managedObject);
	};

	class EditorGameUI : public Kolf::GameUI
	{
		Q_OBJECT
		public:
			EditorGameUI();
		Q_SIGNALS:
			void actionGoToPreviousHole();
			void actionGoToNextHole();
			void actionSave();
		private Q_SLOTS:
			void setCleanDispatcher(bool clean);
		private:
			friend class EditorGame;

			KAction* m_saveAction;
			KAction* m_undoAction;
			KAction* m_redoAction;
	};
}

#endif // KOLF_GAME_EDITOR_H
