/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GAME_NORMAL_H
#define KOLF_GAME_NORMAL_H

#include "game.h"
#include "gameui.h"

namespace Kolf
{
	class LocalPlayer;
	class ScoreCardComponent;

	/**
	 * \class NormalGame
	 * \since 2.0
	 *
	 * The standard game implementation which lets a handful players play locally.
	 */
	class NormalGame : public Kolf::Game
	{
		Q_OBJECT
		public:
			///Constructs a new Kolf::NormalGame instance. Note that the game takes ownership of \a course.
			NormalGame(Kolf::Course* course);
			virtual ~NormalGame();

			Kolf::LocalPlayer* addPlayer(); ///< \warning Do not use after the game has started.
			virtual bool startGame();
		Q_SIGNALS:
			void playerMoveStarted(int player);
			void playerMoveEnded(int player);
			void gameEnded();
		protected:
			virtual void currentHoleAboutToBeChanged();
			virtual void currentHoleChanged();
		private Q_SLOTS:
			void finishMove(); ///< Called by a ball when it stops.
			void advanceHole(); ///< Advance to next hole (or end game if current hole is last one).
			void advancePlayer(); ///< Turn control to next player (or advance to next hole if all players have finished).

			void actionSkipHole();
		private:
			int m_currentPlayer; //counting starts at zero
	};

	class NormalGameUI : public Kolf::GameUI
	{
		Q_OBJECT
		public:
			NormalGameUI();

			void lateInit(Kolf::NormalGame* game);
		Q_SIGNALS:
			void actionSkipHole();
		private:
			Kolf::ScoreCardComponent* m_scoreCard;
	};
}

#endif // KOLF_GAME_NORMAL_H
