/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "game.h"
#include "course.h"
#include "gameui.h"
#include "player-dummy.h"
#include "../elements/putter.h"

#include <KLocalizedString>

Kolf::Game::Game(Kolf::Course* course, Kolf::GameUI* gameUI)
	: m_currentHole(-1)
	, m_course(course)
	, m_gameUI(gameUI)
	, m_parPlayer(new Kolf::DummyPlayer(i18n("Par"), course->holeCount()))
{
	m_parPlayer->setHue(0); //provide some default value for games that display the par ball
	//retrieve par scores
	for (int i = 0; i < m_course->holeCount(); ++i)
		m_parPlayer->setScore(i, m_course->parForHole(i));
	//connect to game UI
	connect(this, SIGNAL(sceneChanged(Kolf::Scene*)), gameUI, SIGNAL(sceneChanged(Kolf::Scene*)));
}

Kolf::Game::~Game()
{
	//detach scenes etc. from all clients
	setCurrentHole(-1);
	delete m_gameUI; //the game UI has to be deleted before the other data, as it might try to reference the data
	//delete everything
	qDeleteAll(m_players);
	delete m_parPlayer;
	delete m_course;
}

Kolf::Course* Kolf::Game::course() const
{
	return m_course;
}

Kolf::GameUI* Kolf::Game::gameUI() const
{
	return m_gameUI;
}

Kolf::Player* Kolf::Game::parPlayer() const
{
	return m_parPlayer;
}

QList<Kolf::Player*> Kolf::Game::players() const
{
	return m_players;
}

int Kolf::Game::currentHole() const
{
	return m_currentHole;
}

Kolf::Scene* Kolf::Game::currentScene() const
{
	return m_course->hole(m_currentHole);
}

void Kolf::Game::addPlayer(Kolf::Player* player)
{
	m_players << player;
	emit playerAdded(player);
}

void Kolf::Game::setCurrentHole(int newHole)
{
	//announce change
	int oldHole = m_currentHole;
	emit currentHoleAboutToBeChanged(newHole, oldHole);
	currentHoleAboutToBeChanged();
	//change current hole
	m_currentHole = newHole;
	Kolf::Scene* scene = currentScene();
	foreach (Kolf::Player* player, m_players)
	{
		player->setCurrentHole(newHole);
		player->setScene(scene);
	}
	//announce change
	currentHoleChanged();
	emit currentHoleChanged(newHole, oldHole);
	emit sceneChanged(scene);
}

void Kolf::Game::currentHoleAboutToBeChanged()
{
}

void Kolf::Game::currentHoleChanged()
{
}

#include "game.moc"
