/***************************************************************************
 *   Copyright 2009 Casper van Donderen <casper.vandonderen@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_VIEW3D_H
#define KOLF_VIEW3D_H
#include <GL/glew.h>
#include <QGLWidget>
class QTimer;

namespace Kolf
{
	class RenderStateManager;
	class Scene;

	class View3D : public QGLWidget
	{
		Q_OBJECT
		private:
			View3D(QGLContext* context);
		public:
			///Use this instead of a constructor.
			static View3D* create();
			virtual ~View3D();

			Kolf::RenderStateManager* rsm() const;
			void setScene(Kolf::Scene* scene);
		public Q_SLOTS:
			void startTimer();
			void stopTimer();
		protected:
			virtual void initializeGL();
			virtual void paintGL();
			virtual void resizeGL(int w, int h);

			virtual void hideEvent(QHideEvent*);
			virtual void showEvent(QShowEvent*);
		private:
			void drawAnimatedObjects();
			void drawBall();
			void drawLandscape();
			void drawStaticObjects();
			void initializeObjects();

		private:
			QGLContext* m_context;
			Kolf::Scene* m_scene;
			Kolf::RenderStateManager* m_rsm;
			QTimer* m_timer;
	};
}


#endif //KOLF_VIEW3D_H
