/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "newgame-coursemodel.h"
#include "../engine/course.h"

#include <QTimer>

Kolf::NewGameCourseModel::NewGameCourseModel(QObject* parent)
	: QStandardItemModel(parent)
	, m_pendingCourses(Kolf::Course::availableCourses())
{
	//defer course parsing to next event loop round (to keep application responsive)
	if (!m_pendingCourses.isEmpty())
		QTimer::singleShot(0, this, SLOT(parseCourse()));
}

void Kolf::NewGameCourseModel::parseCourse()
{
	//parse one course
	const QString courseIdentifier = m_pendingCourses.takeFirst();
	const QString courseTitle = Kolf::Course::courseTitle(courseIdentifier);
	//create item
	QStandardItem* item = new QStandardItem;
	item->setData(courseTitle, Qt::DisplayRole);
	item->setData(courseIdentifier, Qt::UserRole); //we use the UserRole to store the internal identifier
	item->setEditable(false);
	//add item to model (sort by course titles)
	int position = 0;
	while (position < rowCount())
	{
		const QString otherTitle = this->item(position, 0)->data(Qt::DisplayRole).toString();
		if (QString::localeAwareCompare(courseTitle, otherTitle) < 0)
			break;
		else
			++position;
	}
	insertRow(position, item);
	//defer further course parsing to next event loop round (to keep application responsive)
	if (!m_pendingCourses.isEmpty())
		QTimer::singleShot(0, this, SLOT(parseCourse()));
}

#include "newgame-coursemodel.moc"
