/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "managedobjectgroup.h"
#include "managedobject.h"

#include <KConfigGroup>

Kolf::ManagedObjectGroup::ManagedObjectGroup(Kolf::ObjectFactory* objectFactory)
	: m_objectFactory(objectFactory)
{
	connect(this, SIGNAL(substatePushed(Utils::UndoStack*, int)), this, SLOT(slotSubstatePushed(Utils::UndoStack*, int)));
}

QList<Kolf::ManagedObject*> Kolf::ManagedObjectGroup::managedObjects() const
{
	QList<Kolf::ManagedObject*> result;
	foreach (Utils::UndoStack* stack, stacks())
		result << qobject_cast<Kolf::ManagedObject*>(stack);
	return result;
}

void Kolf::ManagedObjectGroup::serialize(KConfigGroup* group)
{
	static const QString objectGroupCaption = QLatin1String("Object %1");
	const QList<Kolf::ManagedObject*> managedObjects = this->managedObjects();
	for (int i = 0; i < managedObjects.count(); ++i)
	{
		KConfigGroup objectGroup(group, objectGroupCaption.arg(i + 1));
		managedObjects[i]->serialize(&objectGroup);
	}
}

void Kolf::ManagedObjectGroup::deserialize(KConfigGroup* group)
{
	static const QString objectGroupCaption = QLatin1String("Object %1");
	const QStringList groupList = group->groupList();
	for (int i = 1; groupList.contains(objectGroupCaption.arg(i)); ++i)
	{
		KConfigGroup objectGroup(group, objectGroupCaption.arg(i));
		Kolf::ManagedObject* managedObject = Kolf::ManagedObject::deserialize(&objectGroup, m_objectFactory);
		addStack(managedObject);
		connect(managedObject, SIGNAL(objectCreated(Kolf::ManagedObject*)), this, SIGNAL(objectCreated(Kolf::ManagedObject*)));
		connect(managedObject, SIGNAL(objectAboutToBeDeleted(Kolf::ManagedObject*)), this, SIGNAL(objectAboutToBeDeleted(Kolf::ManagedObject*)));
	}
	setClean();
}

void Kolf::ManagedObjectGroup::slotSubstatePushed(Utils::UndoStack* stack, int index)
{
	Q_UNUSED(index)
	const QList<Kolf::ManagedObject*> managedObjects = this->managedObjects();
	//allow state merging only for the stack with the newest change
	foreach (Kolf::ManagedObject* managedObject, managedObjects)
		managedObject->setAllowStateMerging(stack == managedObject);
}

#include "managedobjectgroup.moc"
