/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-circleblock.h"
#include "objectfactory.h"
#include "overlay-circle.h"
#include "shape-circle.h"

#include <QBrush>
#include <QPen>

REGISTER_KOLF_OBJECT(Kolf::CircleBlockObject, {
	classObject->screenName = ki18n("Circular block");
	classObject->groupName = QLatin1String("02-basic-obstacles");
})

Kolf::CircleBlockObject::CircleBlockObject(b2World* world)
	: Kolf::Object(world)
	, m_shape(new Kolf::CircleShape(1))
	, m_item(new QGraphicsEllipseItem(QRectF(-1, -1, 2, 2), this))
{
	m_item->setPen(QPen(Qt::gray));
	m_item->setBrush(Qt::gray);
	addShape(m_shape);
}

qreal Kolf::CircleBlockObject::radius() const
{
	return m_shape->radius();
}

void Kolf::CircleBlockObject::setRadius(qreal radius)
{
	if (m_shape->radius() != radius)
	{
		QRectF rect(0, 0, 2 * radius, 2 * radius);
		rect.moveCenter(QPointF());
		m_item->setRect(rect);
		m_shape->setRadius(radius);
		propagateUpdate();
	}
}

Kolf::Overlay* Kolf::CircleBlockObject::createOverlay()
{
	return new Kolf::CircleOverlay(this);
}

#include "object-circleblock.moc"
