/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OBJECT_COURSE_H
#define KOLF_OBJECT_COURSE_H

#include "object.h"

namespace Kolf
{
	class BallObject;
	class WallObject;

	//This object represents some global properties of the course:
	// * The position of this object is the point where players should spawn the balls.
	// * It saves the course size and whether the course has a boundary.
	class CourseObject : public Kolf::Object
	{
		Q_OBJECT
		Q_PROPERTY(QSizeF Size READ size WRITE setSize)
		Q_PROPERTY(bool Boundary READ hasBoundary WRITE setBoundary)
		public:
			CourseObject(b2World* world);

			bool hasBoundary() const;
			void setBoundary(bool hasBoundary);
			QSizeF size() const;
			void setSize(const QSizeF& size);

			Kolf::BallObject* createBallObject();
		protected:
			Kolf::Overlay* createOverlay();
			void updateBoundaryPosition(const QSizeF& size);
		private:
			bool m_hasBoundary;
			QList<Kolf::WallObject*> m_wallObjects;
	};
}

#endif // KOLF_OBJECT_COURSE_H
