/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "overlay-circle.h"
#include "object.h"
#include "overlayhandle.h"
#include "utils-pointf.h"

#include <cmath>

Kolf::CircleOverlay::CircleOverlay(Kolf::Object* object)
	: Kolf::Overlay(object)
	, m_handle(new Kolf::OverlayHandle(Kolf::OverlayHandle::CircleShape, this))
{
	addHandle(m_handle);
	connect(m_handle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	m_handle->setPos(1.0, 0.0);
}

void Kolf::CircleOverlay::update()
{
	Kolf::Overlay::update();
	const qreal radius = object()->property("Radius").value<qreal>();
	const QPointF oldPos = m_handle->pos();
	const QPointF direction = oldPos / sqrt(oldPos * oldPos);
	m_handle->setPos(direction * radius);
}

void Kolf::CircleOverlay::moveHandle(const QPointF& handleScenePos)
{
	QPointF newPosition = mapFromScene(handleScenePos);
	if (newPosition.isNull())
		newPosition = QPointF(Kolf::Overlay::MinimumObjectDimension, 0.0);
	//ensure minimum radius
	qreal newRadius = sqrt(newPosition * newPosition);
	if (newRadius < Kolf::Overlay::MinimumObjectDimension)
	{
		newPosition *= Kolf::Overlay::MinimumObjectDimension / newRadius;
		newRadius = Kolf::Overlay::MinimumObjectDimension;
	}
	//set new radius
	m_handle->setPos(newPosition);
	object()->setProperty("Radius", newRadius);
}

#include "overlay-circle.moc"
