/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "overlay-rectangle.h"
#include "object.h"
#include "overlayhandle.h"

Kolf::RectangleOverlay::RectangleOverlay(Kolf::Object* object)
	: Kolf::Overlay(object)
	, m_topLeftHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_topRightHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_bottomLeftHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_bottomRightHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_topHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_leftHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_rightHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_bottomHandle(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
{
	addHandle(m_topLeftHandle);
	addHandle(m_topRightHandle);
	addHandle(m_bottomLeftHandle);
	addHandle(m_bottomRightHandle);
	addHandle(m_topHandle);
	addHandle(m_leftHandle);
	addHandle(m_rightHandle);
	addHandle(m_bottomHandle);
	connect(m_topLeftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_topRightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomLeftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomRightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_topHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_leftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_rightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
}

void Kolf::RectangleOverlay::update()
{
	Kolf::Overlay::update();
	const QSizeF size = object()->property("Size").value<QSizeF>();
	QRectF rect(QPointF(), size);
	rect.moveCenter(QPointF());

	m_topLeftHandle->setPos(rect.topLeft());
	m_topRightHandle->setPos(rect.topRight());
	m_bottomLeftHandle->setPos(rect.bottomLeft());
	m_bottomRightHandle->setPos(rect.bottomRight());
	m_topHandle->setPos(QPointF(0.0, rect.top()));
	m_bottomHandle->setPos(QPointF(0.0, rect.bottom()));
	m_leftHandle->setPos(QPointF(rect.left(), 0.0));
	m_rightHandle->setPos(QPointF(rect.right(), 0.0));
}

void Kolf::RectangleOverlay::moveHandle(const QPointF& handleScenePos)
{
	const Kolf::OverlayHandle* handle = qobject_cast<Kolf::OverlayHandle*>(sender());
	if (!handle)
		return;
	const QPointF difference(mapFromScene(handleScenePos) - handle->pos());
	const qreal minSpread = Kolf::Overlay::MinimumObjectDimension;
	//read size
	QSizeF size = object()->property("Size").value<QSizeF>();
	//changes on left or right side
	if (handle == m_topLeftHandle || handle == m_leftHandle || handle == m_bottomLeftHandle)
		size.rwidth() -= difference.x();
	else if (handle == m_topRightHandle || handle == m_rightHandle || handle == m_bottomRightHandle)
		size.rwidth() += difference.x();
	if (size.width() < minSpread)
		size.setWidth(minSpread);
	//changes on top or bottom side
	if (handle == m_topLeftHandle || handle == m_topHandle || handle == m_topRightHandle)
		size.rheight() -= difference.y();
	else if (handle == m_bottomLeftHandle || handle == m_bottomHandle || handle == m_bottomRightHandle)
		size.rheight() += difference.y();
	if (size.height() < minSpread)
		size.setHeight(minSpread);
	//save size
	object()->setProperty("Size", qVariantFromValue(size));
}

#include "overlay-rectangle.moc"
