/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "overlay-wall.h"
#include "object.h"
#include "overlayhandle.h"
#include "utils-pointf.h"

#include <cmath>

Kolf::WallOverlay::WallOverlay(Kolf::Object* object)
	: Kolf::Overlay(object)
	, m_handle1(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
	, m_handle2(new Kolf::OverlayHandle(Kolf::OverlayHandle::SquareShape, this))
{
	addHandle(m_handle1);
	addHandle(m_handle2);
	connect(m_handle1, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_handle2, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
}

void Kolf::WallOverlay::update()
{
	Kolf::Overlay::update();
	const QPointF extent = object()->property("Extent").value<QPointF>();
	m_handle1->setPos(-extent);
	m_handle2->setPos(extent);
}

void Kolf::WallOverlay::moveHandle(const QPointF& handleScenePos)
{
	QPointF handlePos = mapFromScene(handleScenePos);
	const QObject* handle = sender();
	//get handle positions
	QPointF handle1Pos = m_handle1->pos();
	QPointF handle2Pos = m_handle2->pos();
	if (handle == m_handle1)
		handle1Pos = handlePos;
	else if (handle == m_handle2)
		handle2Pos = handlePos;
	//ensure minimum length
	static const qreal minLength = Kolf::Overlay::MinimumObjectDimension;
	const QPointF positionDifference = handle1Pos - handle2Pos;
	const qreal length = sqrt(positionDifference * positionDifference);
	if (length < minLength)
	{
		const QPointF additionalExtent = positionDifference * (minLength / length - 1);
		if (handle == m_handle1)
			handle1Pos += additionalExtent;
		else if (handle == m_handle2)
			handle2Pos -= additionalExtent;
	}
	//convert to extent/center
	const QPointF extent = (handle2Pos - handle1Pos) / 2;
	const QPointF center = object()->pos() + (handle2Pos + handle1Pos) / 2;
	object()->setProperty("Extent", qVariantFromValue(extent));
	object()->setPos(center);
}

#include "overlay-wall.moc"
