/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "shape-circle.h"

#include <Box2D/Collision/Shapes/b2CircleShape.h>

Kolf::CircleShape::CircleShape(qreal radius)
	: m_radius(radius)
{
	update();
}

qreal Kolf::CircleShape::radius() const
{
	return m_radius;
}

void Kolf::CircleShape::setRadius(qreal radius)
{
	if (m_radius == radius)
		return;
	m_radius = radius;
	update();
}

b2Shape* Kolf::CircleShape::createShape()
{
	b2CircleShape* shape = new b2CircleShape;
	shape->m_radius = m_radius;
	return shape;
}

void Kolf::CircleShape::createOutlines()
{
	const qreal activationRadius = m_radius + Kolf::Shape::ActivationOutlinePadding;
	const QPointF center(0.0, 0.0);
	//interaction outline
	QPainterPath interactionOutline;
	interactionOutline.addEllipse(center, m_radius, m_radius);
	setInteractionOutline(interactionOutline);
	//interaction outline
	QPainterPath activationOutline;
	activationOutline.addEllipse(center, activationRadius, activationRadius);
	setActivationOutline(activationOutline);
}
