/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "shape-wall.h"

#include <Box2D/Collision/Shapes/b2PolygonShape.h>

Kolf::WallShape::WallShape(const QPointF& extent)
	: m_extent(extent)
{
	update();
}

QPointF Kolf::WallShape::extent() const
{
	return m_extent;
}

void Kolf::WallShape::setExtent(const QPointF& extent)
{
	if (m_extent == extent)
		return;
	m_extent = extent;
	update();
}

b2Shape* Kolf::WallShape::createShape()
{
	b2Vec2 extentVector(m_extent.x(), m_extent.y());
	b2PolygonShape* shape = new b2PolygonShape;
	shape->SetAsEdge(extentVector, -extentVector);
	return shape;
}

void Kolf::WallShape::createOutlines()
{
	const qreal extentAngle = atan2(m_extent.y(), m_extent.x());
	const qreal paddingAngle = extentAngle + M_PI / 2;
	const qreal padding = Kolf::Shape::ActivationOutlinePadding;
	const QPointF paddingVector(padding * cos(paddingAngle), padding * sin(paddingAngle));
	//create interaction outline (a rectangle that is aligned with the wall)
	QPainterPath interactionOutline;
	interactionOutline.moveTo(m_extent + paddingVector);
	interactionOutline.lineTo(m_extent - paddingVector);
	interactionOutline.lineTo(-m_extent - paddingVector);
	interactionOutline.lineTo(-m_extent + paddingVector);
	interactionOutline.closeSubpath();
	setInteractionOutline(interactionOutline);
	//create activation outline (the same rectangle with half-circles at the end)
	QPainterPath activationOutline;
	activationOutline.moveTo(m_extent + paddingVector);
	activationOutline.arcTo(
		QRectF(m_extent.x() - padding, m_extent.y() - padding, 2 * padding, 2 * padding),
		-paddingAngle * 180.0 / M_PI, 180.0
	);
	activationOutline.lineTo(-m_extent - paddingVector);
	activationOutline.arcTo(
		QRectF(-m_extent.x() - padding, -m_extent.y() - padding, 2 * padding, 2 * padding),
		180.0 - paddingAngle * 180.0 / M_PI, 180.0
	);
	activationOutline.closeSubpath();
	setActivationOutline(activationOutline);
}
