/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_BYTEARRAYCODEC_H //krazy:excludeall=includes
#define KOLF_BYTEARRAYCODEC_H

#include <QVariant>

namespace Utils
{
	/**
	 * \namespace ByteArrayCodec
	 *
	 * The rationale behind this namespace is that QVariant does not support all the conversions that we need, such as QByteArray <-> QSizeF/QPointF. These conversions *are* supported by KConfig if we know the target type when the value is retrieved from the config file, but the design of the ManagedObject framework does not allow to determine this target type at this very moment.
	 *
	 * The ByteArrayCodec solves this dilemma by writing values into a temporary KConfig file and immediately reading them again in order to do the conversions when enough information is available about the conversion.
	 */
	namespace ByteArrayCodec
	{
		QVariant decode(const QByteArray& value, const QVariant& defaultValue);
		QByteArray encode(const QVariant& value);
	}
}

#endif // KOLF_BYTEARRAYCODEC_H
