/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GAME_H
#define KOLF_GAME_H

#include <QObject>

namespace Kolf
{
	class Course;
	class Hole;
	class Interface;
	class Player;
	class Scene;
	class ScoreModel;

	/**
	 * \class Game
	 * \since 2.0
	 *
	 * The central game management class which reacts on the movements of the ball and turns control to the players.
	 * Subclasses extend the base implementation to do the actual actions needed to make it a game (i.e. player control management and such).
	 */
	class Game : public QObject
	{
		//TODO: score management
		Q_OBJECT
		public:
			///\note This class takes ownership of the course, but not the interface.
			Game(Kolf::Course* course, Kolf::Interface* interface);
			virtual ~Game();

			///\note The current hole is -1 when the Game is created, which indicates that no hole has been selected. The game has to be started by setCurrentHole(0).
			int currentHoleIndex() const;
			Kolf::Course* course() const;
			Kolf::Interface* interface() const;
			QList<Kolf::Player*> players() const;
			Kolf::ScoreModel* scoreModel() const;
		public Q_SLOTS:
			void setCurrentHoleIndex(int holeIndex);
		Q_SIGNALS:
			void finished();
		protected:
			///Subclasses may expose public interfaces to this method.
			///\note This class takes ownership of the player.
			void addPlayer(Kolf::Player* player);
			///A convenience shortcut for course()->holes().value(currentHoleIndex()).
			Kolf::Hole* currentHole() const;

			///This hook is called after a new hole has been selected. The scene for this hole has already been set, and you may do some further initialisation.
			virtual void bootHole(Kolf::Hole* hole) = 0;
			///This hook is called before a new hole is selected. The scene for this hole is still active on the view, and you may do some custom cleanup.
			virtual void shutdownHole(Kolf::Hole* hole) = 0;
		private:
			int m_currentHoleIndex;
			Kolf::Course* m_course;
			Kolf::Interface* m_interface;
			QList<Kolf::Player*> m_players;
			Kolf::ScoreModel* m_scoreModel;
	};
}

#endif // KOLF_GAME_H
