/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_PLAYER_H
#define KOLF_PLAYER_H

#include <QObject>
#include <QPointer>

namespace Kolf
{
	class BallObject;
	class Putter;
	class Scene;

	/**
	 * \class Player
	 * \since 2.0
	 *
	 * Representation for a player, which manages scores and passes control to this player. One Kolf::Player instance may only be used with one Kolf::Game instance. This class is an abstract base class, the instantiable subclasses represent different types of players.
	 */
	class Player : public QObject
	{
		Q_OBJECT
		public:
			Player(const QString& name, int hue);
			virtual ~Player();

			///\note The ball is regularly deleted (e.g. when the scene changes). In this case, the next call to this getter will create a new Kolf::BallObject, unless you set \a createIfNecessary to false.
			Kolf::BallObject* ball(bool createIfNecessary = true) const;
			Kolf::Putter* putter() const;
			Kolf::Scene* scene() const;
			virtual bool hasControl() const = 0;

			int hue() const;
			QString name() const;
		Q_SIGNALS:
			void ballObjectCreated(Kolf::BallObject* ballObject);
		public Q_SLOTS:
			///The subclass implementation of this slot should enable the putting interface (for human players) or do the putting (for computer players).
			virtual void giveControl() = 0;
			///The subclass implementation of this slot should disable the putting interface for human players if it's active.
			virtual void takeControl() = 0;
			///This slot will change the scene of this player.
			///\warning This will delete the ball on the old scene.
			void setScene(Kolf::Scene* scene, Kolf::Putter* putter);
		private:
			int m_hue;
			QString m_name;

			mutable QPointer<Kolf::BallObject> m_ball;
			Kolf::Putter* m_putter;
			Kolf::Scene* m_scene;
	};
}

#endif // KOLF_PLAYER_H
