/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_SCOREMODEL_H
#define KOLF_SCOREMODEL_H

#include <QAbstractTableModel>
#include <QVector>

namespace Kolf
{
	class Game;
	class Hole;
	class NormalGame;
	class Player;

	/**
	 * \class ScoreModel
	 * \since 2.0
	 *
	 * This model displays the scores of a certain game.
	 */
	class ScoreModel : public QAbstractTableModel
	{
		public:
			///This will only make the parscores editable.
			void setEditable(bool editable);

			virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
			virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
			virtual Qt::ItemFlags flags(const QModelIndex& index) const;
			virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
			virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
			virtual bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
		protected:
			friend class Kolf::Game;
			friend class Kolf::NormalGame;
			///The par scores are preloaded.
			ScoreModel(Kolf::Game* game);
			///This method assumes that the player has been added at the end of the player list.
			void announcePlayerAdded();
			void announceCurrentHoleChanged();
			///\param player give null pointer to retrieve par score
			int score(Kolf::Hole* hole, Kolf::Player* player) const;
			///\param player give null pointer to adjust par score
			void setScore(Kolf::Hole* hole, Kolf::Player* player, int score);
		private:
			Kolf::Game* m_game;
			bool m_editable;
			int m_currentHoleIndex;
			QList<QVector<int> > m_scores; //key for list: hole index (starting from 0), key for vector: player index (starting from 1, 0 = par score)
			QVector<int> m_totalScores; //key is player index (same like above)
	};
}

#endif // KOLF_SCOREMODEL_H
