#include "KGGameData.h"
#include <KDebug>
//#include <QFileList>

//**************************************
//QStandardProjectItem
//**************************************
QStandardProjectItem::QStandardProjectItem( const QIcon & icon, const QString & text ) :
QStandardItem ( icon,text )
    {}
//******************************************************************************
QString QStandardProjectItem::reference()
    { return m_reference; }
//******************************************************************************
KGTabInterface::interfaceType QStandardProjectItem::ifaceType() const
    { return m_type; }
//******************************************************************************
void QStandardProjectItem::setReference(QString reference)
    { m_reference = reference; }
//******************************************************************************
void QStandardProjectItem::setIfaceType(KGTabInterface::interfaceType type)
    { m_type = type; }
//******************************************************************************

//**************************************
//KGGameData
//**************************************

KGGameData::KGGameData()
{
    m_selectedItem = NULL;
    m_newLevelCounter = 0;
    m_friction = 10.0;
    m_gravity = 10.0;
    m_running = true;
    m_jumping = true;
    m_projectItem = new QStandardItem(KIcon("folder-open"),"Project");
    m_projectItem->setEditable(false);
    setItem(0,0,m_projectItem);

    m_levelItem = new QStandardItem(KIcon("folder-open"),"Levels");
    m_levelItem->setEditable(false);
    m_projectItem->setChild(0,0,m_levelItem);

    connect(this,SIGNAL(itemChanged(QStandardItem *)),this,SLOT(renameItem(QStandardItem *)));
}
//******************************************************************************
void KGGameData::renameItem(QStandardItem *item)
{
    if(item != m_projectItem && item != m_levelItem)
    {
	KUrl location = ((QStandardProjectItem *)item)->reference();
	QString newName = item->text();
	if(doesNameExist(newName,location))
	    m_duplicateName = true;
	else
	{
	    m_duplicateName = false;
	    //rename the file
	    QFile::rename(location.path(),location.directory() + "/" + newName + ".KGL"); 
	    //rename the tab
	    emit itemRenamed(location.path(),location.directory() + "/" + newName + ".KGL",newName);
	    //rename the reference
	    ((QStandardProjectItem *)item)->setReference(location.directory() + "/" + newName + ".KGL");
	}
    }
}
//******************************************************************************
QString KGGameData::itemSelectedReference()
    { return m_selectedItem->reference(); }
//******************************************************************************
bool KGGameData::isItemSelected(void)
    { return (m_selectedItem != NULL); }
//******************************************************************************
QStandardItem *KGGameData::selectedItem(void)
    { return m_selectedItem; }
//******************************************************************************
bool KGGameData::duplicateName(void)
    { return m_duplicateName; }
//******************************************************************************
void KGGameData::selectionChanged(QItemSelection selected)
{
    //handles selection change
    kDebug() << "SelectionChanged" << endl;
    if(selected.indexes().count() > 0)
    {
	QStandardItem *item = itemFromIndex(selected.indexes().back());
	if(item != m_projectItem && item != m_levelItem)
	    m_selectedItem = (QStandardProjectItem *)item;
    }
}   
//******************************************************************************
bool KGGameData::saveLevel(KGLevelEditor *editor)
{
    editor->save();
    return true;
}
//******************************************************************************
bool KGGameData::doesNameExist(QString name, KUrl currentUrl)
{
    if(currentUrl.path() == m_location.path() + "/" + name + ".KGL")
	return false;
    if(!KUrl(m_location.path() + "/" + name + ".KGL").isValid())
	return true;
    return QFile::exists(m_location.path() + "/" + name + ".KGL"); 
}
KGTabInterface *KGGameData::openLevel(QString reference)
{
    KGLevelEditor *level = new KGLevelEditor();
    level->setLocation(reference);
    level->setName(reference.section("/",-1).section(".",-2,0));
    level->setType(KGTabInterface::PT_Level);
    level->open();
    return level;
}
KGTabInterface *KGGameData::newLevel()
{
    //create default name
    QString newLevelName;
    m_newLevelCounter++;
    newLevelName = "level " + QString::number(m_newLevelCounter);
    while(doesNameExist(newLevelName,KUrl()))
    {
	m_newLevelCounter++;
	newLevelName = "level " + QString::number(m_newLevelCounter);
    }
    
    //create level
    KGLevelEditor *level = new KGLevelEditor();
    level->setLocation(m_location.path() + "/" + newLevelName + ".KGL");
    level->setName(newLevelName);
    level->setType(KGTabInterface::PT_Level);
    level->save();
    
    //add item to tree view
    QStandardProjectItem *item = new QStandardProjectItem(KIcon("document"),level->name());
    item->setReference(level->location().path());
    item->setIfaceType(KGTabInterface::PT_Level);
    m_levelItem->setChild(m_levelItem->rowCount(),item);

    return level;
}
bool KGGameData::openGame(QString gameFile)
{
    KGContainer *container = KGSelectionInterface::instance()->container();
    QFile openGame(gameFile);
    QDomDocument doc;

    openGame.open(QIODevice::ReadWrite);
    doc.setContent(&openGame);
    openGame.close();
    QDomElement element = doc.documentElement();

    QDomElement locationElement = element.firstChildElement("game");
    m_name = locationElement.attribute("name",QString());
    m_projectItem->setText(m_name);
    m_jumping = locationElement.attribute("jumping","1").toInt();
    m_running = locationElement.attribute("running","1").toInt();
    m_gravity = locationElement.attribute("gravity","10").toInt();
    m_friction = locationElement.attribute("friction","10").toInt();
    m_location = gameFile.section("/",0,-2);
    QDir directory(m_location.path());
    QStringList filters;
    filters.push_front("*.KGL");
    foreach(QFileInfo fileInfo,directory.entryInfoList(filters))
    {
	QString levelName = fileInfo.fileName().section(".",0,-2);
	QStandardProjectItem *item = new QStandardProjectItem(KIcon("document"),levelName);
	item->setReference(fileInfo.filePath());
	item->setIfaceType(KGTabInterface::PT_Level);
	m_levelItem->setChild(m_levelItem->rowCount(),item);
    }
    return true;
}
bool KGGameData::saveGame(void)
{
    QDir dir;
    if(!dir.mkpath(m_location.path()))
	return false;

    QFile gameFile(m_location.path() + "/kolladev.KGP");
    gameFile.open(QIODevice::ReadWrite | QIODevice::Truncate);
    
    QDomDocument doc("KGP");
    QDomElement root = doc.createElement("KGP");
    doc.appendChild(root);

    QDomElement locationElement = doc.createElement("game");
    locationElement.setAttribute("name",m_name);
    QDomElement jumpingElement = doc.createElement("jumping");
    jumpingElement.setAttribute("value",m_jumping);
    QDomElement runningElement = doc.createElement("running");
    runningElement.setAttribute("value",m_running);
    QDomElement gravityElement = doc.createElement("gravity");
    gravityElement.setAttribute("value",m_gravity);
    QDomElement frictionElement = doc.createElement("friction");
    frictionElement.setAttribute("value",m_friction);

    locationElement.appendChild(jumpingElement);
    locationElement.appendChild(runningElement);
    locationElement.appendChild(gravityElement);
    locationElement.appendChild(frictionElement);
    root.appendChild(locationElement);
    QTextStream textStream(&gameFile);
    textStream << doc.toString();
    gameFile.close();
    return true;
}
//******************************************************************************
KUrl KGGameData::location(void)
    { return m_location; }
//******************************************************************************
void KGGameData::setLocation(KUrl location)
    { m_location = location; }
//******************************************************************************
QString KGGameData::name(void)
    { return m_name; }
//******************************************************************************
void KGGameData::setName(QString name)
{ 
    m_projectItem->setText(name);
    m_name = name; 
}
//******************************************************************************
QString KGGameData::introText(void)
    { return m_introText; }
//******************************************************************************
void KGGameData::setIntroText(QString introText)
    { m_introText = introText; }
//******************************************************************************
bool KGGameData::runnning(void)
    { return m_running; }
//******************************************************************************
void KGGameData::setRunning(bool running)
    { m_running = running; }
//******************************************************************************
bool KGGameData::jumping(void)
    { return m_jumping; }
//******************************************************************************
void KGGameData::setJumping(bool jumping)
    { m_jumping = jumping; }
//******************************************************************************
float KGGameData::gravity(void)
    { return m_gravity; }
//******************************************************************************
void KGGameData::setGravity(float gravity)
    { m_gravity = gravity; }
//******************************************************************************
float KGGameData::friction(void)
    { return m_friction; }
//******************************************************************************
void KGGameData::setFriction(float friction)
    { m_friction = friction; }
//******************************************************************************
bool KGGameData::scrollingLeft(void)
    { return m_scrollingData.m_left; }
//******************************************************************************
void KGGameData::setScrollingLeft(bool left)
    { m_scrollingData.m_left = left; }
//******************************************************************************
bool KGGameData::scrollingRight(void )
    { return m_scrollingData.m_right; }
//******************************************************************************
void KGGameData::setScrollingRight(bool right)
    { m_scrollingData.m_right = right; }
//******************************************************************************
bool KGGameData::scrollingUp(void)
    { return m_scrollingData.m_up; }
//******************************************************************************
void KGGameData::setScrollingUp(bool up)
    { m_scrollingData.m_up = up; }
//******************************************************************************
bool KGGameData::scrollingDown(void)
    { return m_scrollingData.m_down; }
//******************************************************************************
void KGGameData::setScrollingDown(bool down)
    { m_scrollingData.m_down = down; }
//******************************************************************************

#include "KGGameData.moc"
