#ifndef _KGGameData_H_
#define _KGGameData_H_
#include <QTextStream>
#include <QDomDocument>
#include <QStandardItemModel>
#include <QItemSelection>
#include <QString>
#include <KIcon>
#include <QFile>
#include <KUrl>
#include <QDir>
#include "kolladev.h"
#include "KGLevelEditor.h"
#include "KGTabInterface.h"

class QStandardProjectItem : public QStandardItem
{
    QString m_reference;
    KGTabInterface::interfaceType m_type;
public:
    QStandardProjectItem( const QIcon & icon, const QString & text );
    QString reference();
    KGTabInterface::interfaceType ifaceType() const;
    void setReference(QString reference);
    void setIfaceType(KGTabInterface::interfaceType type);
};

class KGGameData: public QStandardItemModel
{
    Q_OBJECT
    struct ScrollingData 
    {
	bool m_left;
	bool m_right;
	bool m_down;
	bool m_up;
	ScrollingData()
	{
	    m_left = true;
	    m_right = true;
	    m_down = true;
	    m_up = true;
	}
    };
    KUrl m_location;
    QString m_name;
    QString m_introText;
    bool m_duplicateName;
    bool m_running;
    bool m_jumping;
    float m_gravity;
    float m_friction;
    int m_newLevelCounter;
    QStandardItem *m_projectItem;
    QStandardItem *m_levelItem;
    QStandardProjectItem *m_selectedItem;
    ScrollingData m_scrollingData;
public:
    KGGameData();

    QString itemSelectedReference();
    bool isItemSelected(void);
    QStandardItem *selectedItem(void);
    bool duplicateName(void);
    void selectionChanged(QItemSelection selected);

    bool doesNameExist(QString name, KUrl location);
    bool rename(KUrl &location,QString newName);
    bool saveGame(void);
    bool openGame(QString fileName);

    KGTabInterface *openLevel(QString reference);
    bool saveLevel(KGLevelEditor *editor);
    KGTabInterface *newLevel(void);

    KUrl location(void);
    void setLocation(KUrl location);
    QString name(void);
    void setName(QString name);
    QString introText(void);
    void setIntroText(QString);
    bool runnning(void);
    void setRunning(bool running);
    bool jumping(void);
    void setJumping(bool jumping);
    float gravity(void);
    void setGravity(float gravity);
    float friction(void);
    void setFriction(float friction);
    bool scrollingLeft(void);
    void setScrollingLeft(bool left);
    bool scrollingRight(void);
    void setScrollingRight(bool right);
    bool scrollingUp(void);
    void setScrollingUp(bool up);
    bool scrollingDown(void);
    void setScrollingDown(bool down);
public slots:
    void renameItem(QStandardItem *item);
signals:
    void itemRenamed(QString oldReference, QString newReference, QString newName);
};

#endif
