/*
 * KGLevelEditor.h
 *
 * Copyright (C) 2007 Keith Neuse <KNeuse@Gmail.com>
 */ 
#ifndef _KLEVELEDITOR_H_
#define _KLEVELEDITOR_H_

#include <QGraphicsView>
#include <QGraphicsItem>
#include <QDomDocument>
#include <QTextStream>
#include <QList>
#include "KGObject.h"
#include "KGTabInterface.h"
#include "KGSelectionInterface.h"
#include "kolladev.h"
#include <QPainter>
#include <QPaintEvent>
#include <QTimer>
#include <QFile>

#define TILE_ITEM 1
#define PARENT_ITEM 0
#define KEY_TYPE 1001
#define NULL_LAYER -1

#define MOUSE_ID -2

struct KGLayerInfo {
    int layerID;
    int layerNum;
    bool hiden;
    QGraphicsItemGroup *layerObject;
};

class KGLevelEditor: public QGraphicsView, public KGTabInterface
{
    Q_OBJECT
protected:
    QGraphicsItemGroup *layerGroup;
    QList<KGLayerInfo> layerList;
    QGraphicsScene scene;
    KGObject *mainTile;
    QString reference;
    int mouseX;
    int eraserMode;
    int mouseY;
    bool mousePressed;
    int mouseLayer;
    int workingLayer;
public:
    QList<KGObject *> getObjects(int layerID);
    /**
	constructor 
	@param parent = the parent widget
	@param name = the name for the widget (Currently not implimented)
    **/
    KGLevelEditor(QWidget * parent=0, const char* name=0);

    /**
	constructor 
	@param newTile = sets up the mouse with this tile which is used for placing objects in the level
	@param parent = the parent widget
	@param name = the name for the widget (Currently not implimented)
    **/
    /** deconstructor **/
    virtual ~KGLevelEditor();

    /**
	hide/show a layer
	@param layerID = the layer ID to hide/show
	@param hide = true = hide | false = show
    **/
    void setLayerHide(int layerID, bool hide);

    /**
	sets the current layer for adding objects to
	@param layerID = the layer ID to be working with
    **/
    
    /**
	This adds a new layer with the first add layer being draw first
	@PARAM layerID = the layer ID for this layer
    **/
    int addLayer(int layerID);

    /**
	sets the levelEditor into eraser mode
	@param argEraserMode = true = eraser tiles | false = place tiles
    **/
    

    void addObject(KGObject *object, int layerID, QPoint pos);

    void setReference(QString newReference);
    QString getReference();
    bool save(void);
    bool open(void);
protected:
    int findLayer(int layerID);
    void initGraphicsView(void);
    void initLayers(void);
    void placeObject(QPoint mousePos);

    //events
    void mousePressEvent ( QMouseEvent * e );
    void mouseReleaseEvent ( QMouseEvent * e );
    void mouseMoveEvent ( QMouseEvent * e );
public slots:
    /**
	sets the mouse to this object which is then used for placing objects in the level
	@param the object to be added
    **/
    void setWorkingLayer(int layerID);
    void setEraserMode(bool argEraserMode);
    void setCurrentObject(KGObject *newTile, int layerID);
};

#endif
