#include "KGNewGameWizard.h"

#include <QStringList>
#include <KDebug>
//**************************************
//KGGameSummaryPage
//**************************************
KGGameSummaryPage::KGGameSummaryPage()
{ 
    newGamePage4Widget.setupUi(this); 
    newGamePage4Widget.kurlLocation->setMode(KFile::Directory);
}
//******************************************************************************
void KGGameSummaryPage::setSummary(QString summary)
    { newGamePage4Widget.txtSummary->setText(summary); }
//******************************************************************************
KUrlRequester *KGGameSummaryPage::getLocation(void)
    { return newGamePage4Widget.kurlLocation; }

//**************************************
//KGGameInfoPage
//**************************************
KGGameInfoPage::KGGameInfoPage()
    { newGamePage3Widget.setupUi(this); }
//******************************************************************************
QLineEdit *KGGameInfoPage::getGameObject()
    { return newGamePage3Widget.txtGameName; }
//******************************************************************************
QString KGGameInfoPage::getGameName()
    { return newGamePage3Widget.txtGameName->text(); }
//******************************************************************************
QString KGGameInfoPage::getIntroText() 
    { return newGamePage3Widget.txtIntroText->toPlainText(); }

//**************************************
//KGGameSettingsPage
//**************************************
KGGameSettingsPage::KGGameSettingsPage()
{
    newGamePage2Widget.setupUi(this);
    connect(newGamePage2Widget.chkRunning,SIGNAL(clicked(bool)),this,SLOT(enableTxtFriction(bool)));
    connect(newGamePage2Widget.chkJumping,SIGNAL(clicked(bool)),this,SLOT(enableTxtGravity(bool)));
}
//******************************************************************************
QString KGGameSettingsPage::getFriction()
    { return newGamePage2Widget.txtFriction->text(); }
//******************************************************************************
QString KGGameSettingsPage::getGravity()
    { return newGamePage2Widget.txtGravity->text(); }
//******************************************************************************
bool KGGameSettingsPage::isRunningChecked()
    { return newGamePage2Widget.chkRunning->isChecked(); }
//******************************************************************************
bool KGGameSettingsPage::isJumpingChecked()
    { return newGamePage2Widget.chkJumping->isChecked(); }
//******************************************************************************
bool KGGameSettingsPage::isLeftChecked()
    { return newGamePage2Widget.chkLeft->isChecked(); }
//******************************************************************************
bool KGGameSettingsPage::isRightChecked()
    { return newGamePage2Widget.chkRight->isChecked(); }
//******************************************************************************
bool KGGameSettingsPage::isUpChecked()
    { return newGamePage2Widget.chkUp->isChecked(); }
//******************************************************************************
bool KGGameSettingsPage::isDownChecked()
    { return newGamePage2Widget.chkDown->isChecked(); }
//******************************************************************************
void KGGameSettingsPage::enableTxtFriction(bool state)
    { newGamePage2Widget.txtFriction->setEnabled(state); }
//******************************************************************************
void KGGameSettingsPage::enableTxtGravity(bool state)
    { newGamePage2Widget.txtGravity->setEnabled(state); }

//**************************************
//KGGameTypePage
//**************************************
KGGameTypePage::KGGameTypePage()
{
    newGamePage1Widget.setupUi(this);
    newGamePage1Widget.listView->setIconSize(QSize(256,256));
    
    QStandardItemModel *itemModel = new QStandardItemModel();
    
    newGamePage1Widget.listView->setSelectionBehavior( QAbstractItemView::SelectItems );
    newGamePage1Widget.listView->setSelectionMode( QAbstractItemView::SingleSelection );
    newGamePage1Widget.listView->setMovement( QListView::Static );
    newGamePage1Widget.listView->setFlow(QListView::LeftToRight);
    newGamePage1Widget.listView->setWrapping ( false );
    
    QFont boldFont( font() );
    boldFont.setBold( true );
    setFont( boldFont );

    QList<QStandardItem *> items;
    QStandardItem *item;
    item = new QStandardItem(KIcon("draw-eraser"),"Top Down");
    item->setEditable(false);
    items.push_back(item);

    item = new QStandardItem(KIcon("draw-eraser"),"Side ways");
    item->setEditable(false);
    items.push_back(item);
    itemModel->insertColumn(0,items);
    
    newGamePage1Widget.listView->setModel(itemModel);
    //items.push_front(item);

    //itemModel->appendColumn(items);
    newGamePage1Widget.listView->setItemDelegate( new KPageListViewDelegate(newGamePage1Widget.listView  ) );
}

//**************************************
//KGNewGameWizard
//**************************************
KGNewGameWizard::KGNewGameWizard(QWidget *parent):
QWizard(parent)
{
    resize(600,450);

    //create pages
    summaryPage = new KGGameSummaryPage;
    infoPage = new KGGameInfoPage;
    settingsPage = new KGGameSettingsPage;
    
    setWindowTitle(tr("New Game"));

    //add pages
    setPage(GAMETYPE_ID, new KGGameTypePage);
    setPage(GAMESETTINGS_ID, settingsPage);
    setPage(GAMEINFO_ID, infoPage);
    setPage(GAMESUMMARY_ID, summaryPage);

    connect(this,SIGNAL(currentIdChanged(int)),this,SLOT(idChanged(int)));
    KUrlRequester *urlRequester = summaryPage->getLocation();
    QLineEdit *txtGameName = infoPage->getGameObject();
    if(urlRequester)
 	connect(urlRequester,SIGNAL(textChanged(const QString &)),this,SLOT(validateFinishButton()));
    if(txtGameName)
 	connect(txtGameName,SIGNAL(textChanged(const QString &)),this,SLOT(validateNextButton()));

    //set starting page
    setStartId(GAMETYPE_ID);
}
//******************************************************************************
KGGameData *KGNewGameWizard::getGameData(void)
{
    //retreive the game data
    KGGameData *gameData = new KGGameData();
    gameData->setName(infoPage->getGameName());
    gameData->setIntroText(infoPage->getIntroText());
    gameData->setScrollingLeft(!settingsPage->isLeftChecked());
    gameData->setScrollingRight(!settingsPage->isRightChecked());
    gameData->setScrollingUp(!settingsPage->isUpChecked());
    gameData->setScrollingDown(!settingsPage->isDownChecked());
    gameData->setJumping(settingsPage->isJumpingChecked());
    gameData->setGravity(settingsPage->getGravity().toFloat());
    gameData->setRunning(settingsPage->isRunningChecked());
    gameData->setFriction(settingsPage->getFriction().toFloat());
    KUrlRequester *urlRequester = summaryPage->getLocation();
    if(urlRequester)
	gameData->setLocation(urlRequester->url().url() + "/" + infoPage->getGameName());
    return gameData;
}
//******************************************************************************
void KGNewGameWizard::initializePage ( int id )
{
    if(GAMESUMMARY_ID == id)
    {
	QString summary;
	summary += "Game name: " + infoPage->getGameName() + "\r\n";
	if(settingsPage->isLeftChecked() && settingsPage->isRightChecked() &&
	   settingsPage->isUpChecked() && settingsPage->isDownChecked())
	    summary += "No Scrolling allowed\r\n";
	else
	{
	    summary += "Scrolling allowed for the following directions: \r\n";
	    if(!settingsPage->isLeftChecked())
		summary += "- Left\r\n";
	    if(!settingsPage->isRightChecked())
		summary += "- Right\r\n";
	    if(!settingsPage->isUpChecked())
		summary += "- Up\r\n";
	    if(!settingsPage->isDownChecked())
		summary += "- Down\r\n";
	}
	if(settingsPage->isJumpingChecked())
	{
		summary += "Jumping: allowed\r\n";
		summary += " - Gravity: " + QString::number(settingsPage->getGravity().toFloat()) + "\r\n";
	}
	else
		summary += "Jumping: disabled\r\n";
	if(settingsPage->isRunningChecked())
	{
		summary += "Running: allowed\r\n";
		summary += " - Friction: " + QString::number(settingsPage->getFriction().toFloat()) + "\r\n";
	}
	else
		summary += "Running: disabled\r\n";
	summaryPage->setSummary(summary);
    }
}
void KGNewGameWizard::validateNextButton()
{
    if(currentId() == GAMEINFO_ID)
    {
	if(infoPage->getGameName() == "")
	    button(QWizard::NextButton)->setEnabled(false);
	else
	    button(QWizard::NextButton)->setEnabled(true);
    }
    else
	button(QWizard::NextButton)->setEnabled(true);
}
void KGNewGameWizard::validateFinishButton()
{
    button(QWizard::FinishButton)->setEnabled(false);
    KUrlRequester *urlRequester = summaryPage->getLocation();
    if(urlRequester && infoPage->getGameName() != "")
    {
	kDebug() << urlRequester->url().url() + "/" + infoPage->getGameName() << endl;
	KUrl url(urlRequester->url().url() + "/" + infoPage->getGameName());
	if(url.isValid() && urlRequester->url().isValid())
	    button(QWizard::FinishButton)->setEnabled(true);
    }
}
void KGNewGameWizard::idChanged(int newID)
{
    if(newID == GAMESUMMARY_ID)
	validateFinishButton();
    if(newID == GAMEINFO_ID)
	validateNextButton();
}
#include "KGNewGameWizard.moc"
