#include "KGToolDrawer.h"

#include <KDebug>
KGToolDrawer::KGToolDrawer(int argLayerID, QWidget *parent, QString name):
QWidget(parent)
{
    QVBoxLayout *layout = new QVBoxLayout;

    btnAdd = new KPushButton("Add Tile",this);
    btnAdd->resize(100,30);
    btnAdd->move(width()/2-50,5);
    layout->addWidget(btnAdd);

    selectedObject = NULL;
    m_name = name;
    layerID = argLayerID;
    
    imageView = new KGImageView(this);
    imageView->resize(width(),height()-40);
    imageView->move(0,40);
    layout->addWidget(imageView);

    connect(btnAdd,SIGNAL(clicked()),this,SLOT(addTileFromFile()));
    connect(imageView,SIGNAL(selectionChanged(QGraphicsItem *)),this,SLOT(processSelectionChanged(QGraphicsItem *)));
    setLayout(layout);

    addDefaultObjects();
}
//******************************************************************************
KGObject *KGToolDrawer::getSelectedObject()
    { return selectedObject; }
//******************************************************************************
void KGToolDrawer::addDefaultObjects(void)
{
    KGContainer *container = KGSelectionInterface::instance()->container();
    container = container->findContainer(m_name);
    
    foreach(KGObject *object,container->objects())
    {
	objectList.push_back(object);
	imageView->addItem(object,object->name());
    }    
    if(objectList.size() > 0)
	selectedObject = objectList.front();
}
//******************************************************************************
QString KGToolDrawer::name()
    { return m_name; }
//******************************************************************************
void KGToolDrawer::saveObjects(QDomDocument &doc)
{
    //store panel data
    QDomElement panel = doc.createElement("panel");
    panel.setAttribute("name",m_name);

    //store tiles
    foreach(KGObject *object,objectList)
    {
	QDomElement objectElement = object->saveObject(doc);
	panel.appendChild(objectElement);
    }
    
    //append to document
    doc.documentElement().appendChild(panel);
}
//******************************************************************************
void KGToolDrawer::selectNone(void)
{
    //NOTE: implement me
}
//******************************************************************************
int KGToolDrawer::getLayerID()
    { return layerID; }
//******************************************************************************
void KGToolDrawer::addTileFromFile(void)
{
    //TODO: fix it so filenames don't conflict
    KUrl filename = KFileDialog::getOpenFileName(KUrl(),"*",this,"Open a tile");
    if(filename == NULL)
	return;

    KGTile *newTile = new KGTile(filename.path(),"tile");
    imageView->addItem(newTile,"tile");
}
//******************************************************************************
void KGToolDrawer::addObject(KGTile *addObject)
{
    objectList.push_back(addObject);
    imageView->addItem(addObject,addObject->name());
}
//******************************************************************************
//**************************************
//SLOTS
//**************************************
void KGToolDrawer::processSelectionChanged(QGraphicsItem *item)
    { KGSelectionInterface::instance()->setSelectedObject(qgraphicsitem_cast<KGObject *>(item)); }

#include "KGToolDrawer.moc"
