/*
 * developer.cpp
 *
 * Copyright (C) 2007 Keith Neuse <KNeuse@Gmail.com>
 */
#include "developer.h"

DEVELOPER::DEVELOPER(QWidget *parent, Qt::WindowFlags flags): KXmlGuiWindow(parent,flags)
{
    loadTiles();

    //Create tab windows and add level1 tab
    tabWidget = new KTabWidget(this);

    //add tab window as central widget
    setCentralWidget(tabWidget);

    initMainActions();
    initLayerToolbar();

    KGSelectionInterface::instance();

    //create a new dock widget that contains selection of items
    toolBoxDock = new QDockWidget(this);
    toolBox = new KGToolBox(toolBoxDock);
    toolBoxDock->setWidget(toolBox);
    toolBoxDock->hide();

    addDockWidget(Qt::LeftDockWidgetArea, toolBoxDock);

    projectDock = new QDockWidget(this);
    project = new KGProject(projectDock);
    projectDock->setWidget(project);
    projectDock->hide();

    addDockWidget(Qt::RightDockWidgetArea, projectDock);
    
    //setup toolbox
    connect(project,SIGNAL(tabSelected(QString)),this,SLOT(selectTab(QString)));
    connect(project,SIGNAL(itemRenamed(QString, QString, QString)),this,SLOT(renameTab(QString, QString, QString)));
    connect(KGSelectionInterface::instance(),SIGNAL(eraserModeChanged(bool)),this,SLOT(eraserModeChanged(bool)));
    connect(KGSelectionInterface::instance(),SIGNAL(layerIDChanged(int)),this,SLOT(changeLayerID(int)));
    
    QString resourceLocation = KGlobal::dirs()->findResource("data", "kollagame/kolladev/kolladev.rc");
    setupGUI(ToolBar | Create, resourceLocation);
}
//******************************************************************************
void DEVELOPER::initMainActions()
{
    QAction *action = NULL;
	
    KStandardAction::quit(qApp, SLOT(quit()), actionCollection());

    //new level
    action = new KAction(KIcon("document-new"), i18n("New &Game"), this);
    actionCollection()->addAction("new_game", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(newGame()));

    //new level
    action = new KAction(KIcon("document-open"), i18n("open &Game"), this);
    actionCollection()->addAction("open_game", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(openGame()));

    //new level
    action = new KAction(KIcon("tab-new"), i18n("&New Level"), this);
    actionCollection()->addAction("new_level", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(newLevel()));

    //save level
    action = new KAction(KIcon("document-save"), i18n("&Save Level"), this);
    actionCollection()->addAction("save_level", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(saveLevel()));

    //close level
    action = new KAction(KIcon("tab-close"), i18n("&Close Level"), this);
    actionCollection()->addAction("close_level", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(closeLevel()));

    //eraser
    eraserAction = new KToggleAction(KIcon("draw-eraser"), i18n("Eraser"), this);
    actionCollection()->addAction("eraser", eraserAction);
    connect(eraserAction,SIGNAL(triggered(bool)),this,SLOT(setEraserMode(bool)));
    
}
//******************************************************************************
DEVELOPER::~DEVELOPER()
    { delete eraserObject; }
//******************************************************************************
void DEVELOPER::loadTiles()
{
    //erasor Image
    KIcon icon("draw-eraser");
    QPixmap eraser = icon.pixmap(TILE_SIZE,TILE_SIZE);
    eraserObject = new KGEraser(eraser);
}
//******************************************************************************
void DEVELOPER::initLayerToolbar(void)
{
    /*backgroundAction = new KAction(KIcon("tab-new"), i18n("hide layer background"), this);
    actionCollection()->addAction("hideshow_layer_background", backgroundAction);
    connect(backgroundAction, SIGNAL(triggered(bool)), this, SLOT(selectbackgroundLayer(bool)));*/

    tileLayerAction = new KToggleAction(KIcon("view-list-icons"), i18n("Tile Layer"), this);
    actionCollection()->addAction("hideshow_layer_tiles", tileLayerAction);
    connect(tileLayerAction, SIGNAL(triggered(bool)), this, SLOT(selectTileLayer(bool)));

    objectLayerAction = new KToggleAction(KIcon("preferences-other"), i18n("Item Layer"), this);
    actionCollection()->addAction("hideshow_layer_items", objectLayerAction);
    connect(objectLayerAction, SIGNAL(triggered(bool)), this, SLOT(selectObjectLayer(bool)));

    tileLayerAction->setChecked(true);
}
//**************************************
//SLOTS
//**************************************
void DEVELOPER::renameTab(QString oldReference, QString newReference, QString newName)
{
    for(int loop = 0; loop < tabWidget->count(); loop++)
    {
	KGTabInterface *interface = dynamic_cast<KGTabInterface *>(tabWidget->widget(loop));
	if(interface->location() == oldReference)
	{
	    tabWidget->setTabText(loop,newName);
	    interface->setLocation(newReference);
	    return;
	}
    }
}
//******************************************************************************
void DEVELOPER::selectTab(QString reference)
{
    for(int loop = 0; loop < tabWidget->count(); loop++)
    {
	KGTabInterface *interface = dynamic_cast<KGTabInterface *>(tabWidget->widget(loop));
	if(interface->location() == reference)
	{
	    tabWidget->setCurrentIndex(loop);
	    return;
	}
    }

    KGTabInterface *tabObject = project->openLevel(reference);
    tabWidget->addTab(dynamic_cast<QWidget *>(tabObject),tabObject->name());
    tabWidget->setCurrentIndex(tabWidget->count()-1);    
}
//******************************************************************************
void DEVELOPER::createNewGame()
{
    gameData = newGameWizard->getGameData();
    if(!gameData->saveGame())
    {
	QMessageBox::critical(this, tr("kolladev"), tr("Failed to create new game"), QMessageBox::Ok);
	return;
    }
    setupGame(gameData);
    newLevel();
}
//******************************************************************************
void DEVELOPER::setupGame(KGGameData *gameData)
{
    while(tabWidget->count() > 0)
	tabWidget->removeTab(tabWidget->currentIndex());

    project->setGameData(gameData);
    toolBoxDock->show();
    projectDock->show();
}
//******************************************************************************
void DEVELOPER::openGame()
{
    KUrl filename = KFileDialog::getOpenFileName(KUrl(),"*.KGP",this,"Open a Game");
    if(filename == NULL)
	return;

    gameData = new KGGameData();
    gameData->openGame(filename.path());
    setupGame(gameData);
}
//******************************************************************************
void DEVELOPER::newGame()
{
    newGameWizard = new KGNewGameWizard();
    newGameWizard->show();
    connect(newGameWizard, SIGNAL(accepted()),this,SLOT(createNewGame()));
}
//******************************************************************************
void DEVELOPER::closeLevel()
    { tabWidget->removeTab(tabWidget->currentIndex()); }
//******************************************************************************
void DEVELOPER::openLevel()
{
   /* KUrl filename = KFileDialog::getOpenFileName(KUrl(),"*",this,"Open a level");
    if(filename == NULL)
	return;
    
    QFile openLevel(filename.path());
    QDomDocument doc;

    openLevel.open(QIODevice::ReadWrite);
    doc.setContent(&openLevel);
    openLevel.close();
    QDomElement element = doc.documentElement();
    QDomNodeList elementList = element.childNodes();

    KGLevelEditor *levelEditor = newLevel();
    if(!levelEditor)
	return;
    
    for(unsigned int loop = 0; loop < elementList.length(); loop++)
    {
	QDomElement objectElement = elementList.item(loop).toElement();
	KGObject *object = container.findObject(objectElement.attribute("reference",QString()));
	if(!object)
	    continue;
	
	QDomElement locationElement = objectElement.firstChildElement("location");
	QString y = locationElement.attribute("y",QString());
	QString x = locationElement.attribute("x",QString());
	levelEditor->addObject(object,TILE_LAYER,QPoint(x.toInt(),y.toInt()));
    }*/
    //filename.path()
}
//******************************************************************************
void DEVELOPER::saveLevel()
    { dynamic_cast<KGTabInterface *>(tabWidget->currentWidget())->save(); }
//******************************************************************************
void DEVELOPER::saveLevelAs()
    {}
//******************************************************************************
KGObject *DEVELOPER::getSelectedObject()
    { return KGSelectionInterface::instance()->selectedObject(); }
//******************************************************************************
void DEVELOPER::newLevel()
{ 
    KGTabInterface *newTabObject = project->newLevel();
    tabWidget->addTab(dynamic_cast<QWidget *>(newTabObject),newTabObject->name());
    tabWidget->setCurrentIndex(tabWidget->count()-1);
}
//******************************************************************************
void DEVELOPER::setEraserMode(bool mode)
    { KGSelectionInterface::instance()->setEraserMode(mode); }
//******************************************************************************
void DEVELOPER::eraserModeChanged(bool mode)
    {  eraserAction->setChecked(mode); }
//******************************************************************************
void DEVELOPER::changeLayerID(int layerID)
{
    switch(layerID)
    {
	case TILE_LAYER:
	    tileLayerAction->setChecked(true);
	    objectLayerAction->setChecked(false);
	    break;
	case OBJECT_LAYER:
	    tileLayerAction->setChecked(false);
	    objectLayerAction->setChecked(true);
	    break;
    }
}
//******************************************************************************
void DEVELOPER::selectTileLayer(bool state)
{
    KGSelectionInterface::instance()->setlayerID(TILE_LAYER);
    tileLayerAction->setChecked(true);
    objectLayerAction->setChecked(false);
}
//******************************************************************************
void DEVELOPER::selectObjectLayer(bool state)
{
    KGSelectionInterface::instance()->setlayerID(OBJECT_LAYER);
    objectLayerAction->setChecked(true);
    tileLayerAction->setChecked(false);    
}

#include "developer.moc"
