#include "KGContainer.h"

#include <KDebug>
//**************************************
//static
//**************************************
KGObject *KGContainer::createObjectFromType(QString name)
{
    if(name == "tile")
	return new KGTile();
    else
	return NULL;
}
//**************************************
//public
//**************************************
//******************************************************************************
KGContainer::KGContainer(KGContainer *parent, int layerID)
{ 
    m_parent = parent; 
    m_layerID = layerID;
}
//******************************************************************************
KGContainer::~KGContainer()
{
    /*foreach(KGObject *object,objectList)
	delete object;*/
    foreach(KGContainer *container, containerList)
	delete container;
}
//******************************************************************************
int KGContainer::layerID(void)
    { return m_layerID; }
//******************************************************************************
int KGContainer::layerIDFromObject(KGObject *object)
{
    KGContainer *container = findContainer(object->reference().section("/",0,-2));
    if(container)
	return container->layerID();
    else
	return INVALID_LAYER;
}
//******************************************************************************
KGObject *KGContainer::findFirstObject()
    { return recursive_findFirstObject(this); }
//******************************************************************************
KGObject *KGContainer::recursive_findFirstObject(KGContainer *parentContainer)
{
    foreach(KGContainer *container,parentContainer->containers())
    {
	KGObject *object = recursive_findFirstObject(container);
	if(object)
	    return object;
    }
    foreach(KGObject *object,parentContainer->objects())
    {
	if(object)
	    return object;
    }
    return NULL;
}
//******************************************************************************
KGObject *KGContainer::findObject(QString reference)
{
    KGContainer *container = findContainer(reference);
    if(container)
    {
	kDebug() << reference.section("/",-1) << endl;
	return container->findObject(reference.section("/",-1));
    }
    foreach(KGObject *object,objectList)
    {
	if(object->name() == reference.section("/",-1))
	{
	    kDebug() << "object Found" << endl;
	    return object;
	}
    }
    return recursive_loadContainer(openSystemDefaults(), reference);
}
//******************************************************************************
KGContainer *KGContainer::findContainer(QString reference)
{
    foreach(KGContainer *container,containerList)
    {
	if(container->reference() == reference.section("/",0,0))
	{
	    KGContainer *returnContainer = container->findContainer(reference.section("/",1));
	    if(returnContainer)
		return returnContainer;
	    else
		return container;
	}
    }
    return NULL;
}
//******************************************************************************
QDomElement KGContainer::findElementByAttribute(QDomElement parentElement, QString tagName, QString attributeName, QString attributeValue)
{
    QDomNodeList elementList = parentElement.childNodes();

    //for each element
    for(unsigned int loop = 0; loop < elementList.length(); loop++)
    {
	QDomElement elementItem = elementList.item(loop).toElement();
	if(elementItem.tagName() == tagName && elementItem.attribute(attributeName,QString()) == attributeValue)
	{
	    return elementItem;
	}
    }
    return parentElement;
}
//******************************************************************************
KGObject *KGContainer::recursive_loadContainer(QDomElement rootElement, QString reference)
{
    QDomElement childElement = findElementByAttribute(rootElement,CATEGORY_TAG_NAME,"name",reference.section("/",0,0));
    if(childElement != rootElement)
	return recursive_loadContainer(childElement,reference.section("/",1));
    childElement = findElementByAttribute(rootElement,"tile","name",reference.section("/",0,0));
    if(childElement != rootElement)
	return loadObjectFromDefaults(this, childElement,reference);
    return NULL;
}
//******************************************************************************
KGObject *KGContainer::loadObjectFromDefaults(KGContainer *parentContainer, QDomElement elementItem, QString reference)
{
    kDebug() << "loading object from file" << endl;
    //load object
    KGObject *newObject = createObjectFromType(elementItem.tagName());

    //if object loaded
    if(newObject)
    {
	//load object data and add to object list
	newObject->loadObject(elementItem);
	newObject->setReference(reference + newObject->name());
	parentContainer->addObject(newObject);
    }
    return newObject;
}
//******************************************************************************
KGContainer *KGContainer::loadContainerFromDefaults(KGContainer *parentContainer, QDomElement elementItem)
{
    //create new container
    KGContainer *newContainer = new KGContainer(parentContainer,parentContainer->layerID());

    newContainer->setReference(elementItem.attribute("name",QString()));
	
    //add container to list of containers
    parentContainer->addContainer(newContainer);

    return newContainer;
}
//******************************************************************************
void KGContainer::recursive_preLoadAllObjects(QDomElement rootElement,KGContainer *parentContainer, QString reference)
{
    QDomNodeList elementList = rootElement.childNodes();
    //for each element
    for(unsigned int loop = 0; loop < elementList.length(); loop++)
    {
	QDomElement elementItem = elementList.item(loop).toElement();
	
	if(isCategory(elementItem.tagName()))
	{
	    KGContainer *newContainer = loadContainerFromDefaults(parentContainer,elementItem);
	    //load container data
	
	    if(newContainer)
		recursive_preLoadAllObjects(elementItem,newContainer, newContainer->reference() + "/");
	}
	else
	    loadObjectFromDefaults(parentContainer, elementItem, reference);
    }
}
//******************************************************************************
QDomElement KGContainer::openSystemDefaults()
{
    //NOTE: This class needs to get the selected item as it adds each tile onto the widget. 
    //TODO: have KGSelectionWidget get the selected item for its own widget, 
    //	    then part of this code can be moved KGSelectionWidget

    QString directory = "kollagame/";

    //find default objects fille
    QString docLocation = KGlobal::dirs()->findResource("data", directory + SYSTEM_FILE_NAME);

    //get xml data
    QFile inputFile(docLocation);
    inputFile.open(QIODevice::ReadWrite);
    QDomDocument doc;
    doc.setContent(&inputFile);
    inputFile.close();
    return doc.documentElement();
}
//******************************************************************************
void KGContainer::preLoadAll(void)
{ 
    QDomNodeList elementList = openSystemDefaults().childNodes();
    createContainer("background images", BACKGROUND_LAYER);
    createContainer("ground tiles", TILE_LAYER);
    createContainer("obstical tiles", TILE_LAYER);
    createContainer("items", OBJECT_LAYER);
    createContainer("spells", OBJECT_LAYER);
    createContainer("weapons", OBJECT_LAYER);
    createContainer("bosses", OBJECT_LAYER);
    createContainer("enemies", OBJECT_LAYER);
    createContainer("characters", OBJECT_LAYER);
    createContainer("players", OBJECT_LAYER);

    for(int loop = 0; loop < containerList.count() && (unsigned int)loop < elementList.length();  loop++)
    {
	QDomElement elementItem = elementList.item(loop).toElement();

	//found proper tag
	if(containerList[loop]->reference() == elementItem.attribute("name",QString()))
	    recursive_preLoadAllObjects(elementItem,containerList[loop], containerList[loop]->reference() + "/");
	else
	{
	    for(unsigned int loop2 = 0; loop2 < elementList.length(); loop2++)
	    {
		QDomElement elementItem2 = elementList.item(loop2).toElement();

		if(containerList[loop]->reference() == elementItem2.attribute("name",QString()))
		    recursive_preLoadAllObjects(elementItem2,containerList[loop], containerList[loop]->reference() + "/");
	    }
	}
    }
}
//******************************************************************************
void KGContainer::createContainer(QString name, int layerID)
{
    KGContainer *newContainer = new KGContainer(this,layerID);
    newContainer->setReference(name);
    addContainer(newContainer);
}
//******************************************************************************
void KGContainer::unloadAll(void)
    {}
//******************************************************************************
void KGContainer::addObject(KGObject *object)
    { objectList.push_back(object); }
//******************************************************************************
void KGContainer::addContainer(KGContainer *container)
    { containerList.push_back(container); }
//******************************************************************************
QList<KGContainer *> KGContainer::containers()
    { return containerList; }
//******************************************************************************
QList<KGObject *> KGContainer::objects()
    { return objectList; }
//******************************************************************************
QString KGContainer::reference(bool /*fullPath*/)
    { return m_reference; }
//******************************************************************************
void KGContainer::setReference(QString reference)
    { m_reference = reference; }
//******************************************************************************
bool KGContainer::isCategory(QString name)
    { return (name == CATEGORY_TAG_NAME); }

