/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "blanktiledialog.h"
#include "blanktiledialog.moc"

#include <QLabel>
#include <qlayout.h>
#include <QWidget>

#include <klocale.h>
#include <kcombobox.h>

#include "tile.h"

blankTileDialog::blankTileDialog(QWidget *parent)
 : KDialog( parent)
{
    setCaption( i18n("Edit Blank Tile") );
    setButtons( Ok|Apply|Cancel );
    setDefaultButton( Ok );

	m_tile = 0;

	QWidget *main = new QWidget(this);
	setMainWidget(main);
	QVBoxLayout *vbox = new QVBoxLayout(main);

	QLabel *lbl1 = new QLabel(i18n("You can set here which character(s) this blank tile represents."), main);
	lbl1->setWordWrap(true);
	vbox->addWidget(lbl1);

	m_text = new QLabel(main);
	vbox->addWidget(m_text);

	QHBoxLayout *hbox = new QHBoxLayout();
	vbox->addLayout(hbox);
	hbox->addWidget(new QLabel(i18n("New letter(s):"), main));

	m_combo = new KComboBox(main);
	
	hbox->addWidget(m_combo);

	connect( this, SIGNAL( okClicked() ), this, SLOT( slotOk() ) );
	connect( this, SIGNAL( applyClicked() ), this, SLOT( slotApply() ) );
	connect( this, SIGNAL( cancelClicked() ), this, SLOT( slotCancel() ) );
}

void blankTileDialog::setData(tile *t, const QList< QPair<QString, int> > &tileStringList)
{
	m_tile = t;
	m_origString = m_tile->string();

	m_combo->clear();

	QPair<QString, int> p;
	for(int i = 0; i < tileStringList.size(); i++)
		m_combo->addItem(tileStringList[i].first);
	int index = m_combo->findText(m_origString);
	if (index > -1)
		m_combo->setCurrentIndex(index);

	updateWidgets();
}

void blankTileDialog::slotOk()
{
	m_tile->setString(m_combo->currentText());
	emit tileChanged();
	accept();
}

void blankTileDialog::slotApply()
{
	m_tile->setString(m_combo->currentText());
	emit tileChanged();
	updateWidgets();
}

void blankTileDialog::slotCancel()
{
	m_tile->setString(m_origString);
	emit tileChanged();
	reject();
}

void blankTileDialog::updateWidgets()
{
	if (m_tile != 0)
		m_text->setText(i18n("Actual setting: \"%1\"", m_tile->string()));
}
