/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "board.h"

#include <QApplication>
#include <QDrag>
#include <QDragMoveEvent>
#include <QDropEvent>
#include <QMouseEvent>
#include <QPainter>
#include <QStringList>

#include <kdebug.h>
#include <kglobal.h>

#include "square.h"
#include "tile.h"
#include "tilewidget.h"
#include "utils.h"
#include "settings.h"
#include "error.h"
#include "dictionary.h"

board::board() : m_numFixedTiles(0), m_dictionary(0)
{
	m_squares[ 0][ 0].setKind(square::TRIPLE_WORD);
	m_squares[ 7][ 0].setKind(square::TRIPLE_WORD);
	m_squares[14][ 0].setKind(square::TRIPLE_WORD);
	m_squares[ 0][ 7].setKind(square::TRIPLE_WORD);
	m_squares[14][ 7].setKind(square::TRIPLE_WORD);
	m_squares[ 0][14].setKind(square::TRIPLE_WORD);
	m_squares[ 7][14].setKind(square::TRIPLE_WORD);
	m_squares[14][14].setKind(square::TRIPLE_WORD);
	m_squares[ 1][ 1].setKind(square::DOUBLE_WORD);
	m_squares[ 1][13].setKind(square::DOUBLE_WORD);
	m_squares[ 2][ 2].setKind(square::DOUBLE_WORD);
	m_squares[ 2][12].setKind(square::DOUBLE_WORD);
	m_squares[ 3][ 3].setKind(square::DOUBLE_WORD);
	m_squares[ 3][11].setKind(square::DOUBLE_WORD);
	m_squares[ 4][ 4].setKind(square::DOUBLE_WORD);
	m_squares[ 4][10].setKind(square::DOUBLE_WORD);
	m_squares[10][ 4].setKind(square::DOUBLE_WORD);
	m_squares[10][10].setKind(square::DOUBLE_WORD);
	m_squares[11][ 3].setKind(square::DOUBLE_WORD);
	m_squares[11][11].setKind(square::DOUBLE_WORD);
	m_squares[12][ 2].setKind(square::DOUBLE_WORD);
	m_squares[12][12].setKind(square::DOUBLE_WORD);
	m_squares[13][ 1].setKind(square::DOUBLE_WORD);
	m_squares[13][13].setKind(square::DOUBLE_WORD);
	m_squares[ 7][ 7].setKind(square::CENTER);
	m_squares[ 1][ 5].setKind(square::TRIPLE_LETTER);
	m_squares[ 1][ 9].setKind(square::TRIPLE_LETTER);
	m_squares[ 5][ 1].setKind(square::TRIPLE_LETTER);
	m_squares[ 5][ 5].setKind(square::TRIPLE_LETTER);
	m_squares[ 5][ 9].setKind(square::TRIPLE_LETTER);
	m_squares[ 5][13].setKind(square::TRIPLE_LETTER);
	m_squares[ 9][ 1].setKind(square::TRIPLE_LETTER);
	m_squares[ 9][ 5].setKind(square::TRIPLE_LETTER);
	m_squares[ 9][ 9].setKind(square::TRIPLE_LETTER);
	m_squares[ 9][13].setKind(square::TRIPLE_LETTER);
	m_squares[13][ 5].setKind(square::TRIPLE_LETTER);
	m_squares[13][ 9].setKind(square::TRIPLE_LETTER);
	m_squares[ 0][ 3].setKind(square::DOUBLE_LETTER);
	m_squares[ 0][11].setKind(square::DOUBLE_LETTER);
	m_squares[ 2][ 6].setKind(square::DOUBLE_LETTER);
	m_squares[ 2][ 8].setKind(square::DOUBLE_LETTER);
	m_squares[ 3][ 0].setKind(square::DOUBLE_LETTER);
	m_squares[ 3][ 7].setKind(square::DOUBLE_LETTER);
	m_squares[ 3][14].setKind(square::DOUBLE_LETTER);
	m_squares[ 6][ 2].setKind(square::DOUBLE_LETTER);
	m_squares[ 6][ 6].setKind(square::DOUBLE_LETTER);
	m_squares[ 6][ 8].setKind(square::DOUBLE_LETTER);
	m_squares[ 6][12].setKind(square::DOUBLE_LETTER);
	m_squares[ 7][ 3].setKind(square::DOUBLE_LETTER);
	m_squares[ 7][11].setKind(square::DOUBLE_LETTER);
	m_squares[ 8][ 2].setKind(square::DOUBLE_LETTER);
	m_squares[ 8][ 6].setKind(square::DOUBLE_LETTER);
	m_squares[ 8][ 8].setKind(square::DOUBLE_LETTER);
	m_squares[ 8][12].setKind(square::DOUBLE_LETTER);
	m_squares[11][ 0].setKind(square::DOUBLE_LETTER);
	m_squares[11][ 7].setKind(square::DOUBLE_LETTER);
	m_squares[11][14].setKind(square::DOUBLE_LETTER);
	m_squares[12][ 6].setKind(square::DOUBLE_LETTER);
	m_squares[12][ 8].setKind(square::DOUBLE_LETTER);
	m_squares[14][ 3].setKind(square::DOUBLE_LETTER);
	m_squares[14][11].setKind(square::DOUBLE_LETTER);

	m_squares[ 7][ 7].setToAnchor();
}

board::~board()
{
	if (m_dictionary)
		delete m_dictionary;
}

bool board::checkValidCross( Orientation orientation, int x, int y ) const
{
	int minX, minY, maxX, maxY;
	minX = maxX = x;
	minY = maxY = y;

	//kDebug() << "getting possible word..." << endl;
	tilePositionList mainword = getWord(x, y, orientation);
	QString checkWord = "";

	for (int i = 0; i < mainword.count(); i++ )
	{
		QString temp = ((mainword.at(i)).ourTile())->string();
		checkWord.append(temp);
	}

	//kDebug() << "checkValidCross, slowo do spr: " << checkWord << endl;

	return m_dictionary->isCorrect(checkWord);
}

#define EXIT_WITH_ERROR(e) \
{ \
	error = e; \
	wordsAndTilePositions emptylist;\
	return emptylist; \
}
wordsAndTilePositions board::checkValidTilesPositions(int &score, int &error) const
{
	score = 0;

	//only need to check if at least one tile is on anchor, tiles are alighed, and there are no empty spaces betwean tiles
	int numAddedTiles = m_playedTiles.size();
	bool onAnchor = false;
	for (int i = 0; i < numAddedTiles; i++)
	{
		onAnchor |= (m_squares[m_playedTiles.at(i).x()][m_playedTiles.at(i).y()].state() == square::ANCHOR);
	}
	if (!(onAnchor))
	{
		if (m_numFixedTiles == 0) EXIT_WITH_ERROR(ERROR_FIRST_WORD_NOT_CENTERED);
		EXIT_WITH_ERROR(ERROR_TILES_ISOLATED);
	}

	
	int minX, minY, maxX, maxY;
	bool haligned = false;
	bool valigned = false;
	
	//allign check
	minY = maxY = m_playedTiles.at(0).y();
	minX = maxX = m_playedTiles.at(0).x();
	for (int i = 1; i < numAddedTiles; i++)
	{
		if (minY > m_playedTiles.at(i).y()) minY = m_playedTiles.at(i).y();
		else if (maxY < m_playedTiles.at(i).y()) maxY = m_playedTiles.at(i).y();

		if (minX > m_playedTiles.at(i).x()) minX = m_playedTiles.at(i).x();
		else if (maxX < m_playedTiles.at(i).x()) maxX = m_playedTiles.at(i).x();
	}
	haligned = (minY == maxY);
	valigned = (minX == maxX);
	if (!(haligned || valigned)) EXIT_WITH_ERROR(ERROR_TILES_NOT_ALIGNED);

	if (haligned && valigned)
	{
		bool newAlign = false;
		if (minX > 0) newAlign |= (m_squares[minX-1][minY].ourTile() != 0);
		if (minX < 14) newAlign |= (m_squares[minX+1][minY].ourTile() != 0);
		haligned = newAlign;
		newAlign = false;
		if (minY > 0) newAlign |= (m_squares[minX][minY-1].ourTile() != 0);
		if (minY < 14) newAlign |= (m_squares[minX][minY+1].ourTile() != 0);
		valigned = newAlign;
	}

	QList<tilePositionList> wordspos;
	tilePositionList mainword;
	
	//empty spaces & addign tiles to mainword
	if (haligned)
		for(int i = minX; i <= maxX; i++)
		{
			if ( ! (m_squares[i][minY].ourTile())) EXIT_WITH_ERROR(ERROR_TILES_ISOLATED);
			mainword << tilePosition(const_cast<tile*>(m_squares[i][minY].ourTile()), i, minY);
		}
	else
		for(int i = minY; i <= maxY; i++)
		{
			if ( ! (m_squares[minX][i].ourTile())) EXIT_WITH_ERROR(ERROR_TILES_ISOLATED);
			mainword << tilePosition(const_cast<tile*>(m_squares[minX][i].ourTile()), minX, i);
		}

	//adding letters to main word that are before placed tiles
	if (haligned) valigned = false;
	int xc = haligned ? minX - 1 : m_playedTiles.at(0).x();
	int yc = valigned ? minY - 1 : m_playedTiles.at(0).y();
	while (((haligned ? xc : yc) >= 0) && m_squares[xc][yc].ourTile())
	{
		mainword.insert(0, tilePosition(const_cast<tile*>(m_squares[xc][yc].ourTile()), xc, yc));
		xc += haligned ? -1 : 0;
		yc += valigned ? -1 : 0;
	}

	//adding letters to main word that are after placed tiles
	xc = haligned ? maxX + 1 : m_playedTiles.at(0).x();
	yc = valigned ? maxY + 1 : m_playedTiles.at(0).y();
	while (((haligned ? xc : yc) < 15) && m_squares[xc][yc].ourTile())
	{
		mainword << tilePosition(const_cast<tile*>(m_squares[xc][yc].ourTile()), xc, yc);
		xc += haligned ? 1 : 0;
		yc += valigned ? 1 : 0;
	}

	wordspos << mainword;

	//adding cross words
	for (int i = 0; i < mainword.size(); i++)
	{
		if (!m_squares[mainword.at(i).x()][mainword.at(i).y()].isLocked())
		{
			tilePositionList tmpword = getWord(mainword.at(i).x(), mainword.at(i).y(), haligned ? VERTICAL : HORIZONTAL);
			if (tmpword.count() > 1)
				wordspos << tmpword;
		}
	}

	wordsAndTilePositions words;
	
	int wordmod = 1;
	int tmpscore = 0;
	int crossScore = 0;
	foreach (const tilePosition &tp, mainword)
	{
		const tile *t = tp.ourTileC();
		int tmpletterscore = t->score();
		int crossMod = 1;
		if (!m_squares[tp.x()][tp.y()].isLocked())
		{
			switch 	(m_squares[tp.x()][tp.y()].kind())
			{
				case square::CENTER:
				case square::DOUBLE_WORD:
					wordmod *= 2; crossMod = 2;
				break;
				case square::TRIPLE_WORD:
					wordmod *= 3; crossMod = 3;
				break;
				case square::DOUBLE_LETTER:
					tmpletterscore *= 2;
				break;
				case square::TRIPLE_LETTER:
					tmpletterscore *= 3;
				break;
				case square::NORMAL:
				break;
			}

			if (haligned && m_squares[tp.x()][tp.y()].verticalPoints() > 0)
				crossScore += (tmpletterscore + m_squares[tp.x()][tp.y()].verticalPoints()) * crossMod;
			else if (valigned && m_squares[tp.x()][tp.y()].horizontalPoints() > 0)
				crossScore += (tmpletterscore + m_squares[tp.x()][tp.y()].horizontalPoints()) * crossMod;
		}
		tmpscore += tmpletterscore;
	}
	score += tmpscore * wordmod + crossScore;
	if (numAddedTiles == 7) score += 50;

	foreach(const tilePositionList &tmplist, wordspos)
	{
		QString tmpword;
		foreach (const tilePosition &tp, tmplist)
		{
			const tile *t = tp.ourTileC();
			if (!t->isBlank() || (t->isBlank() && !t->string().trimmed().isEmpty()))
				tmpword += t->string();
			else
				EXIT_WITH_ERROR(ERROR_BLANK_TILE_NOT_REPLACED)
		}
		words << qMakePair(tmpword, tmplist);
	}

	error = 0;
	return words;
}
#undef EXIT_WITH_ERROR

tilePositionList board::getWord(int x, int y, Orientation align) const
{
	tilePositionList emptylist;
//	if ((x > 14) || (y > 14)) return emptylist;

	int xcoord = x + (align == HORIZONTAL ? -1 : 0);
	int ycoord = y + (align == VERTICAL ? -1 : 0);
	tilePositionList list;
	while (((align == HORIZONTAL ? xcoord : ycoord) >= 0) && m_squares[xcoord][ycoord].ourTile())
	{
		list.insert(0, tilePosition(const_cast<tile*>(m_squares[xcoord][ycoord].ourTile()), xcoord, ycoord));
		xcoord += align == HORIZONTAL ? -1 : 0;
		ycoord += align == VERTICAL ? -1 : 0;
	}
	list << tilePosition(const_cast<tile*>(m_squares[x][y].ourTile()), x, y);
	xcoord = x + (align == HORIZONTAL ? 1 : 0);
	ycoord = y + (align == VERTICAL ? 1 : 0);
	while (((align == HORIZONTAL ? xcoord : ycoord) < 15) && m_squares[xcoord][ycoord].ourTile())
	{
		list << tilePosition(const_cast<tile*>(m_squares[xcoord][ycoord].ourTile()), xcoord, ycoord);
		xcoord += align == HORIZONTAL ? 1 : 0;
		ycoord += align == VERTICAL ? 1 : 0;
	}

	if (list.count() == 1) return emptylist;

	return list;
}

/*void board::addAndLockTiles(const tilePositionList &tiles)
{
	for (int i = 0; i < tiles.size(); i++)
	{
		if (!m_squares[tiles.at(i).x()][tiles.at(i).y()].ourTile())
		{
			m_squares[tiles.at(i).x()][tiles.at(i).y()].setTile(tiles.at(i).ourTile());
			m_squares[tiles.at(i).x()][tiles.at(i).y()].setLocked(true);
		}
	}
}*/

void board::setPlayedTiles(const tilePositionList &tiles)
{
	finalizePlayedTiles(DELETE);
	m_playedTiles = tiles;
	int x, y;
	//kDebug() << "other tiles count: " << m_playedTiles.size() << endl;
	for (int i = m_playedTiles.size() - 1; i >=0; i--)
	{
		x = m_playedTiles.at(i).x();
		y = m_playedTiles.at(i).y();
		if (m_squares[x][y].ourTile() && m_squares[x][y].isLocked())
		{
			m_playedTiles.removeAtPos(x, y);
		}
		else
		{
			m_squares[x][y].setTile(tiles.at(i).ourTile());
		}
	}
}

const tilePositionList& board::getPlayedTiles()
{
	return m_playedTiles;
}

void board::finalizePlayedTiles(const FinalizeAction act)
{
	int minX, minY, maxX, maxY;
	bool haligned = false;
	bool valigned = false;
	tilePositionList crossWord;
	QList<tilePositionList> wordspos;
	switch (act)
	{
		case ACCEPT:
			int x, y;
			
			//updateCrossLetters should be done for whole new word not only new tiles
			//allign check
			if (m_playedTiles.count() > 0)
			{
				minY = maxY = m_playedTiles.at(0).y();
				minX = maxX = m_playedTiles.at(0).x();
				for (int i = 1; i < m_playedTiles.count(); i++)
				{
					if (minY > m_playedTiles.at(i).y()) minY = m_playedTiles.at(i).y();
					else if (maxY < m_playedTiles.at(i).y()) maxY = m_playedTiles.at(i).y();
	
					if (minX > m_playedTiles.at(i).x()) minX = m_playedTiles.at(i).x();
					else if (maxX < m_playedTiles.at(i).x()) maxX = m_playedTiles.at(i).x();
				}
				haligned = (minY == maxY);
				valigned = (minX == maxX);

				crossWord = getWord(minX, minY, haligned ? HORIZONTAL : VERTICAL);

				// one letter move exeption - crossWord have to be valid to updateCrossLetters
				// crossWord .size() < 2 only if haligned == valigned == true
				if (crossWord.size() < 2)
				{
					crossWord = getWord(minX, minY, VERTICAL);
					haligned = false;
				}	
			}
			for (int i = 0; i < m_playedTiles.size(); i++)
			{
				tilePositionList tmpword = getWord(m_playedTiles.at(i).x(), m_playedTiles.at(i).y(), haligned ? VERTICAL : HORIZONTAL);
				if (tmpword.count() > 1)
					wordspos << tmpword;
			}

			for (int j = 0; j < wordspos.count(); j++)
			{
				for (int i = 0; i < wordspos[j].size(); i++)
				{
					x = wordspos[j].at(i).x();
					y = wordspos[j].at(i).y();
					updateCrossLetters(x+1, y, HORIZONTAL);
					updateCrossLetters(x-1, y, HORIZONTAL);
					updateCrossLetters(x, y+1, VERTICAL);
					updateCrossLetters(x, y-1, VERTICAL);
				}
			}

			for (int i = 0; i < m_playedTiles.size(); i++)
			{
				x = m_playedTiles.at(i).x();
				y = m_playedTiles.at(i).y();	
				if (m_squares[x][y].ourTile())
				{
					addCrossPoints(m_playedTiles.at(i).ourTileC()->score(), x, y);
				}
			}
			for (int i = 0; i < m_playedTiles.size(); i++)
			{
				x = m_playedTiles.at(i).x();
				y = m_playedTiles.at(i).y();	
				if (m_squares[x][y].ourTile())
				{
					m_squares[x][y].setLocked(true);
					m_squares[x][y].ourTile()->setKind(tile::NORMAL);
					m_squares[x][y].setToTile();
					if (x < 14) m_squares[x+1][y].setToAnchor();
					if (y < 14) m_squares[x][y+1].setToAnchor();
					if (y > 0) m_squares[x][y-1].setToAnchor();
					if (x > 0) m_squares[x-1][y].setToAnchor();
				}
			}
			m_numFixedTiles += m_playedTiles.size();

			if (crossWord.size() > 1)
			for (int i = 0; i < crossWord.size(); i++)
			{
				if (m_squares[x][y].ourTile())
				{
					x = crossWord.at(i).x();
					y = crossWord.at(i).y();
					updateCrossLetters(x+1, y, HORIZONTAL);
					updateCrossLetters(x-1, y, HORIZONTAL);
					updateCrossLetters(x, y+1, VERTICAL);
					updateCrossLetters(x, y-1, VERTICAL);
				}
			}
			m_playedTiles.clear();
		break;
		case DELETE:
			for (int i = 0; i < m_playedTiles.size(); i++)
			{
				m_squares[m_playedTiles.at(i).x()][m_playedTiles.at(i).y()].setTile(0L);
				m_squares[m_playedTiles.at(i).x()][m_playedTiles.at(i).y()].setLocked(false);
			}
			m_playedTiles.clear();
		break;
	}
	//dumpBoard();
}

void board::dumpBoard()
{
	kDebug() << "Locked status and letter" <<  endl;
	for (int i = 0; i < 15; i++)
	{
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			tmp += QString(m_squares[j][i].isLocked() ? " [%1]" : " (%1)").arg(m_squares[j][i].ourTile() ? m_squares[j][i].ourTile()->string(true) : "-");
		}
		kDebug() << tmp << endl;
	}
	kDebug() << "cross points - vertical" << endl;
	for (int i = 0; i < 15; i++)
	{
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			tmp += QString(m_squares[j][i].isLocked() ? " [%1]" : " (%1)").arg( m_squares[j][i].verticalPoints());
		}
		kDebug() << tmp << endl;
	}
	kDebug() << "cross points - horizontal" << endl;
	for (int i = 0; i < 15; i++)
	{
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			tmp += QString(m_squares[j][i].isLocked() ? " [%1]" : " (%1)").arg( m_squares[j][i].horizontalPoints());
		}
		kDebug() << tmp << endl;
	}
	//crossLettersState
	kDebug() << "cross letters - vertical" << endl;
	for (int i = 0; i < 15; i++)
	{
		QBitArray testBA (32);
		testBA.fill(true);
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			if (testBA==m_squares[j][i].crossLettersState(VERTICAL)) continue;
			QString field = "";
			field += (QString)"x=" += QString("%1 ").arg(j) += (QString)"y=" += QString("%1 ").arg(i);
			for (int k = 0; k < m_squares[j][i].crossLettersState(VERTICAL).count(); k++)
			{
				if ((m_squares[j][i].crossLettersState(VERTICAL).at(k) == 1) && (k < 27))
				{
					field += m_tilesStringList[k].first;
				}
			}
			tmp += QString(m_squares[j][i].isLocked() ? " [%1]" : " (%1)").arg(field);
		}
		kDebug() << tmp << endl;
	}
	kDebug() << "cross letters - horizonstal" << endl;
	for (int i = 0; i < 15; i++)
	{
		QBitArray testBA (32);
		testBA.fill(true);
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			if (testBA==m_squares[j][i].crossLettersState(HORIZONTAL)) continue;
			QString field = "";
			field += (QString)"x=" += QString("%1 ").arg(j) += (QString)"y=" += QString("%1 ").arg(i);
			for (int k = 0; k < m_squares[j][i].crossLettersState(HORIZONTAL).count(); k++)
			{
				if ((m_squares[j][i].crossLettersState(HORIZONTAL).at(k) == 1) && (k < 27))
				{
					field += m_tilesStringList[k].first;
				}
			}
			tmp += QString(m_squares[j][i].isLocked() ? " [%1]" : " (%1)").arg(field);
		}
		kDebug() << tmp << endl;
	}
	kDebug() << "Anchors, Tile, Normal" << endl;
	for (int i = 0; i < 15; i++)
	{
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			if (m_squares[j][i].state() == square::ANCHOR)
				tmp += QString("[A]");
			else if (m_squares[j][i].state() == square::TILE)
				tmp += QString("[T]");
			else
				tmp += QString("[ ]");
		}
		kDebug() << tmp << endl;
	}
	kDebug() << "Anchors, Tile, Normal" << endl;
	for (int i = 0; i < 15; i++)
	{
		QString tmp = "";
		for (int j = 0; j < 15; j++)
		{
			if (m_squares[j][i].isLocked() == true)
				tmp += QString("[L]");
			else
				tmp += QString("[ ]");
		}
		kDebug() << tmp << endl;
	}
}

//
void board::updateCrossLetters(int x, int y, Orientation orientation)
{
	if ((x < 0) || (y < 0 ) || (x > 14) || (y > 14)) return;
	if (m_squares[x][y].isAnchor() == false) return;
	if (m_squares[x][y].ourTile() != 0) return;
	//add each letter and check word created in specified orientation
	Orientation opposite;
	if (orientation == HORIZONTAL)
		opposite = VERTICAL;
	else
		opposite = HORIZONTAL;

	tile checkTile("");
	for (int i = 1; i < m_tilesStringList.size(); ++i) //i = 1 to skip blank with = 0
	{
		checkTile.setString("A"); //test
		QString test = m_tilesStringList[i].first;
		checkTile.setString( m_tilesStringList[i].first );//letter string		

		//kDebug() << "adding tested tile" << endl;
		// and adding the new one
		m_squares[x][y].setTile(&checkTile);
		
		//const wordsAndTilePositions &wordsAndTiles = m_board.checkValidTilesPositions(score, error);
		//jesli ok ustal flage na 1 jesli nie to na 0
		//kDebug() << "checking if it creates word..." << endl;
		if ( checkValidCross( orientation, x, y ) == true )
			m_squares[x][y].turnOnCrossLetter(i, opposite);
		else
			m_squares[x][y].turnOffCrossLetter(i, opposite);

		// removing the previous tiles
		m_squares[x][y].setTile(0L);
	}
}

//finds squares that was affected by new tile and add cross points. Destination square can't have tile.
void board::addCrossPoints(const uint score, int x, int y)
{
	int oldVerticalScore = m_squares[x][y].verticalPoints();
	for (int i = y+1; i < 15; i++)
	{
		if (m_squares[x][i].ourTile())
		{
			if (m_squares[x][i].isLocked())
				oldVerticalScore = 0;
		}
		else
		{
			m_squares[x][i].addVerticalPoints(score + oldVerticalScore);
			break;
		}
	}
	oldVerticalScore = m_squares[x][y].verticalPoints();
	for (int i = y-1; i >= 0; i--)
	{
		if (m_squares[x][i].ourTile())
		{
			if (m_squares[x][i].isLocked())
				oldVerticalScore = 0;
		}
		else
		{
			m_squares[x][i].addVerticalPoints(score + oldVerticalScore);
			break;
		}
	}
	
	int oldHorizontalScore = m_squares[x][y].horizontalPoints();
	for (int i = x+1; i < 15; i++)
	{
		if (m_squares[i][y].ourTile())
		{
			if (m_squares[i][y].isLocked())
				oldHorizontalScore = 0;
		}
		else
		{
			m_squares[i][y].addHorizontalPoints(score + oldHorizontalScore);
			break;
		}
	}
	oldHorizontalScore = m_squares[x][y].horizontalPoints();
	for (int i = x-1; i >= 0; i--)
	{
		if (m_squares[i][y].ourTile())
		{
			if (m_squares[i][y].isLocked())
				oldHorizontalScore = 0;
		}
		else
		{
			m_squares[i][y].addHorizontalPoints(score + oldHorizontalScore);
			break;
		}
	}
}

QList<QList<tilePosition*> *>* board::getPossibleMoves(QList<tile*> &tiles)
{
	/*
	 * For each anchor check allowed letters.
	 * - If ai has this letter then put it on board and generate leftsize.
	 * - If there is no this letter but rack constains blank put it instead and generate leftsize.
	 * - no letter and no blank - check next allowed letter.
	 * Left part will contain either tiles from our rack, or already on the board, but not both.
	 * If there is empty square before anchor empty left side is possible. 
	 * - if there is left part on board check it in dictionary with added anchor letter (can create word). If ok try to create right part.
	 * - if there is no left part check max length.
	 * - for each length
	 */

	QList<QList<tilePosition*> *> * wordList = new QList<QList<tilePosition*> *>;
	QList<tilePosition> partialWord;
	letter* dict = m_dictionary->getInnerTable();
	rack *playerRack = new rack(tiles);
	//kDebug() << "getPossibleMoves rack contains: " << endl;
	//for (int zz = 0; zz < tiles.size(); zz++)
	//	kDebug() << zz << ": " << tiles[zz]->string() << endl;

	//dumpBoard();

	for (int i = 0; i < 15; i++)
	 for (int j = 0; j < 15; j++)
	{
		if (m_squares[i][j].isAnchor())
		{
			//kDebug() << "getPossibleMoves, anchor at x: " << i << " y: " << j << endl;
			bool leftSideOnBoard = false;
			
			int leftSize = findMaxLeftSize(i, j, HORIZONTAL, leftSideOnBoard, tiles.size() - 1);
			//kDebug() << "Fount max left size: " << leftSize << endl;
			partialWord.clear();
			if (leftSideOnBoard)
			{
				extendRight(partialWord, playerRack, 0, dict, i-leftSize, j, HORIZONTAL, wordList, false);
			}
			else
			{	
				leftPart(partialWord, playerRack, 0, dict, i, j, HORIZONTAL, wordList, leftSize);
			}
			
			leftSize = findMaxLeftSize(i, j, VERTICAL, leftSideOnBoard, tiles.size() - 1);
			partialWord.clear();
			if (leftSideOnBoard)
			{
				extendRight(partialWord, playerRack, 0, dict, i, j-leftSize, VERTICAL, wordList, false);
			}
			else
			{	
				leftPart(partialWord, playerRack, 0, dict, i, j, VERTICAL, wordList, leftSize);
			}
		}
	}

	delete playerRack;
	return wordList;
}

int board::findMaxLeftSize(int x, int y, Orientation orientation, bool &leftSideOnBoard, int maxSize)
{
	//board end
	if (((x == 0) && (orientation == HORIZONTAL)) || ((y == 0) && (orientation == VERTICAL) ))
		return 0;
	int dx = 0;
	int dy = 0;
	if (orientation == HORIZONTAL)
		dx = -1;
	else
		dy = -1;
	
	x+=dx;
	y+=dy;
	if (m_squares[x][y].state() == square::ANCHOR) return 0;
	square::State baseState = m_squares[x][y].state();
	if (baseState == square::TILE)
		leftSideOnBoard = true;
	else 
		leftSideOnBoard = false;
	int size = 1;
	while ((x+dx > -1) && (y+dy > -1) && ((size < maxSize) || (baseState == square::TILE)))
	{
		x += dx;
		y += dy;
		//if (m_squares[x][y].state() == ANCHOR) return size;
		if (m_squares[x][y].state() != baseState) return size;
		size++;
	}
	return size;
}

void board::leftPart(QList<tilePosition> &partialWord, rack* playerRack, int nodeIndex, letter* letters, int x, int y, Orientation orientation, QList<QList<tilePosition*> *> *wordList, int maxSize)
{
	if ((nodeIndex == 0) && (playerRack->started())) return;
	for (int i = 0; i < partialWord.size(); i++)
	{
		if (orientation == HORIZONTAL)
			partialWord[i].setX(x-partialWord.size()+i);
		else
			partialWord[i].setY(y-partialWord.size()+i);
	}	
	extendRight(partialWord, playerRack, nodeIndex, letters, x, y, orientation, wordList, false);
	if (maxSize > 0)
	{
		int letterIndex = letters[nodeIndex].nextAddress;
		letterIndex--;
		do
		{
			letterIndex++;
			if (playerRack->contains( m_dictionary->decodeLetter(letters[letterIndex].letter)))
			{
				tile *tempLetter = playerRack->removeLetter(m_dictionary->decodeLetter(letters[letterIndex].letter));
				partialWord.append(tilePosition(tempLetter, x, y));
				leftPart(partialWord, playerRack, letterIndex,letters,x,y, orientation, wordList, maxSize-1);
				partialWord.removeLast();
				playerRack->putLetter(tempLetter);
			}
		}
		while(letters[letterIndex].isSequenceEnd == false);
	}
}

void board::extendRight(QList<tilePosition> &partialWord, rack* playerRack, int nodeIndex, letter* letters, int x, int y, Orientation orientation, QList<QList<tilePosition*> *> *wordList, bool tileOnAnchor)
{
	quint32 nextPossibleLetter = letters[nodeIndex].nextAddress;
	if ((nodeIndex == 0) && (playerRack->started())) return;
	tile* tempLetter;
	int dx = 0;
	int dy = 0;
	if (orientation == HORIZONTAL) dx = 1;
	else dy = 1;
	if ((x<15) && (y<15) && (m_squares[x][y].state() == square::TILE))
	{
		qint8 letterOnBoard = m_dictionary->letterCode((m_squares[x][y].ourTile()->string()));
		nextPossibleLetter--;
		do
		{
			nextPossibleLetter++;
			if (letters[nextPossibleLetter].letter == letterOnBoard)
			{
				tempLetter = m_squares[x][y].ourTile();
				partialWord.append(tilePosition(tempLetter, x, y));
				extendRight(partialWord, playerRack, nextPossibleLetter, letters, x+dx, y+dy, orientation, wordList, tileOnAnchor);
				partialWord.removeLast();
				//TODO: check if letter disapear from board
			}
		}
		while(letters[nextPossibleLetter].isSequenceEnd == false);
	}
	else
	{
		if ((letters[nodeIndex].isWordEnd) && (nodeIndex > 0) && tileOnAnchor)
		{
			QString checkWord = "";
			for (int i = 0; i < partialWord.count(); i++ )
			{
				QString temp = ((partialWord.at(i)).ourTile())->string();
				int i = 0;
				i++;
				checkWord.append(temp);
			}
			if (m_dictionary->isCorrect(checkWord) == false) return;

			//copy tiles
			QList<tilePosition*> *copy = new QList<tilePosition*>();
			for (int i=0; i < partialWord.size(); i++)
			{
				tile* tempTile = new tile(partialWord[i].ourTile()->string());
				tempTile->setScore( partialWord[i].ourTile()->score() );
				tempTile->setBlank( partialWord[i].ourTile()->isBlank() );
				tilePosition* tempPosition = new tilePosition(tempTile, partialWord[i].x(), partialWord[i].y());
				copy->append(tempPosition);
			}
			wordList->append(copy);
			//wordList.append(partialWord);
		}
		if ((x > 14) || (y > 14)) return;
		nextPossibleLetter--; //to take off first ++
		do
		{
			nextPossibleLetter++;
			//next possible letter is in rack and is good cross letter...
			if (playerRack->contains( m_dictionary->decodeLetter(letters[nextPossibleLetter].letter)) &&
			m_squares[x][y].crossLetterState( letters[nextPossibleLetter].letter, orientation ))
			{
				tempLetter = playerRack->removeLetter(m_dictionary->decodeLetter(letters[nextPossibleLetter].letter));
				partialWord.append(tilePosition(tempLetter, x, y));
				extendRight(partialWord, playerRack, nextPossibleLetter, letters, x+dx, y+dy, orientation, wordList, true);
				partialWord.removeLast();
				playerRack->putLetter(tempLetter);
			}
		}
		while(letters[nextPossibleLetter].isSequenceEnd == false);
	}
}

int board::pointsForMove(QList<tilePosition*> &tiles)
{
	QList<tilePosition> tilesToSet;
	foreach (const tilePosition *tp, tiles)
	{
		tilesToSet << *tp;
	}
	setPlayedTiles(tilesToSet);

	int minX, minY, maxX, maxY;
	bool haligned = false;
	bool valigned = false;
	
	//allign check
	minY = maxY = tiles.at(0)->y();
	minX = maxX = tiles.at(0)->x();
	for (int i = 1; i < tiles.size(); i++)
	{
		if (minY > tiles.at(i)->y()) minY = tiles.at(i)->y();
		else if (maxY < tiles.at(i)->y()) maxY = tiles.at(i)->y();

		if (minX > tiles.at(i)->x()) minX = tiles.at(i)->x();
		else if (maxX < tiles.at(i)->x()) maxX = tiles.at(i)->x();
	}
	haligned = (minY == maxY);
	valigned = (minX == maxX);

	if (haligned && valigned)
	{
		bool newAlign = false;
		if (minX > 0) newAlign |= (m_squares[minX-1][minY].ourTile() != 0);
		if (minX < 14) newAlign |= (m_squares[minX+1][minY].ourTile() != 0);
		haligned = newAlign;
		newAlign = false;
		if (minY > 0) newAlign |= (m_squares[minX][minY-1].ourTile() != 0);
		if (minY < 14) newAlign |= (m_squares[minX][minY+1].ourTile() != 0);
		valigned = newAlign;
	}
	
	int score = 0;
	int wordmod = 1;
	int tmpscore = 0;
	int crossScore = 0;
	foreach (const tilePosition *tp, tiles)
	{
		const tile *t = tp->ourTileC();
		int tmpletterscore = t->score();
		int crossMod = 1;
		if (!m_squares[tp->x()][tp->y()].isLocked())
		{
			switch 	(m_squares[tp->x()][tp->y()].kind())
			{
				case square::CENTER:
				case square::DOUBLE_WORD:
					wordmod *= 2; crossMod = 2;
				break;
				case square::TRIPLE_WORD:
					wordmod *= 3; crossMod = 3;
				break;
				case square::DOUBLE_LETTER:
					tmpletterscore *= 2;
				break;
				case square::TRIPLE_LETTER:
					tmpletterscore *= 3;
				break;
				case square::NORMAL:
				break;
			}

			if (haligned && m_squares[tp->x()][tp->y()].verticalPoints() > 0)
				crossScore += (tmpletterscore + m_squares[tp->x()][tp->y()].verticalPoints()) * crossMod;
			else if (valigned && m_squares[tp->x()][tp->y()].horizontalPoints() > 0)
				crossScore += (tmpletterscore + m_squares[tp->x()][tp->y()].horizontalPoints()) * crossMod;
		}
		tmpscore += tmpletterscore;
	}
	score += tmpscore * wordmod + crossScore;
	if (tiles.size() == 7) score += 50;

	finalizePlayedTiles(DELETE);

	//kDebug() << "Points for move: " << score << endl;
	return score;
}

void board::setTileStringList(const QList< QPair<QString, int> > &tileStrings)
{
	m_tilesStringList = tileStrings;
}

void board::setDictionary(dictionary *dict)
{
	m_dictionary = dict;
}

void board::reset()
{
	for (int i = 0; i < 15; i++)
	for (int j = 0; j < 15; j++)
	{
		m_squares[i][j].reset();
	}
	m_squares[ 7][ 7].setToAnchor();
}
