/****************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>           *
 * Copyright (C) 2007-2008 by Michal Sciubidlo <michal.sciubidlo@gmail.com> *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License, or      *
 *   (at your option) any later version.                                    *
 ****************************************************************************/

#ifndef BOARDWIDGET_H 
#define BOARDWIDGET_H 

#include <QPoint>
#include <QList>
#include <QGraphicsItem>
#include <qwidget.h>

#include "board.h"
#include "square.h"
#include "tileposition.h"

class blankTileDialog;
class QMenu;

class boardWidget : public QObject, public QGraphicsItem, public board
{
Q_OBJECT
#if QT_VERSION >= 0x040600
    Q_INTERFACES(QGraphicsItem)
#endif	
	public:
		boardWidget(blankTileDialog * dlg, QMenu * itemMenu);
		~boardWidget();

		void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
		QRectF boundingRect() const;
		void resize( double size );

		void setTileStringList(const QList< QPair<QString, int> > &tileStrings);
		const QList< QPair<QString, int > > &getTileStringList() const;
 
		void removeDroppedTile(tile * t = 0);
		bool wereTilesAdded() const;
		tilePositionList tilesAdded() const;

		void reset();
// 
//		static double tileFontSize;

	public slots:
		/*!
		 * Resume drag drop and moving tiles.
		 */
		void nextPlayer(const QString &nick, bool isItMe);
		/*!
		 * Sets tiles send by other player.
		 */
		void slotSetPlayerTiles(const tilePositionList &tiles);
		/*!
		 * Sent after lost voting. Removes tiles from other player or ours.
		 */
		void rejectPlayerTiles();
		/*!
		 * Sent after good move has been made. Should lock tiles on boarda and find cross letters/points.
		 */
		void slotLockTilesAdded();
		/*!
		 * Deletes tiles added by slotSetPlayerTiles()
		 */
		void slotDeleteLastTiles();

	signals:
		void tileSizeChanged(double);
		void moveTileAway(tile *);

	protected:
		void mousePressEvent(QGraphicsSceneMouseEvent *event);
		void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
		void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

		virtual void dragEnterEvent(QGraphicsSceneDragDropEvent * event);
		virtual void dragMoveEvent(QGraphicsSceneDragDropEvent * event);
		virtual void dropEvent(QGraphicsSceneDragDropEvent * event);

//		virtual void resizeEvent(QResizeEvent *event);
//		virtual void contextMenuEvent(QContextMenuEvent *e);

	private:
		void removeTileFromNewlyAdded(const QPoint &p);
		tilePositionList m_addedtiles;
//		void checkFontSize() const;
//		bool tilesFitIn(double pointSize) const;
		QMenu * m_menu;
		blankTileDialog * m_dlg;
		double m_tileSize;
		QPoint m_activeSquare;
		QPoint m_startdrag;
};

#endif
