/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "confirmwidget.h"
#include "miniprogressbar.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QToolTip>
#include <QVBoxLayout>

#include <kguiitem.h>
#include <klocale.h>
#include <kpushbutton.h>

confirmWidget::confirmWidget(QWidget *parent) : QFrame(parent)
{
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	setPalette(QToolTip::palette());
	setFrameStyle(Box);
	setFrameShadow(Plain);
	setLineWidth(1);

	QHBoxLayout *mainlay = new QHBoxLayout(this);
	mainlay->setMargin(10);
	mainlay->setSpacing(5);

	QVBoxLayout *lay = new QVBoxLayout();
	lay->setSpacing(10);
	mainlay->addLayout(lay);
	
	m_text = new QLabel(this);
	m_text->setWordWrap(true);
	lay->addWidget(m_text);

	lay->addItem(new QSpacerItem(5, 5, QSizePolicy::Fixed, QSizePolicy::Expanding));

	QHBoxLayout *hlay = new QHBoxLayout();
	hlay->setSpacing(10);
	lay->addLayout(hlay);

	hlay->addItem(new QSpacerItem(5, 5, QSizePolicy::Expanding, QSizePolicy::Fixed));

	m_reject = new KPushButton(this);
	m_reject->setGuiItem(KGuiItem(i18n("&Reject"), "button_cancel"));
	hlay->addWidget(m_reject);

	m_accept = new KPushButton(this);
	m_accept->setGuiItem(KGuiItem(i18n("&Accept"), "button_ok"));
	hlay->addWidget(m_accept);

	m_progressBar = new miniProgressBar(Qt::Vertical, this);
	m_progressBar->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
	m_progressBar->setProgress(100.0);
	mainlay->addWidget(m_progressBar);

	connect(m_accept, SIGNAL(clicked()), this, SLOT(acceptClicked()));
	connect(m_reject, SIGNAL(clicked()), this, SLOT(rejectClicked()));
	connect(&m_timer, SIGNAL(timeout()), this, SLOT(timeoutOver()));

	m_nick = QString();
	m_timeout = -1;
	m_timer.setSingleShot(false);
	m_timer.setInterval(150);

	resetText();
}

void confirmWidget::setTimeout(int seconds)
{
	if (m_timer.isActive())
		m_timer.stop();
	m_progressBar->setProgress(100.0);
	m_timeout = seconds;
}

int confirmWidget::timeout() const
{
	return m_timeout;
}

void confirmWidget::setNick(const QString &nick)
{
	m_nick = nick;
	resetText();
}

void confirmWidget::startTimeout()
{
	stopTimeout();
	if (m_timeout > 0)
		m_timer.start();
}

void confirmWidget::stopTimeout()
{
	if (m_timer.isActive())
		m_timer.stop();
}

void confirmWidget::acceptClicked()
{
	stopTimeout();
	m_progressBar->setProgress(100.0);
	emit accepted();
}

void confirmWidget::rejectClicked()
{
	stopTimeout();
	m_progressBar->setProgress(100.0);
	emit rejected();
}

void confirmWidget::timeoutOver()
{
	double time = m_progressBar->progress() - 100 * m_timer.interval() / (m_timeout * 1000);
	m_progressBar->setProgress(time);
	if (time <= 0.0)
	{
		acceptClicked();
	}
}

void confirmWidget::resetText()
{
	m_text->setText(QLatin1String("<html><body>") + i18n("<b>%1</b> sent the highlighted tiles.<br>Do you want to accept or reject the word they have created?<br>If you do not answer, we will assume you accept it.", m_nick) + QLatin1String("</body></html>"));
}

#include "confirmwidget.moc"
