/****************************************************************************
 * Copyright (C) 2007-2008 by Michal Sciubidlo <michal.sciubidlo@gmail.com> *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License, or      *
 *   (at your option) any later version.                                    *
 ****************************************************************************/
#include "kombinationscene.h"
#include "square.h"
#include "tilewidget.h"
#include "boardwidget.h"
#include "blanktiledialog.h"
#include "renderer.h"

#include <klocale.h>

#include <QMenu>
#include <QAction>
#include <QResizeEvent>
#include <QGraphicsPixmapItem>

KombinationScene::KombinationScene(QObject *parent)
     : QGraphicsScene(parent)
{
	QMenu *m_itemMenu = new QMenu((QWidget*)parent);
	QAction *act = m_itemMenu->addAction(i18n("&Edit Blank Tile..."));
	act->setData(QVariant(0));
	act = m_itemMenu->addAction(i18n("&Edit Blank Tile..."));
	act->setData(QVariant(0));

	blankTileDialog * m_dlg = new blankTileDialog((QWidget*)parent);

	m_boardWidget = new boardWidget(m_dlg, m_itemMenu);
	addItem(m_boardWidget);
	m_boardWidget->setPos(0, 0);
	m_boardWidget->setAcceptsHoverEvents(true);
	m_boardWidget->setAcceptedMouseButtons(Qt::LeftButton | Qt::RightButton);
	m_boardWidget->setAcceptDrops(true);

	m_rackWidget = new tileWidget(7);
	addItem(m_rackWidget);
	m_rackWidget->setPos(0, 15 * 32);
	m_rackWidget->setAcceptsHoverEvents(true);
	m_rackWidget->setAcceptedMouseButtons(Qt::LeftButton | Qt::RightButton);
	m_rackWidget->setAcceptDrops(true);

	m_exchangeRack = new tileWidget(7);
	addItem(m_exchangeRack);
	m_exchangeRack->setPos(8*32, 15 * 32);
	m_exchangeRack->setAcceptsHoverEvents(true);
	m_exchangeRack->setAcceptedMouseButtons(Qt::LeftButton | Qt::RightButton);
	m_exchangeRack->setAcceptDrops(true);
}

KombinationScene::~KombinationScene()
{
}

void KombinationScene::resize(int width, int height)
{
	double minSize = (double)width / 15.0;
	double minHeight = (double)height / 16.0; //16 = 15 for board and 1 for racks
	if (minSize > minHeight)
		minSize = minHeight;
	setSceneRect(0, 0, width, height);
	m_boardWidget->resize( minSize );
	m_boardWidget->setPos(0, 0);
	m_rackWidget->resize( minSize );
	m_rackWidget->setPos(0, 15 * minSize);
	m_exchangeRack->resize( minSize );
	m_exchangeRack->setPos(8 * minSize, 15 * minSize);
	renderer::self()->resize( minSize );
}

KombinationView::KombinationView( KombinationScene* scene, QWidget *parent )
    : QGraphicsView(scene, parent), m_scene(scene)
{
}

void KombinationView::resizeEvent( QResizeEvent *ev )
{
    m_scene->resize( ev->size().width(), ev->size().height() );
}
