/***************************************************************************
 *   Copyright (C) 2006 by Pino Toscano <toscano.pino@tiscali.it>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "kombinationserver.h"
#include "chatwidget.h"
#include "eventslogwidget.h"
#include "protocol.h"
#include "server.h"
#include "settings.h"
#include "tilefactory.h"

#include <qlayout.h>
#include <QLineEdit>
#include <QPushButton>
#include <QSplitter>
#include <QTableView>

#include <kaction.h>
#include <kglobal.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandardgameaction.h>
#include <kicon.h>
#include <kactioncollection.h>

KombinationServer::KombinationServer() : KXmlGuiWindow(), m_server(0), m_ready(false)
{
	QWidget *w = new QWidget(this);
	QVBoxLayout *lay = new QVBoxLayout(w);

	QSplitter *splitter = new QSplitter(w);
	lay->addWidget(splitter);

	QSplitter *foo1 = new QSplitter(Qt::Vertical, splitter);
	m_gamesView = new QTableView(foo1);
	foo1->addWidget(m_gamesView);

	m_eventsLog = new eventsLogWidget(foo1);
	foo1->addWidget(m_eventsLog);

	splitter->addWidget(foo1);

	QWidget *foo2 = new QWidget(w);
	QVBoxLayout *foo2lay = new QVBoxLayout(foo2);
	foo2lay->setMargin(0);

	m_chatWidget = new chatWidget(foo2);
	m_chatWidget->setEnabled(false);
	foo2lay->addWidget(m_chatWidget);

	splitter->addWidget(foo2);
	
	m_actStartServer = actionCollection()->addAction("startServer");
	m_actStartServer->setText( i18n("Start the Server"));
	m_actStartServer->setIcon(KIcon("launch"));
	connect(m_actStartServer, SIGNAL(triggered()), this, SLOT(startServer()));
	
	m_actStopServer = actionCollection()->addAction("stopServer");
	m_actStopServer->setText(i18n("Stop the Server"));
	m_actStopServer->setIcon(KIcon("process-stop"));
	m_actStopServer->setEnabled(false);
	connect(m_actStopServer, SIGNAL(triggered()), this, SLOT(disableServer()));
	QAction *a = KStandardGameAction::quit(this, SLOT(close()), actionCollection());
	actionCollection()->addAction("quit", a);

	setCentralWidget(w);
	setupGUI(Keys | Save | Create);
	show();
}

KombinationServer::~KombinationServer()
{
	disableServer();
}

void KombinationServer::startServer()
{
	bool ok = true;
	int port = KInputDialog::getInteger(i18n("Port of the Game"), i18n("Choose the port for the server:"), Settings::serverPort(), 0, 32000, 1, &ok, this);
	if (ok)
	{
		// paranoia check
		if (m_server)
		{
			if (KMessageBox::questionYesNo(this, i18n("There is a server already running, are you sure you want to restart it?"), i18n("Server already running")) == KMessageBox::No) return;
		}
		disableServer();
		
		int nPlayers = KInputDialog::getInteger(i18n("Number of players"), i18n("Choose the maximum number of players for the server:"), 1000, 2, 32000, 1, &ok, this);
		if (ok)
		{
			m_server = new server(nPlayers, server::MULTISERVER);
			m_gamesView->setModel(m_server);
			m_port = port;
			Settings::setServerPort(port);
			connect(m_server, SIGNAL(connected()), this, SLOT(serverStarted()));
			connect(m_server, SIGNAL(error(const QString &)), this, SLOT(serverError(const QString &)));
			m_server->start(port);
			Settings::self()->writeConfig();
		}
	}
}

void KombinationServer::serverStarted()
{
	m_ready = true;
	m_actStopServer->setEnabled(true);
	m_actStartServer->setEnabled(false);
	m_eventsLog->addMessage(i18n("Server started"));
}

void KombinationServer::disableServer()
{
	if (!m_server) return;

	m_server->deleteLater();
	m_server = 0;

	m_actStartServer->setEnabled(true);
	m_actStopServer->setEnabled(false);
	if (m_ready)
	{
		m_eventsLog->addMessage(i18n("Server stopped"));
	}
}

void KombinationServer::serverError(const QString &text)
{
	m_eventsLog->addErrorMessage(text);
	disableServer();
//	KMessageBox::error(this, text, i18n("Server Error"));
}

#include "kombinationserver.moc"
