/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef PLAYER_H 
#define PLAYER_H 

#include <QList>

#include "spectator.h"
#include "tileposition.h"

class tile;

/**
 * This is the representation of a player on the server side 
 */ 

class player : public spectator
{
Q_OBJECT
	friend class aiPlayer;
	public:
		enum VotingState
		{
			VOTING_UNKNOWN = 0,
			VOTING_ACCEPTED,
			VOTING_REJECTED
		};

		//explicit player(spectator *s);
		player(spectator *s);
		player();
		~player();
		
		bool ready() const;

		void startGame(const QList<tile *> &tiles);
		void sendNewTiles(const QList<tile *> &tiles);
		void nextTurn(const QString &nick);
		void sendTiles(const tilePositionList &tiles);
		void correctWord(int score);
		void unknownWords();
		//void confirmVotedTiles();
		void rejectVotedTiles(const QString &nick);
		void usesExclusiveTiles(const QString &nick);
		void sendExchangedTiles(const QList<tile*> &tiles);
		void gameEnded(const QString &finisher, const QString &winner);

		int freeTiles() const;

		VotingState votingState() const;
		void prepareForNewVoting();

		int score() const;
		void addToScore(int s);

		bool hasTile(const QString &tileText) const;
	
	signals:
		void readyChanged(player *p);
		void gotTiles(player *p, const tilePositionList &tiles);
		void votingChanged();
		void passTurn(player *p);
		void exchangeTiles(player *p, const QList<tile*> &tiles);

	protected slots:
		void processNewMessage(const message &m);
	
	protected:
		QList<tile *> m_tiles;
		bool m_myturn;
		int m_score;
		VotingState m_votingState;
		bool m_ready;
		QList<tile *> m_tilesIHaveUsedInThisTurn;
		
	private:
		bool checkValidTiles(const QList<tile*> &list);
};

#endif
