/***************************************************************************
 *   Copyright (C) 2007 by Michal Sciubidlo <michal.sciubidlo@gmail.com>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef RENDERER_H
#define RENDERER_H

#include <QPixmap>
#include <QSizeF>
#include "square.h"

class KSvgRenderer;
class KPixmapCache;

class renderer{
	public:
		static renderer* self();
		bool loadTheme(const QString &themeName);
		QPixmap tileBackground(bool isBlank = false);
		QPixmap boardElement(square::Kind elemType);
		QPixmap rackBackground();

		/**
		 * Sets cell render size
		 * Call to this function will reset the cache and put fresh pixmaps to it
		 */
		void setCellSize( const QSizeF &size );
		/**
		 * @return current cell render size
		 */
		QSizeF cellSize() const { return m_size; }
		void resize( double size );
	private:
		renderer();
                Q_DISABLE_COPY(renderer)
		~renderer();

		void loadTile(const QString &cacheName, QPixmap*& dest);

		/**
		 * Renderer used to render all graphics from svg file
		 */
		KSvgRenderer *m_renderer;
		/**
		 * Pixmap cache
		 */
		KPixmapCache *m_cache;

		QPixmap *m_rackBackground;
		QPixmap *m_tileBackground;
		QPixmap *m_blankBackground;
		QPixmap *m_boardBackground[6];

		QString m_currentTheme;
		QSizeF m_size;
};

#endif
