/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "server.h"
#include "game.h"
#include "player.h"
#include "protocol.h"
#include "tileposition.h"
#include "dictionary.h"

#include <QStringList>
#include <QTcpServer>

#include <kdebug.h>
#include <klocale.h>
#include <krandom.h>

#include <stdlib.h>

//on windows #include "winsock2.h" ??
#include <sys/socket.h>

/* Some template functions used to operate over
	QList<player *> m_clients;
	QList<spectator *> m_spectators;
   I'm not sure if that can be done in a non template way, but it works this way :-D
*/

/*
template<class x>
void sendMessage(const QList<x> &list, const spectator *s, const QString &message)
{
	foreach(x it, list)
		if (it != s) it->sendMessage(s->nick(), message);
}*/
/*
template<class x>
void removePlayer(const QList<x> &list, const QString &nick)
{
	foreach(x it, list)
		it->removePlayer(nick);
}
*/
/* server */

server::server(int maxPlayers, eKindOfServer type) : m_gameId(0), m_maxPlayers(maxPlayers), m_type(type)
{
	//m_serverSocket = new KNetwork::KServerSocket(QString::number(port));
	//connect(m_serverSocket, SIGNAL(readyAccept()), this, SLOT(slotReadyAccept()));
	//connect(m_serverSocket, SIGNAL(gotError(int)), this, SLOT(slotSocketError(int)));
	//connect(m_serverSocket, SIGNAL(bound(const KNetwork::KResolverEntry &)), this, SIGNAL(connected()));
	m_serverSocket = new QTcpServer(this);

	int sockd = m_serverSocket->socketDescriptor();
	int turnOn = 1;
	setsockopt(sockd, SOL_SOCKET, SO_REUSEADDR, &turnOn, sizeof(turnOn));

	connect(m_serverSocket, SIGNAL(newConnection()), this, SLOT(slotReadyAccept()));
}

server::~server()
{
	m_serverSocket->close();
	delete m_serverSocket;
}

void server::start(int port)
{
	if (m_serverSocket->listen(QHostAddress::Any, port) == false)
	{
		emit error(i18n("Could not start the server because the address is in use. Maybe you already have a server running?"));
		return;
	}
	emit connected();
}

int server::rowCount(const QModelIndex &) const
{
	return m_games.count();
}

int server::columnCount(const QModelIndex &) const
{
	return 8;
}

QVariant server::data(const QModelIndex &index, int role) const
{
	if (!index.isValid()) return QVariant();
	
	int column, row;
	column = index.column();
	row = index.row();
	
	if (role != Qt::DisplayRole) return QVariant();
	
	game *g = m_games[row];
	switch (column)
	{
		case 0: return g->name(); break;
		case 1: return g->numberOfPlayers(); break;
		case 2: return g->numberOfSpectators(); break;
		case 3: return g->maxNumberOfPlayers(); break;
		case 4: return g->hasStarted(); break;
		case 5: return g->allowsSpectators(); break;
		case 6: return g->comment(); break;
		case 7: return g->language(); break;
		default: return QVariant(); break;
	}
}

QVariant server::headerData(int section, Qt::Orientation orientation, int role) const
{
	if (role != Qt::DisplayRole) return QVariant();
	
	if (orientation == Qt::Horizontal)
	{
		switch (section)
		{
			case 0: return i18n("Name"); break;
			case 1: return i18n("Players"); break;
			case 2: return i18n("Spectators"); break;
			case 3: return i18n("Max Players"); break;
			case 4: return i18n("Started"); break;
			case 5: return i18n("Allows Spectators"); break;
			case 6: return i18n("Comment"); break;
			case 7: return i18n("Language"); break;
			default: return QVariant(); break;
		}
	}
	else
	{
		if ((section < 0) || (section >= m_games.size())) return QVariant();
		return m_games.at(section)->id();
	}
}

void server::slotReadyAccept()
{
	QString nick;
	QTcpSocket *socket = m_serverSocket->nextPendingConnection();

	spectator *s = new spectator(socket);
	connect(s, SIGNAL(wantToJoin(spectator*)), this, SLOT(wantToJoin(spectator*)));
}

void server::wantToJoin(spectator *s)
{
	if (m_maxPlayers != 0 && m_connectedClients.count() >= m_maxPlayers)
	{
		s->tooManyPlayers();
	}
	else
	{
		connect(s, SIGNAL(gotNick(spectator *)), this, SLOT(addPlayer(spectator *)));
		connect(s, SIGNAL(remotelyDisconnected(spectator *)), this, SLOT(removePlayer(spectator *)));
		connect(s, SIGNAL(gotMessage(spectator *, const QString &)), this, SLOT(sendMessage(spectator *, const QString &)));
		connect(s, SIGNAL(joinGame(spectator *, int, bool)), this, SLOT(addPlayerToGame(spectator *, int, bool)));
		connect(s, SIGNAL(createServerGame(spectator *,const QString &, int, const QString &, const QString &, bool)), this, SLOT(createGame(spectator *,const QString &, int, const QString &, const QString &, bool)));
		s->askForNick();
	}
}

void server::slotSocketError(int e)
{
	/*if (e == KNetwork::KSocketBase::AddressInUse)
	{
		emit error(i18n("Could not start the server because the address is in use. Maybe you already have a server running?"));
	}
	// TODO tell the user
	kDebug() << "SERVER ERROR: " << m_serverSocket->errorString() << endl;*/
}

void server::addPlayer(spectator *s)
{
	bool nickFree = true;
	QList<spectator *>::const_iterator itClients, itClientsEnd;
	itClients = m_connectedClients.constBegin();
	itClientsEnd = m_connectedClients.constEnd();
	while (nickFree && itClients != itClientsEnd)
	{
		if ((*itClients)->nick().toLower() == s->nick().toLower()) nickFree = false;
		else ++itClients;
	}
	
	if (nickFree)
	{
		if (m_type == MULTISERVER)
		{
			// notify the player all the players not playing and games created
			QStringList players;
			itClients = m_connectedClients.constBegin();
			itClientsEnd = m_connectedClients.constEnd();
			for ( ; itClients != itClientsEnd; ++itClients)
			{
				if (!(*itClients)->currentGame()) players << (*itClients)->nick();
			}
			s->acceptedInServer(players, m_games);
		
			// notify all the players we have a new player
			itClients = m_connectedClients.constBegin();
			for ( ; itClients != itClientsEnd; ++itClients)
			{
				(*itClients)->addPlayer(s->nick());
			}
		
			m_connectedClients.append(s);
		}
		else
		{
			if (s->isSpectator())
			{
				addPlayerToGame(s, (m_games.first())->id(), true);
			}
			else
			{
				addPlayerToGame(s, (m_games.first())->id(), false);
			}
		}
	}
	else s->nickInUse();
}

void server::addPlayerToGame(spectator *s, int gameId, bool joinAsSpectator)
{
	QList<game *>::const_iterator itGames, itGamesEnd;
	bool found = false;
	itGames = m_games.constBegin();
	itGamesEnd = m_games.constEnd();
	int row = 0;
	while (!found && itGames != itGamesEnd)
	{
		if ((*itGames)->id() == gameId) found = true;
		else
		{
			++itGames;
			++row;
		}
	}
	if (found)
	{
		spectator *newSpectator = (*itGames)->addPlayer(s, joinAsSpectator);
		if (newSpectator)
		{
			if (s != newSpectator)
			{
				m_connectedClients.removeAll(s);
				m_connectedClients.append(newSpectator);
				
				connect(newSpectator, SIGNAL(gotNick(spectator *)), this, SLOT(addPlayer(spectator *)));
				connect(newSpectator, SIGNAL(remotelyDisconnected(spectator *)), this, SLOT(removePlayer(spectator *)));
				connect(newSpectator, SIGNAL(gotMessage(spectator *, const QString &)), this, SLOT(sendMessage(spectator *, const QString &)));
				connect(newSpectator, SIGNAL(joinGame(spectator *, int, bool)), this, SLOT(addPlayerToGame(spectator *, int, bool)));
			}
			
			// warn all the other players that are not in a game that this player entered game
			foreach(spectator *sIt, m_connectedClients)
			{
				sIt->otherPlayerEnteredAGame(newSpectator->nick(), gameId);
			}
			
			emit dataChanged(index(row, 1), index(row, 1));
		}
	}
	else
	{
		// TODO CHEATER!
	}
}

void server::createGame(spectator *creator, const QString &name, int maxPlayers, const QString &comment, const QString &languageCode, bool allowSpectators, int aiCount)
{
	kDebug() << "Creategame" << endl;
	if (maxPlayers >= 2)
	{
		if (!name.isEmpty())
		{
			if (!findGameWithName(name))
			{
				dictionary *dict;
				//TODO: mutex.lock();
				if (m_dictionaries.contains(languageCode))
				{
					dict = m_dictionaries.value(languageCode);
				}
				else
				{
					dict = new dictionary(languageCode);
					m_dictionaries.insert(languageCode, dict);
				}
				//TODO: mutex.open();
				
				game *g = new game(m_gameId, name, maxPlayers, comment, dict, allowSpectators);

				for (int i = 0; i < aiCount; i++)
				{
					g->addAi("comp1", languageCode);
				}
				beginInsertRows(QModelIndex(), m_games.count(), m_games.count());
				m_games.append(g);
				endInsertRows();
				++m_gameId;
				// TODO check for overflow? use another way of giving id?
				
				foreach(spectator *s, m_connectedClients)
				{
					s->addGame(g);
				}
			}
			else
			{
				creator->canNotCreateGame("repeatedName");
			}
		}
		else
		{
			creator->canNotCreateGame("notEmptyName");
		}
	}
	else
	{
		creator->canNotCreateGame("tooFewMaxPlayers");
	}
}


void server::sendMessage(spectator *s, const QString &message)
{
	foreach(spectator *s2, m_connectedClients)
		if (s->currentGame() == s2->currentGame() && s2->nick() != s->nick()) s2->sendMessage(s->nick(), message);
}

void server::removePlayer(spectator *s)
{
/*	if (!s->wasNickInUse())
	{
		player *p = dynamic_cast<player*>(s);
		if (p)
		{
			int i = m_clients.indexOf(p);
			if (i != -1)
			{
				m_clients.removeAt(i);
				m_lostClients.append(p);
			}
			else
			{
				// TODO something strange happened
			}
		}
		else
		{
			int i = m_spectators.indexOf(s);
			if (i != -1)
				m_spectators.removeAt(i);
			else
			{
				// TODO something strange happened
			}
		}
		
		::removePlayer(m_clients, s->nick());
		::removePlayer(m_spectators, s->nick());
	}
	s->deleteLater();*/
}

game *server::findGameWithName(const QString &name) const
{
	foreach(game *g, m_games)
	{
		if (g->name() == name) return g;
	}
	return 0;
}

#include "server.moc"
