/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "tile.h"

#include <QVariant>

tile::tile()
{
	init(QString());
}

tile::tile(const QString &s)
{
	init(s);
}

void tile::init(const QString &s)
{
	m_string = s;
	m_score = 0;
	m_kind = NORMAL;
	m_isBlank = s.isEmpty();
	m_isBlankSet = !s.isNull();
}

void tile::setString(const QString &s)
{
	m_string = s;
	if (!m_isBlankSet)
	{
		m_isBlank = s.isEmpty();
		m_isBlankSet = true;
	}
}

QString tile::string(bool spaceIfEmpty) const
{
	return m_string.isEmpty() && spaceIfEmpty ? QLatin1String(" ") : m_string;
}

void tile::setBlank(bool b)
{
	m_isBlank = b;
}

bool tile::isBlank() const
{
	return m_isBlank;
}

void tile::setScore(const uint score)
{
	m_score = score;
}

uint tile::score() const
{
	return m_score;
}

void tile::setKind(const tile::Kind k)
{
	m_kind = k;
}

tile::Kind tile::kind() const
{
	return m_kind;
}

bool tile::operator==(const tile &t) const
{
	return m_string == t.string() && m_score == t.score();
}

QMap<QString,QString> tile::toMap(const QString &tag) const
{
	QMap<QString,QString> res;
	res[tag + "string"] = m_string;
	res[tag + "score"] = QString::number(m_score);
	res[tag + "isBlank"] = QString::number(m_isBlank);
	return res;
}

void tile::fromMap(const QMap<QString,QString> &data, const QString &tag)
{
	m_string = data[tag + "string"];
	m_score = data[tag + "score"].toInt();
	m_isBlank = QVariant( data[tag + "isBlank"] ).toBool();
	m_isBlankSet = true;
}

rack::rack(QList<tile*> &tiles)
{
	m_tiles = tiles; //TODO: should i copy them?
	maxCount = m_tiles.count();
}

bool rack::contains(const QString &letter)
{
	int size = m_tiles.size();
	for (int n = 0; n < m_tiles.size(); n++)
	{
		if (m_tiles[n]->isBlank() || (m_tiles[n]->string() == letter))
			return true;
	}
	return false;
}

tile* rack::removeLetter(const QString &letter)
{
	bool haveBlank = false;
	for (int n = 0; n < m_tiles.size(); n++)
	{
		if (m_tiles[n]->isBlank()) haveBlank = true;
		if (m_tiles[n]->string() == letter)
			return m_tiles.takeAt(n);
	}
	if (haveBlank)
	{
		for (int n = 0; n < m_tiles.size(); n++)
		if (m_tiles[n]->isBlank())
		{
			//set blank letter before return
			m_tiles[n]->setString(letter);
			return m_tiles.takeAt(n);
		}
	}

	return 0;
}

void rack::putLetter(tile* newTile)
{
	if (newTile->isBlank())
		newTile->setString("");
	m_tiles.append(newTile); //TODO: copy tile ?
}
