/****************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>             *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>      *
 * Copyright (C) 2007-2008 by Michal Sciubidlo <michal.sciubidlo@gmail.com> *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License, or      *
 *   (at your option) any later version.                                    *
 ****************************************************************************/

#ifndef TILEWIDGET_H
#define TILEWIDGET_H

#include <QPoint>
#include <QObject>
#include <QGraphicsItem>
#include <qwidget.h>
#include <qvector.h>

class tile;
class QPixmap;


class tileWidget : public QObject, public QGraphicsItem
{
Q_OBJECT
#if QT_VERSION >= 0x040600
    Q_INTERFACES(QGraphicsItem)
#endif	
	public:
		tileWidget(int nTiles);
		~tileWidget();

		void removeDroppedTile(tile *t = 0);

		int freeSpaces() const;
		bool isFull() const;
		const QVector<tile*> &tiles() const;
		//void setTiles(const QVector<tile*> &tiles);
		void removeTiles(const QVector<tile*> &tiles);
		void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
		QRectF boundingRect() const;
		void resize( double size );
		void reset();

	public slots:
		//void changeTileSize(double size);

		void addTile(tile *t);
	
	signals:
		void numTilesChanged(int numFreeSpaces);

	protected:
		void mousePressEvent(QGraphicsSceneMouseEvent *event);
		void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
		void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

		virtual void dragEnterEvent(QGraphicsSceneDragDropEvent * event);
		virtual void dragMoveEvent(QGraphicsSceneDragDropEvent * event);
		virtual void dropEvent(QGraphicsSceneDragDropEvent * event);

	private:
		void setTile(tile *t, int index);
		void insertAndShift(tile *t, int pos, int oldpos);

		QVector<tile*> m_tiles;
		double m_tileSize;
		QPoint m_startdrag;
		int m_oldDragIndex;
};

#endif
