/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "qlabel.h"
#include "qlayout.h"
#include "qpixmap.h"
#include "qpainter.h"
#include "qfont.h"
#include "qcolor.h"
#include "qbitmap.h"
#include "qcursor.h"
#include "cup.h"

#include <kapplication.h>
#include <kstandarddirs.h>
#include <kglobalsettings.h>

QBitmap* mask = NULL;

#define CUPX 20
#define INDW 28
#define INDH 16

Cup::Cup(QWidget* parent, int active)
: QPushButton( parent )
{
  if(active) setCursor( QCursor(Qt::PointingHandCursor) );

  cup.load("imgs/cup.png", "PNG");
  setFixedSize( cup.width()+CUPX, cup.height() );
  number=0;
  indicator=0;
  movecolor=0;

  if( mask == NULL ) createMask();
  else setMask( *mask );
  setBackgroundMode( NoBackground );

}

void
Cup::createMask()
{
  int w  = width();
  int h  = height();

  QBitmap buffer( w, h );

  QPainter p;
  p.begin( &buffer );
    p.setBrush( QColor( color0 ) );
    p.eraseRect     ( 0,0,width(),height());
    p.drawPixmap    ( CUPX,0, QBitmap( locate("imgs","cupmask.xpm"), "XPM" ) );
    p.setBrush( QColor( color1 ) );
    p.drawRect      ( 0,0,INDW,INDH );
  p.end();

  mask = new QBitmap(buffer); //buffer.createHeuristicMask();
  setMask(buffer);
}

void
Cup::reset(int n)
{
  movecolor=0;
  indicator=0;
  number=n;
  update();
}

void
Cup::setRedIndicator( int tf )
{
  movecolor = tf;
}

void
Cup::paintEvent( QPaintEvent* )
{
  int w  = width();
  int h  = height();

  if(mask->isNull()) createMask();

  QString numstr;
  numstr.sprintf("%d",number);

  QString numstr2;
       if(indicator>0) numstr2.sprintf("+%d",   indicator );
  else if(indicator<0) numstr2.sprintf("- %d", -indicator );
  else                 numstr2.sprintf("0");

  QPixmap buffer(width(),height());
  //buffer.setMask(*mask);

  QPainter p;

  p.begin( &buffer, this );
    p.setBackgroundMode( QPainter::TransparentMode );
    p.eraseRect     ( 0,0,width(),height());
    p.drawPixmap    ( CUPX,0, cup);
    p.setFont       ( QFont("arial",30,QFont::Bold) );
    p.setPen        ( QColor( 0xffffff ) );
    p.drawText      ( CUPX,0,w-CUPX,h, Qt::AlignCenter, numstr );

    if(movecolor) p.setBrush ( QColor( 0xcc3300 ) );
    else          p.setBrush ( QColor( 0x333300 ) );

    p.drawRect      ( 0,0,INDW,INDH );
    p.setFont       ( QFont("arial",10,QFont::Bold) );
    p.drawText      ( 0,0,INDW,INDH, Qt::AlignCenter, numstr2 );
  p.end();

  p.begin(this);
    p.setBackgroundMode( QPainter::TransparentMode );
    p.drawPixmap(0,0,buffer);
  p.end();
}

#include "cup.moc"


