/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GAMEWIDGET_H
#define GAMEWIDGET_H

#include "stdio.h"
//#include "unistd.h"

#include "qwidget.h"
#include "qpushbutton.h"
#include "qlcdnumber.h"
#include "qlabel.h"
#include "qapplication.h"
#include "qlayout.h"
#include "qmessagebox.h"
#include "qmenubar.h"
#include "qlabel.h"
#include "qslider.h"
#include "qtimer.h"

#include "owarewizard.h"
#include "scorelabel.h"
#include "cup.h"

extern "C"{
#include "game.h"
#include "alphabeta.h"
};

#define HUMGOESFIRST 0
#define WIZGOESFIRST 1 
#define RNDGOESFIRST 2

class
GameWidget :  public QWidget
{
  Q_OBJECT

  public:
    GameWidget ( QWidget* parent=0, const char* name=0 );

    void move1( int i );
    void move2(       );
    void onGameOver();
    int  onmove(int distcup);
    void onEndAnimation1();
    void onEndAnimation2();

    void resizeEvent(QResizeEvent* r);
    inline QSize sizeHint() const { return QSize(backgroundPixmap()->width(),backgroundPixmap()->height()); }     

    OwareWizard* owizard;
    ScoreLabel*  p1ScoreLabel;
    ScoreLabel*  p2ScoreLabel;
    Cup*         cups[12];
    QPushButton* buttons[6];

    struct game  game;
    int          aidepth;
    bool         endoptimize;
    int          heuristic;

    struct game  saved[1024];
    int          saved_top;
    int          redo_top;

    int          animate_distcup;
    int          animate_remaining;
    int          animate_cursor;
    QTimer*      animate_timer;
    int          animating;

    int          whogoesfirst;


  public slots:
    void pressed0();
    void pressed1();
    void pressed2();
    void pressed3();
    void pressed4();
    void pressed5();

    void newGame();
    void animateCups();
    void undo();
    void redo();

    inline void setAiDepth(int d)              { aidepth=d;         };
    inline void setEndOptimize(bool onoff)     { endoptimize=onoff; };
    inline void setDefenseHeuristic(bool onoff){ heuristic = (onoff==false) ? 0 : 1; };
    inline void setWhoGoesFirst(int wgf )      { whogoesfirst = wgf; }
};

#define saved_push(game) saved[saved_top++]=game
#define saved_pop()      saved[--saved_top]
#define saved_last()     saved[saved_top-1]

#endif


