/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "mainwindow.h"
#include "qpixmap.h"
#include "qinputdialog.h"
#include "qmessagebox.h"
#include "qfile.h"
#include "qsplashscreen.h"
#include "qtimer.h"

#include <kapplication.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <kstandarddirs.h>
#include <kcmdlineargs.h>

static const char description[] = I18N_NOOP("The Ancient African Mathematical Strategy Game");

static const char toot[]        = 
I18N_NOOP(
  "Oware is a game of great antiquity, and has been played around the world \n"
  "in various forms for millenia. Currently it is the official board game of \n"
  "Africa, and the most popular mathematical game in the world. The Oware \n"
  "Wizard was created by Michael Collins and features an AI opponent that \n"
  "help you learn and excel at this ancient game. \n\n"
  "Michael Collins is a software engineer from Buffalo, NY USA. He is \n"
  "currently seeking employment and/or business opportunities related to \n"
  "software engineering and executive management at a forward thinking \n"
  "technology company. \n\n"
  "He has a Masters Degree in Business Administration (MBA) and is a creative,\n"
  "visionary, and persistent professional. \n\n"
  "michaelcollins@ivorycity.com"
);


int
main( int argc, char** argv )
{
  KAboutData aboutData
  ( 
    "oware", 
    I18N_NOOP("The Oware Wizard"), 
    "1.0.10", 
    "A modern AI version of the ancient African game", 
    KAboutData::License_GPL, 
    "(c) 2003-2005, Michael Collins",
    toot,
    "http://oware.ivorycity.com",
    "michaelcollins@ivorycity.com"
  );

  aboutData.addAuthor("Michael Collins", 0, "michaelcollins@ivorycity.com", "http://oware.ivorycity.com");
  KCmdLineArgs::init( argc, argv, &aboutData );

  KApplication a;
  KGlobal::locale()->insertCatalogue("libkdegames");
  KGlobal::dirs()->addResourceType("imgs", KStandardDirs::kde_default("data") + "koware/imgs/");

  QSplashScreen* spl = new QSplashScreen( QPixmap( locate("imgs","startsplash.png") ) );
  spl->show(); 
  
  MainWindow* mw = new MainWindow(0,0);
  sleep(2);  
  spl->finish(mw);
    
  QDesktopWidget *d = QApplication::desktop();
  int dw = d->width();     
  int dh = d->height(); 

  mw->setCaption ( i18n("The Oware Wizard") );
  mw->setIconText( i18n("The Oware Wizard") );

  mw->setIcon(QPixmap( locate("imgs","owareicon16x16.png") ));
  a.setMainWidget( mw );
  mw->move( (dw-mw->width())/2,(dh-mw->height())/2);    
  mw->show();
  return a.exec();

  return 0;
}

