/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#ifndef _LAUNCH_H_
#define _LAUNCH_H_

#include <qobject.h>
#include <qstring.h>

#include <kprocess.h>

class Server;
class GameManager;

/**
 * Class used to launch the client game (binary)
 * and connect to a server
 *
 * @short Game launcher
 */
class Launch: public QObject
{
    Q_OBJECT
public:
    /**
     * CTOR
     */
    Launch( GameManager * manager );

    /**
     * DTOR
     */
    ~Launch();

    /**
     * Set @p password for locked servers
     */
    void setPassword( const QCString & password );

    /**
     * Unset the password
     */
    void clearPassword();

    /**
     * Connect to a generic Quake-family server (Q3, ET, MOHAA, etc)
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool quake_exec( Server * server, const QString & command );

    /**
     * Connect to an Unreal Tournament family server (UT, UT 200?)
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool ut_exec( Server * server, const QString & command );

    /**
     * Connect to an Unreal Tournament family server (UT, UT 200?)
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool savage_exec( Server * server, const QString & command );

    /**
     * Connect to an Unreal Tournament family server (UT, UT 200?)
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool netpanzer_exec( Server * server, const QString & command );

    /**
     * Connect to a Battlefield 1942 family server
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool bf1942_exec( Server * server, const QString & command );

    /**
     * Connect to a Descent 3 server
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool descent3_exec( Server * server, const QString & command );

    /**
     * Connect to a Tribes 2 server
     * @param server server to connect to
     * @param command full path to the binary used to launch the game
     * @return true on successul binary start
     */
    bool t2_exec( Server * server, const QString & command );

private slots:
    /**
     * Invoked when the client game exits, displays an error dialog box
     * in case the process (m_process) failed
     */
    void slotProcessExited( KProcess * proc );

private:
    GameManager * m_manager;
    KProcess * m_process;
    QCString m_password;
};

#endif
