/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#ifndef _QSTATIFACE_H_
#define _QSTATIFACE_H_

#include "gameManager.h"

#include <qobject.h>
#include <qdom.h>

class KProcess;

/**
 * This class does all the grunt work with interfacing to qstat CLI.
 *
 * This class is fully async and uses KProcess to handle interaction with
 * the qstat binary. The public methods provide entry points to get
 * information about the servers; when a result is ready, connect to the
 * signal result(pid_t, const QDomElement &) to obtain the actual data.
 *
 * @short qstat interface class
 */
class QStatIface: public QObject
{
    Q_OBJECT
public:
    /**
     * CTOR
     */
    QStatIface( GameManager * manager ):
        m_manager( manager ) {  }
    virtual ~QStatIface();

    /**
     * @return the server list for a game specified by @p gameID
     */
    pid_t getServerList( const QString & gameID );

    /**
     * @return the server stats and its players
     */
    pid_t getServerInfo( const QString & server, const QString & gameID );

signals:
    /**
     * Emitted when the result buffer is ready
     * @return the @p pid of the process and the resulting QDomElement @p elem
     */
    void result( pid_t pid, const QDomElement & elem );

private slots:
    void slotRecvdStdout( KProcess *, char * buf, int len );
    void slotProcessExited( KProcess * proc );

private:
    /**
     * Core stuff, call the qstat CLI and fill m_buffer with resulting XML
     * @return pid of the child qstat process, or 0 in case of errors
     * @param args additional arguments; server IP is expected as the first one, if present
     */
    pid_t callQStat( const QStringList & args );

    QString m_buffer;
    /// transfered pointer to the game manager
    GameManager * m_manager;
};

#endif
