/****************************************************************
Copyright 2008  Matt Williams matt@milliams.com
Copyright 2008  Parker Coates parker.coates@gmail.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************/

#include "kscoreboarddialog.h"

#include "kscoremanager.h"
#include "kscoretable.h"

#include <KDebug>
#include <KGlobal>
#include <KLocale>
#include <KTabWidget>


class KScoreBoardDialog::KScoreBoardDialogPrivate
{
    public:
        KScoreBoardDialogPrivate(KScoreBoardDialog* parent);
        void addTab(const QString& groupKey);
        void aboutToShow();
        void newScore(const QString& groupKey, int rank);

        KScoreBoardDialog * const q; //Q-Pointer
        QMap<QString, KScoreTable*> tables;
        KTabWidget * tabWidget;
};

KScoreBoardDialog::KScoreBoardDialogPrivate::KScoreBoardDialogPrivate(KScoreBoardDialog* parent)
  : q(parent)
{
}

void KScoreBoardDialog::KScoreBoardDialogPrivate::addTab(const QString& groupKey)
{
    KScoreTable * table = new KScoreTable(groupKey);

    tables.insert(groupKey, table);
    tabWidget->addTab(table, q->groupNames().value(groupKey));
}

void KScoreBoardDialog::KScoreBoardDialogPrivate::aboutToShow()
{
    if (q->nameHasBeenRequested() && tables.contains(q->newestScore().first))
    {
        tabWidget->setCurrentWidget( tables.value(q->newestScore().first) );
        q->setButtons( KDialog::Ok );
    }
    else
    {
        q->setButtons( KDialog::Close );
    }
}

void KScoreBoardDialog::KScoreBoardDialogPrivate::newScore(const QString& groupKey, int rank)
{
    Q_UNUSED(rank)
    if (!tables.contains(groupKey))
    {
        addTab(groupKey);
    }
    aboutToShow();
}



KScoreBoardDialog::KScoreBoardDialog()
  : d(new KScoreBoardDialogPrivate(this))
{
    d->tabWidget = new KTabWidget(this);
    setMainWidget(d->tabWidget);

    foreach(const QString& groupKey, groups())
    {
        d->addTab(groupKey);
    }

    connect(KScoreManager::self(), SIGNAL(newScore(QString,int)), this, SLOT(newScore(QString,int)));
}

KScoreBoardDialog::~KScoreBoardDialog()
{
    delete d;
}

void KScoreBoardDialog::setDisplayOptions(const KScoreBoardBase::DisplayOptions & options)
{
    foreach (KScoreTable * table, d->tables)
    {
        table->setDisplayOptions(options);
    }
    KScoreBoardBase::setDisplayOptions(options);
}

void KScoreBoardDialog::switchToGroup(const QString& groupKey)
{
    if (d->tables.contains(groupKey))
    {
        d->tabWidget->setCurrentWidget(d->tables.value(groupKey));
    }
}

void KScoreBoardDialog::show()
{
    d->aboutToShow();
    KDialog::show();
}

void KScoreBoardDialog::exec()
{
    d->aboutToShow();
    KDialog::exec();
}

void KScoreBoardDialog::accept()
{
    QDialog::accept();
}

void KScoreBoardDialog::reject()
{
    if (nameHasBeenRequested() && d->tables.contains(newestScore().first))
    {
        d->tables.value(newestScore().first)->cancelNameRequest();
    }
    QDialog::reject();
}

#include "kscoreboarddialog.moc"
