/****************************************************************
Copyright 2008  Parker Coates parker.coates@gmail.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************/

#include "ksinglescoreboarddialog.h"

#include "kscoremanager.h"
#include "kscoretable.h"

#include <KDebug>
#include <KGlobal>
#include <KLocale>


class KSingleScoreBoardDialog::KSingleScoreBoardDialogPrivate
{
    public:
        KSingleScoreBoardDialogPrivate(KSingleScoreBoardDialog* parent);
        void setupButtons();
        void newScore(const QString& groupKey, int rank);

        KSingleScoreBoardDialog* const q; //Q-Pointer
        KScoreTable * table;
        QString groupKey;
};

KSingleScoreBoardDialog::KSingleScoreBoardDialogPrivate::KSingleScoreBoardDialogPrivate(KSingleScoreBoardDialog* parent)
  : q(parent)
{
}

void KSingleScoreBoardDialog::KSingleScoreBoardDialogPrivate::setupButtons()
{
    if (q->nameHasBeenRequested() && q->newestScore().first == table->group())
    {
        q->setButtons( KDialog::Ok );
    }
    else
    {
        q->setButtons( KDialog::Close );
    }
}

void KSingleScoreBoardDialog::KSingleScoreBoardDialogPrivate::newScore(const QString& groupKey, int rank)
{
    Q_UNUSED(rank)
    if (groupKey == groupKey && q->isVisible())
    {
        setupButtons();
    }
}

KSingleScoreBoardDialog::KSingleScoreBoardDialog(const QString& groupKey)
  : d(new KSingleScoreBoardDialogPrivate(this))
{
    d->table = new KScoreTable(groupKey);
    setMainWidget(d->table);

    connect(KScoreManager::self(), SIGNAL(newScore(QString,int)), this, SLOT(newScore(QString,int)));
}

KSingleScoreBoardDialog::~KSingleScoreBoardDialog()
{
    delete d;
}

void KSingleScoreBoardDialog::setGroup(const QString & groupKey)
{
    d->table->setGroup(groupKey);
}

QString KSingleScoreBoardDialog::group() const
{
    return d->table->group();
}

void KSingleScoreBoardDialog::setDisplayOptions(const KScoreBoardBase::DisplayOptions & options)
{
    d->table->setDisplayOptions(options);
    KScoreBoardBase::setDisplayOptions(options);
}

void KSingleScoreBoardDialog::show()
{
    d->setupButtons();
    KDialog::show();
}

void KSingleScoreBoardDialog::exec()
{
    d->setupButtons();
    KDialog::exec();
}

void KSingleScoreBoardDialog::accept()
{
    d->table->acceptName();
    KDialog::accept();
}

void KSingleScoreBoardDialog::reject()
{
    d->table->cancelNameRequest();
    KDialog::reject();
}

#include "ksinglescoreboarddialog.moc"
