/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include "ksimililastcombinationwidget.h"



#include <QGridLayout>
#include <QLabel>
#include <QString>


#include <klocale.h>


#include "ksimilicard.h"



KSimiliLastCombinationWidget::KSimiliLastCombinationWidget(QWidget *parent) : QFrame(parent)
{
	setMinimumSize(QSize(200, 85));
	setMaximumSize(QSize(16777215, 85));

	QGridLayout *layout = new QGridLayout;

	for (int i=0; i<3; i++) {
		m_cards[i] = new KSimiliCard(this, NULL,  -1, -i);
		m_cards[i]->setMaximumSize(QSize(70, 70));
		layout->addWidget(m_cards[i], 0, i, 3, 1);
	}


	QString txtUnknow = i18n("?");

	QLabel *labelTxtColor = new QLabel(this);
	labelTxtColor->setText(i18n("<b>Color:</b>"));
	labelTxtColor->setAlignment(Qt::AlignHCenter);
	layout->addWidget(labelTxtColor,0,4);

	m_labelResultColor = new QLabel(this);
	m_labelResultColor->setText(txtUnknow);
	m_labelResultColor->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelResultColor,1,4);

	m_labelColor = new QLabel(this);
	m_labelColor->setText(txtUnknow);
	m_labelColor->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelColor,2,4);


	QLabel *labelTxtForm = new QLabel(this);
	labelTxtForm->setText(i18n("<b>Form:</b>"));
	labelTxtForm->setAlignment(Qt::AlignHCenter);
	layout->addWidget(labelTxtForm,0,5);

	m_labelResultForm = new QLabel(this);
	m_labelResultForm->setText(txtUnknow);
	m_labelResultForm->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelResultForm,1,5);

	m_labelForm = new QLabel(this);
	m_labelForm->setText(txtUnknow);
	m_labelForm->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelForm,2,5);


	QLabel *labelTxtFilling = new QLabel(this);
	labelTxtFilling->setText(i18n("<b>Filling:</b>"));
	labelTxtFilling->setAlignment(Qt::AlignHCenter);
	layout->addWidget(labelTxtFilling,0,6);

	m_labelResultFilling = new QLabel(this);
	m_labelResultFilling->setText(txtUnknow);
	m_labelResultFilling->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelResultFilling,1,6);

	m_labelFilling = new QLabel(this);
	m_labelFilling->setText(txtUnknow);
	m_labelFilling->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelFilling,2,6);


	QLabel *labelTxtNumber = new QLabel(this);
	labelTxtNumber->setText(i18n("<b>Number:</b>"));
	labelTxtNumber->setAlignment(Qt::AlignHCenter);
	layout->addWidget(labelTxtNumber,0,7);

	m_labelResultNumber = new QLabel(this);
	m_labelResultNumber->setText(txtUnknow);
	m_labelResultNumber->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelResultNumber,1,7);

	m_labelNumber = new QLabel(this);
	m_labelNumber->setText(txtUnknow);
	m_labelNumber->setAlignment(Qt::AlignHCenter);
	layout->addWidget(m_labelNumber,2,7);

	//layout->addStretch();

	setLayout(layout);
}



//
// Slots
//

void KSimiliLastCombinationWidget::selectionOf3Cards(QList <KSimiliCard *> cards)
{
	for (int i=0;i<3;i++) {
		m_cards[i]->setNewCardId(cards[i]->getCardId());
	}

	QString txtRight = i18n("<font color=\"green\">Right</font>");
	QString txtWrong = i18n("<font color=\"red\">Wrong</font>");
	QString txtSameResult = "<b>=</b>";
	QString txtDifferentResult = "<b>!=</b>"; // TODO ≠ Unicode # 8800  
	QString txtWrongResult = "<font color=\"red\">-</font>";
	

	if (((m_cards[0]->color + m_cards[1]->color + m_cards[2]->color) % 3)==0) {
		m_labelColor->setText(txtRight);
		if (m_cards[0]->color==m_cards[1]->color)
			m_labelResultColor->setText(txtSameResult);
		else
			m_labelResultColor->setText(txtDifferentResult);
	}
	else {
		m_labelColor->setText(txtWrong);
		m_labelResultColor->setText(txtWrongResult);
	}
		
	if (((m_cards[0]->form + m_cards[1]->form + m_cards[2]->form) % 3)==0) {
		m_labelForm->setText(txtRight);
		if (m_cards[0]->form==m_cards[1]->form)
			m_labelResultForm->setText(txtSameResult);
		else
			m_labelResultForm->setText(txtDifferentResult);
	}
	else {
		m_labelForm->setText(txtWrong);
		m_labelResultForm->setText(txtWrongResult);
	}
	
	if (((m_cards[0]->filling + m_cards[1]->filling + m_cards[2]->filling) % 3)==0) {
		m_labelFilling->setText(txtRight);
		if (m_cards[0]->filling==m_cards[1]->filling)
			m_labelResultFilling->setText(txtSameResult);
		else
			m_labelResultFilling->setText(txtDifferentResult);
	}
	else {
		m_labelFilling->setText(txtWrong);
		m_labelResultFilling->setText(txtWrongResult);
	}

	if (((m_cards[0]->number + m_cards[1]->number + m_cards[2]->number) % 3)==0) {
		m_labelNumber->setText(txtRight);
		if (m_cards[0]->number==m_cards[1]->number)
			m_labelResultNumber->setText(txtSameResult);
		else
			m_labelResultNumber->setText(txtDifferentResult);
	}
	else {
		m_labelNumber->setText(txtWrong);
		m_labelResultNumber->setText(txtWrongResult);
	}
	
}


#include "ksimililastcombinationwidget.moc"
