/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include "ksimilimainwindowcentralwidget.h"



#include <QAction>
#include <QDateTime>
#include <QTimer>
#include <QVBoxLayout>
#include <QWidget>


#include "ksimilicard.h"
#include "ksimiliboard.h"
#include "ksimililastcombinationwidget.h"
#include "ksimilimainwindow.h"
#include "ksimilistatswidget.h"



KSimiliMainWindowCentralWidget::KSimiliMainWindowCentralWidget(KSimiliMainWindow *parent)
	: QFrame(parent)
{
	m_mainWindow = parent;

	QVBoxLayout *layout = new QVBoxLayout;
	layout->setAlignment(m_statsWidget, Qt::AlignTop);

	m_statsWidget = new KSimiliStatsWidget(this);
	layout->addWidget(m_statsWidget);
	m_board = new KSimiliBoard(this);
	layout->addWidget(m_board);
	m_lastCombinationWidget = new KSimiliLastCombinationWidget(this);
	layout->addWidget(m_lastCombinationWidget);

	connect(m_board, SIGNAL(newCombinationFound()), m_statsWidget, SLOT(newCombinationFound()));
	connect(m_board, SIGNAL(newErrorFound()), m_statsWidget, SLOT(newErrorFound()));
	connect(m_board, SIGNAL(selectionOf3Cards(QList <KSimiliCard *>)), m_lastCombinationWidget, SLOT(selectionOf3Cards(QList <KSimiliCard *>)));
	connect(m_board, SIGNAL(possibleCombinations(int)), m_statsWidget, SLOT(displayPossibleCombinations(int)));

	setLayout(layout);

	m_timer = new QTimer(this);
	connect(m_timer, SIGNAL(timeout()), this, SLOT(timeSlotOut()));
}




// Slots
void KSimiliMainWindowCentralWidget::newGame(const int sizeBoard, const int timeToPlay)
{
	m_board->chooseNewCards(sizeBoard);
	m_statsWidget->newGame(timeToPlay);
	m_board->setGameRunning(true);
	m_timeLeft = timeToPlay;
	m_timer->start(1000);
}


void KSimiliMainWindowCentralWidget::gameOver()
{
	m_timer->stop();
	m_statsWidget->gameOver();
	m_board->setGameRunning(false);
	m_mainWindow->gameOver(m_statsWidget->getCombinationsPerHour());
}


void KSimiliMainWindowCentralWidget::pauseGame(bool pause)
{
	if (pause) {
		timeSlotOut();
		m_timer->stop();
		m_board->hide();
	} else {
		m_timer->start();
		m_board->show();
	}
}


void KSimiliMainWindowCentralWidget::timeSlotOut()
{
	m_timeLeft--;
	m_statsWidget->timeLeftChanged(m_timeLeft);
	m_board->timeLeftChanged();
	if (m_timeLeft<=0)
		gameOver();
}


void KSimiliMainWindowCentralWidget::activateDisplayPossibleCombinations(bool activation)
{
	m_statsWidget->activateDisplayPossibleCombinations(activation);
}


void KSimiliMainWindowCentralWidget::solve()
{
	m_board->showNextSolution();
}


void KSimiliMainWindowCentralWidget::hint()
{
	m_board->showHint();
}


#include "ksimilimainwindowcentralwidget.moc"
