/*
    This file is part of KSpiral.
    Copyright 2009 Felix Lemke <hobbyblobby@arcor.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "cannon.h"

#include "ball.h"
#include <QGraphicsItem>
#include <QGraphicsScene>
#include <KGameRenderer>
#include <QTimer>

KSpiral::Cannon::Cannon(KGameRenderer* renderer, const QString& itemName, int velocity)
	: KGameRenderedObjectItem(renderer, itemName)
	, m_top(0)
	, m_ballVelocity(velocity)
	, m_ballToShoot(0)
{
  const QSizeF size = renderer->boundsOnSprite(itemName).size();
  setOffset(-size.width() / 2, -size.height() / 2);
  setFixedSize(size.toSize());
}

KSpiral::Cannon::~Cannon() {
}

void KSpiral::Cannon::loadCannon() {
	//import information
	m_cannonCenter = mapToScene(boundingRect().center());

	if(!m_ballToShoot) {
		m_ballToShoot = new KSpiral::Ball(m_cannonCenter, renderer(), randomBalltype());
		m_ballToShoot->setZValue(10);
		scene()->addItem(m_ballToShoot);
	}
}

void KSpiral::Cannon::setAngle(const qreal& angle) {
  rotate((angle - m_angle) * 180 / M_PI);
	if (m_top)
		m_top->rotate((angle - m_angle) * 180 / M_PI);
  m_angle = angle;
}

KSpiral::Ball* KSpiral::Cannon::shoot() {
	KSpiral::Ball* oldBall = m_ballToShoot;
	if(m_ballToShoot) {
		//store cannonball and shoot
		m_ballToShoot->shoot(m_cannonCenter, m_angle, m_ballVelocity);
		m_ballToShoot = 0;

		//reload cannon after a while
		QTimer::singleShot(1000, this, SLOT(loadCannon()));
	}
	return oldBall;
}
