/***************************************************************************
 *   Copyright (C) 2003 by Albert Astals Cid                               *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "evaluation.h"

#include <kglobal.h> // for Q_ASSERT

evaluation::evaluation()
{
	isPossible = false;
	place_x = -1;
	place_y = -1;
	remove_x = -1;
	remove_y = -1;
}

evaluation::evaluation(evaluation *e)
{
	score = e -> score;
	place_x = e -> place_x;
	place_y = e -> place_y;
	remove_x = e -> remove_x;
	remove_y = e -> remove_y;
	isPossible = e -> isPossible;
}

evaluation::~evaluation()
{
}

void evaluation::setPlace(int p_x, int p_y)
{
	Q_ASSERT(p_x >= 0);
	Q_ASSERT(p_x <= 2);
	Q_ASSERT(p_y >= 0);
	Q_ASSERT(p_y <= 2);
	place_x = p_x;
	place_y = p_y;
}

void evaluation::setRemove(int r_x, int r_y)
{
	Q_ASSERT(r_x >= 0);
	Q_ASSERT(r_x <= 2);
	Q_ASSERT(r_y >= 0);
	Q_ASSERT(r_y <= 2);
	remove_x = r_x;
	remove_y = r_y;
}

void evaluation::setValue(int value)
{
	score = value;
	isPossible = true;
}

int evaluation::getValue() const
{
	Q_ASSERT(isPossible);
	return score;
}

bool evaluation::possible() const
{
	return isPossible;
}

evaluation* evaluation::bestScore(evaluation* e, kcase::PLAYER o)
{
	Q_ASSERT(o != kcase::NOBODY);
	Q_ASSERT(e -> possible());
	evaluation *max;
	evaluation *min;

	if (!possible())
	{
		max = e;
		min = e;
	}
	else if (score > e -> score)
	{
		max = this;
		min = e;
	}
	else
	{
		max = e;
		min = this;
	}
	if (o == kcase::PLAYER1) return max;
	else return min;
}

int evaluation::p_x() const
{
	return place_x;
}

int evaluation::r_x() const
{
	return remove_x;
}

int evaluation::p_y() const
{
	return place_y;
}

int evaluation::r_y() const
{
	return remove_y;
}

/*           */

boardEvaluation::boardEvaluation()
{
	board = new evaluation** [3];
	for (int i = 0; i < 3; i++)
	{
		board[i] = new evaluation* [3];
		for (int j = 0; j < 3; j++)
		{
			board[i][j] = new evaluation();
			board[i][j] -> setPlace(i, j);
		}
	}
}

boardEvaluation::~boardEvaluation()
{
	for (int i = 0; i < 3; i++)
	{
		for (int j = 0; j < 3; j++)
		{
			delete board[i][j];
		}
		delete[] board[i];
	}
	delete[] board;
}

evaluation* boardEvaluation::getEvaluation(int i, int j)
{
	Q_ASSERT(i >= 0 && i < 3 && j >= 0 && j < 3);
	return board[i][j];
}

void boardEvaluation::setEvaluation(int i, int j, evaluation *e)
{
	Q_ASSERT(i >= 0 && i < 3 && j >= 0 && j < 3);
	Q_ASSERT(e -> possible());
	delete board[i][j];
	board[i][j] = e;
	board[i][j] -> setPlace(i, j); // remind the evaluation where it belongs
}

int boardEvaluation::getValue(int i, int j) const
{
	Q_ASSERT(i >= 0 && i < 3 && j >= 0 && j < 3);
	return board[i][j] -> getValue();
}

bool boardEvaluation::possible(int i, int j) const
{
	Q_ASSERT(i >= 0 && i < 3 && j >= 0 && j < 3);
	return board[i][j] -> possible();
}

evaluation* boardEvaluation::bestMove(kcase::PLAYER o)
{
	/* TODO redo to not be so fixed */
	Q_ASSERT(o != kcase::NOBODY);
	evaluation *ret = board[0][0];
	for (int i = 0; i < 3; i++)
	{
		for (int j = 0; j < 3; j++)
		{
			if (possible(i, j))
			{
				ret = ret -> bestScore(getEvaluation(i, j), o);
			}
		}
	}
	
	return new evaluation(ret);
}
