/***************************************************************************
 *   Copyright (C) 2003-2007 by Albert Astals Cid                          *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "kcasewidget.h"

#include <qpainter.h>

#include "settings.h"

kcaseWidget::kcaseWidget(QWidget *parent, int x, int y) : QFrame(parent), kcase(x, y)
{
	setFrameStyle(QFrame::Box | QFrame::Plain);
	setLineWidth(2);
	updateColors();
}

void kcaseWidget::setEmpty()
{
	kcase::setEmpty();
	update();
}

void kcaseWidget::setOcupation(PLAYER o)
{
	kcase::setOcupation(o);
	update();
}

void kcaseWidget::toggleSelected()
{
	kcase::toggleSelected();
	update();
}

void kcaseWidget::updateColors()
{
	update();
}

QColor kcaseWidget::getBackgroundColor() const
{
	return ktttSettings::background();
}

QColor kcaseWidget::getBorderColor() const
{
	return ktttSettings::border();
}

QColor kcaseWidget::getPlayerColor() const
{
	Q_ASSERT(p_o != NOBODY);

	if (selected)
	{
		return ktttSettings::selected();
	}
	else
	{
		if (p_o == PLAYER1) return ktttSettings::player1_color();
		else return ktttSettings::player2_color();
	}
}

void kcaseWidget::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == Qt::LeftButton) emit caseClicked(p_x, p_y);
}

void kcaseWidget::paintEvent(QPaintEvent *e)
{
	QFrame::paintEvent(e); // the QFrame draws its border

	QPainter p(this);
	p.fillRect(2, 2, width()-4, height()-4, getBackgroundColor());
	if (p_o != NOBODY)
	{
		p.setRenderHint(QPainter::Antialiasing);
		double k;
		// k is the minimum of height and width
		k = width() > height() ? height() : width();

		p.setPen(getBorderColor());
		p.setBrush(getPlayerColor());

		if (getShape() == ktttSettings::EnumPlayer1_shape::Cross)
		{
			double c, g, i;
			i = k / 2.0;
			c = 3.0 * k / 8.0;
			g = k * 0.05;
			// pa is a cross at the axis
			QPolygonF pa;
			pa.append(QPointF(-c, -g));
			pa.append(QPointF(-g, -g));
			pa.append(QPointF(-g, -c));
			pa.append(QPointF(g, -c));
			pa.append(QPointF(g, -g));
			pa.append(QPointF(c, -g));
			pa.append(QPointF(c, g));
			pa.append(QPointF(g, g));
			pa.append(QPointF(g, c));
			pa.append(QPointF(-g, c));
			pa.append(QPointF(-g, g));
			pa.append(QPointF(-c, g));
			// move the axis to the center of the case
			p.translate( width()/2, height()/2);
			// rotate the axis
			p.rotate(45);
			// draw the cross in the new axis
			p.drawPolygon(pa);
		}
		else
		{
			double a;
			a = k * 0.75;
			// draw a center circle in the case that is 75% of it
			// well all the centered it can be assuming Qt only draws
			p.drawChord(QRectF((width() - a)/2, (height() - a)/2, a, a), 0, 5760);
		}
	}
}

int kcaseWidget::getShape() const
{
	Q_ASSERT(p_o != kcase::NOBODY);
	if (p_o == kcase::PLAYER1) return ktttSettings::player1_shape();
	else return ktttSettings::player2_shape();
}

#include "kcasewidget.moc"
