//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#include "item.h"

#include <KDebug>

#include "renderer.h"

MagazynierItem::MagazynierItem(KGameRenderer* renderer, const QString& spriteKey,
			       QGraphicsItem *parent) :
	KGameRenderedItem(renderer, spriteKey, parent),
	m_itemOrientation(*LevelModel::m_moveDirUp)
{
	setRenderSize(QSize(Renderer::instance()->getItemSize(), Renderer::instance()->getItemSize()));
}

void MagazynierItem::updatePixmap(void)
{
	QString name;
	switch (m_itemType) {
	case Empty:
		name = "Empty";
		break;
	case Box:
		name = "Box";
		break;
	case Goal:
		name = "Goal";
		break;
	case Goal|Box:
		name = "Goal_Box";
		break;
	case Goal|Player:
		if (m_itemOrientation == *LevelModel::m_moveDirUp) {
			name = "Goal_Player_Up";
		} else if (m_itemOrientation == *LevelModel::m_moveDirRight) {
			name = "Goal_Player_Right";
		} else if (m_itemOrientation == *LevelModel::m_moveDirDown) {
			name = "Goal_Player_Down";
		} else if (m_itemOrientation == *LevelModel::m_moveDirLeft) {
			name = "Goal_Player_Left";
		}
		break;
	case Player:
		if (m_itemOrientation == *LevelModel::m_moveDirUp) {
			name = "Player_Up";
		} else if (m_itemOrientation == *LevelModel::m_moveDirRight) {
			name = "Player_Right";
		} else if (m_itemOrientation == *LevelModel::m_moveDirDown) {
			name = "Player_Down";
		} else if (m_itemOrientation == *LevelModel::m_moveDirLeft) {
			name = "Player_Left";
		}

		break;
	case Wall:
		name = "Wall";
		break;
	}
	setSpriteKey(name);
}

int MagazynierItem::itemType(void) const
{
	return m_itemType;
}

void MagazynierItem::setItemType(int itemType)
{
	m_itemType = itemType;
	updatePixmap();
	return;
}

void MagazynierItem::setLevelPos(QPoint pos)
{
	int cellSize = Renderer::instance()->getItemSize();

	setPos(pos.x() * cellSize, pos.y() * cellSize);
	return;
}

void MagazynierItem::setOrientation(QPoint orientation)
{
	if (orientation == m_itemOrientation) {
		return;
	}
	m_itemOrientation = orientation;
	updatePixmap();
}
