//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#ifndef LEVEL_MODEL_H
#define LEVEL_MODEL_H

#include <QGraphicsScene>
#include <QObject>
#include <QPainter>
#include <QPoint>
#include <QVector>


enum ItemType {
	Empty = 0,
	Goal = 1,
	Box = 2,
	Player = 4,
	Wall = 8
} ;


class QGraphicsPixmapItem;
class QPoint;
class QUndoCommand;

class MagazynierCommand;
class MagazynierItem;

class LevelModel : public QGraphicsScene
{
	Q_OBJECT

public:
	static QPoint *m_moveDirUp;
	static QPoint *m_moveDirLeft;
	static QPoint *m_moveDirDown;
	static QPoint *m_moveDirRight;


	LevelModel(QObject *Parent=0);
	~LevelModel();
	void initLevelData(QString &data);
	void populateScene(void);

	int getColumnCount(void) const;
	int getRowCount(void) const;
	int getMoveCount(void);
	int getShiftCount(void);

	MagazynierCommand *move(QPoint pos, QPoint delta, QUndoCommand *parentCommand = 0);

	void moveFromEmptyToEmpty(QPoint pos, QPoint delta);
	void moveFromEmptyToGoal(QPoint pos, QPoint delta);
	void moveFromGoalToEmpty(QPoint pos, QPoint delta);
	void moveFromGoalToGoal(QPoint pos, QPoint delta);

	MagazynierItem *getItem(QPoint pos) const;

	void mouseReleaseEvent(QGraphicsSceneMouseEvent * mouseEvent);

	void drawBackground(QPainter *painter, const QRectF &rect);
private:
	QVector<QVector<MagazynierItem *>*> *m_levelItems;
	int m_goalsLeft;
	int m_shiftCount;
	bool m_levelCompleted;
	int m_moveCount;

	void setItem(QPoint pos, MagazynierItem *item);

	QPoint *m_playerPos;
	QPoint m_playerOrientation;
	void updatePlayerOrientation(QPoint orientation);

	int m_columnCount, m_rowCount;


	bool canMove(QPoint pos, QPoint delta);

public slots:
	void moveUp(void);
	void moveLeft(void);
	void moveRight(void);
	void moveDown(void);

	void moveDone(int moveCount, int shiftCount);
	void moveUndone(int moveCount, int shiftCount);

signals:
	void addNewUndoCommand(MagazynierCommand *command);
	void levelCompleted(void);
	void moveCountChanged(int i);
	void shiftCountChanged(int i);
};

#endif
