//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#ifndef MAGAZYNIER_WINDOW_H
#define MAGAZYNIER_WINDOW_H

#include <KXmlGuiWindow>

class KGameClock;
class KGamePopupItem;
class KUndoStack;

class QActionGroup;
class QGraphicsScene;
class QGraphicsView;
class QUndoCommand;

class MagazynierCommand;
class LevelModel;
class LevelPack;
class Renderer;

class MagazynierWindow : public KXmlGuiWindow
{
	Q_OBJECT
	public:
		MagazynierWindow(QWidget *parent=0);
		MagazynierWindow(const MagazynierWindow&);

		void pushNewCommand(MagazynierCommand *command);
	private:
		void setupLastPlayedLevelPack(void);

		void setupActions(void);

		QMap<QString,LevelPack *> m_levelPacks;
		LevelPack *m_levelPack;
		QActionGroup *m_levelPackActionGroup;

		LevelModel *m_levelModel;
		void findLevelPacks(void);
		void setupLevelPackMenu(void);
		
		QGraphicsView *m_view;

		KUndoStack *m_undoStack;

		KGameClock *m_gameClock;
		bool m_gameClockRunning;

		KGamePopupItem* m_gamePopupItem;

	protected:
		void resizeEvent(QResizeEvent *event);

	public slots:
		void levelChanged(LevelModel *newLevelModel);
		void updateLevelChangeActions(void);
		void connectMoveActions(void);
		void disconnectMoveActions(void);
		void levelPackChanged(LevelPack *newLevelPack);
		void redrawNeeded(void);
		void newUndoCommandCreated(MagazynierCommand *command);
		void updateMoveCounter(int i);
		void updateShiftCounter(int i);
		void configureSettings(void);
		void reloadTheme(void);
		void updateTimer(void);
		void showHighscore(void);
		void addHighscore(void);
		void showLevelCompletedPopup(void);
		void changeLevel(void);
};

#endif
