/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "colorselector.h"
#include "ds/data.h"
#include "colorbutton.h"

#include <QGridLayout>
#include <QGroupBox>
#include <QRadioButton>
#include <QPushButton>
#include <QColor>
#include <QLayoutIterator>

#include <KDebug>
#ifndef KXMLGUIWINDOW_H
    #include <KXmlGuiWindow>
#endif
#include <KActionCollection>
#include <KLocalizedString>
#include <KAction>


uint qHash ( const QColor & key ) {
    return qHash( key.name() );
}

ColorSelector::ColorSelector(KXmlGuiWindow *parent) :
    QWidget(parent)
{
    m_layout = new QGridLayout;


    //TODO: don't assume a maximum amount of colors. Can this be done dynamically?
    int max_colors = 10;
    int color_per_line = 5;

    int line = 0;
    int col = 0;

    QGroupBox *groupBox = new QGroupBox();

    for (int i = 0; i < max_colors; ++i) {
        ColorButton *r = new ColorButton(this);
        r->setChecked(i == 0);

        m_colorbuttons.append(r);

        m_layout->addWidget(r, line, col);

        col++;

        if ( col %color_per_line == 0) {
            line++;
            col=0;
        }
    }

    groupBox->setLayout(m_layout);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(groupBox);
    setLayout(vbox);

    KAction* nextColorAction = new KAction(this);
    nextColorAction->setText(i18n("Next Color"));
    nextColorAction->setIcon(KIcon("color"));
    nextColorAction->setShortcut(Qt::Key_C);
    parent->actionCollection()->addAction("nextColor", nextColorAction);
    connect(nextColorAction, SIGNAL(triggered(bool)), SLOT(nextColor()));

}

void ColorSelector::setData(Data * data)
{
    kDebug() << "Received setData Signal";

    QList<QColor> * cList = data->availableColors();


    m_colorMap = new QHash<QColor, ColorButton*>();

    //TODO: don't assume there's always enough space (cList.size() <= max_colors)!

    for (int i = 0; i < m_colorbuttons.size(); ++i) {

        ColorButton *r = m_colorbuttons.at(i);
        kDebug() << "Disconnecting everything from button " << r;
        r->disconnect();

        if (i < cList->size()) {
            QColor c = cList->at(i);
            r->setColor(c);
            r->setVisible(true);

            m_colorMap->insert(c, r);

            //TODO: Get rid of this in and out of signals and slots. One should be enough.

            //This triggers an internal slot in the colorbutton, so the assigned _color_ can be emitted
            connect(r, SIGNAL(clicked()), r, SLOT(emitColorSignal()));

            //This is the necessary connection: it sets the current color in the data.
            connect(r, SIGNAL(activated(const QColor &)), data, SLOT(setActiveColor(const QColor &)));
        } else {

            r->setVisible(false);
        }
    }

    m_colorbuttons.first()->click();
}

void ColorSelector::nextColor() {

    kDebug() << "Trying to find currently selected color.";
    int currentIndex = -1;
    for(int i = 0; i < m_colorbuttons.size() && currentIndex < 0 && m_colorbuttons.at(i+1)->isVisible(); i++) {
        kDebug() << "Checking " << m_colorbuttons.at(i)->isChecked() << " [" << i << "]";
        if (m_colorbuttons.at(i)->isChecked()) {
            kDebug() << "Found it at: " << i;
            currentIndex = i;
        }
    }

    currentIndex += 1;
    if (currentIndex >= m_colorbuttons.size()) {
        currentIndex = 0;
    }

    m_colorbuttons.at(currentIndex)->click();

}

void ColorSelector::selectColor(QColor* color) {
    QColor c = *color;
    ColorButton * colorbutton = m_colorMap->value(c);
    if (colorbutton != NULL) {
        colorbutton->click();
    }
}





