/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef GRID_CENTER_H
#define GRID_CENTER_H

#include <QWidget>
#include <QMouseEvent>
#include <QPaintEvent>

#include "gui/a_grid.h"

#include "ds/data.h"

/**
 * This class provides a the central grid-widget, that reacts to
 * mouse-clicks.
 */
class GridCenter : public Abstract_Grid
{
    Q_OBJECT

    public:
        GridCenter(QWidget *parent);
        ~GridCenter() { }

    public slots:
        void setData(Data* newData);
        void up();
        void down();
        void left();
        void right();
        void toggleCurrent();
        virtual void readConfig();

    signals:
        void togglePosition(int x, int y);
        void toggle_UndefinedOn(int x, int y);
        void toggle_UndefinedOff(int x, int y);
        void currentToggled();
        void continueLastChange(int x, int y);


    protected:
        bool doPaintHorizontal();
        bool doPaintVertical();

        int widthOfLastLine();
        int heightOfLastLine();

        void paintAdditionalHorizontal(QPainter * painter, int pos_y, int row);
        void paintAdditionalVertical(QPainter * painter, int pos_x, int column);

        virtual void mousePressEvent(QMouseEvent * event);
        virtual void mouseReleaseEvent(QMouseEvent * event);
        virtual void mouseMoveEvent(QMouseEvent * event);

    private:
        void paintBox(int x, int y, int width, int height,
                Data::MatrixEntry * entry, QPainter * painter);

        QPoint* adjustManualSelection(QPoint *p);

        Qt::MouseButton lastPressedButton;
        QPoint * lastMoveGridPosition;
        QColor offStateDotColor;
};

#endif
