/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "./originalsize_picture.h"

#include <qpainter.h>

#include <KDebug>
#include <KComponentData>
#include <KConfig>
#include <KConfigGroup>
#include <KGlobal>
#include <ds/config.h>

OriginalSizePicture::OriginalSizePicture(QWidget *parent) :
    QWidget(parent)
{
    setAutoFillBackground(true);
}

void OriginalSizePicture::setData(Data * data) {

    m_data = data;

    m_cols = m_data->getColumnCount();
    m_rows = m_data->getRowCount();

    //repaint preview as well
    connect(data, SIGNAL(positionChanged()), SLOT(update()));

    update();

    kDebug() << "Changed data: ["<<m_cols<<" cols / "<< m_rows<<" rows]";
}

void OriginalSizePicture::paintEvent(QPaintEvent * /*event*/) {

    if (m_data == NULL) {
        return;
    }

    QPainter painter(this);

    for (int x = 0; x<m_cols; x++) {
        for (int y = 0; y<m_rows; y++) {

            Data::MatrixEntry * entry = m_data->entry(x, y);
            painter.setPen(entry->box_color);
            painter.setBrush(entry->box_color);

            if (entry->box_status == Data::BOX_OFF || entry->box_status == Data::BOX_ON) {

                painter.drawRect(x*2, y*2, 2, 2);
            }
            else {
                //TODO: Should BOX_UNDEFINED status be painted?
                //@see GridCenter::paintBox
            }
        }
    }
}

void OriginalSizePicture::readConfig() {
    KConfigGroup config = KGlobal::config()->group(Config::CONFIG_GROUP_COLORS);
    QColor background = config.readEntry( Config::CONFIG_COLOR_BACKGROUND, Config::DEFAULT_COLOR_BACKGROUND );

    setPalette(QPalette(background));

    update();
}

