/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "io_factory.h"

#include <KDebug>

NonogramIo * Io_Factory::createIoForType(const QString & type)
{
    static NonogramIo * retval = NULL;

    kDebug() << "Supported image formats: "
            << QImageReader::supportedImageFormats();
    if (QImageReader::supportedImageFormats().contains(type.toAscii() )) {

        NonogramIo * in = new XbmFormat();
        retval = in;
    }
    else
        if (type == "pic") {

            NonogramIo * in = new PicFormat();
            retval = in;
        }
        else
            if (type == "nog") {

                NonogramIo * in = new NogFormat();
                retval = in;
            }
            else {
                kDebug() << "Unknown type [" << type << "]: returning null";
                retval = NULL;
            }

    return retval;
}

Data * Io_Factory::readFile(const QString & filename)
{
    QString type = filename;
    type.remove(0, type.lastIndexOf(".")+1);

    NonogramIo * in = Io_Factory::createIoForType(type);
    return in->readFile(filename);
}

