/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "io/nog_format.h"

#include <QXmlAttributes>
#include <QXmlInputSource>
#include <KDebug>

#include "ds/data.h"

NogFormat::NogFormat(QObject *parent): NonogramIo(parent){
    m_isGame = false;
}

Data * NogFormat::readFile(const QString & fileName)
{
    kDebug() << "Reading from " << fileName;

    QFile file(fileName);

    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        kDebug() << "Cannot read file: " << fileName << "\n" << file.errorString();
        return NULL;
    }

    QXmlInputSource xmlInputSource(&file);


    return handleXml(xmlInputSource);
}

Data * NogFormat::readData(const QByteArray & data)
{
    QXmlInputSource xmlInputSource;
    xmlInputSource.setData(data);
    return handleXml(xmlInputSource);
}

Data* NogFormat::handleXml(const QXmlInputSource & xmlInputSource )
{
    QXmlSimpleReader reader;
    reader.setContentHandler(this);
    reader.setErrorHandler(this);

    kDebug() << "Begin parsing";
    reader.parse(xmlInputSource);
    kDebug() << "Done parsing";

    return m_data;
}

bool NogFormat::startElement(const QString & /* namespaceURI */,
        const QString & /* localName */, const QString & qName,
        const QXmlAttributes & /* attributes */ )
{

    bool retval = true;

    m_currentText.clear();
    if (qName == NogOut::GAME_TAG && !m_isGame) {

        m_data = new Data(parent());
        Description * d = new Description();
        m_data->setDescription(d);

        m_isGame = true;
        retval = true;
    }


    return retval;
}

bool NogFormat::endElement(const QString & /* namespaceURI */,
        const QString & /* localName */, const QString & qName)
{
    /*not even a game definition*/
    if (! m_isGame) {
        return false;
    }

    /*only the ending part of the definition. just finish it*/
    if (qName == NogOut::GAME_TAG) {

        m_isGame = false;

        m_data->update();

        emit dataRead(m_data);
        return true;
    }

    bool retval = false;

    /*This is where the actual values are set.*/
    if (qName == NogOut::NAME_TAG) {

        m_data->description()->setName(m_currentText);
        retval = true;
    } else if (qName == NogOut::HINT_TAG) {

        m_data->description()->setHint(m_currentText);
        retval = true;
    } else if (qName == NogOut::DESCRIPTION_TAG) {

        m_data->description()->setDescription(m_currentText);
        retval = true;
    } else if (qName == NogOut::ROWS_TAG) {

        m_data->setRowHints(Data::fromString(m_currentText));
        retval = true;
    } else if (qName == NogOut::COLUMNS_TAG) {

        m_data->setColumnHints(Data::fromString(m_currentText));
        retval = true;
    } else if (qName == NogOut::MATRIX_TAG) {

        kDebug() << "Not implemented yet";
        retval = true;
    }

    return retval;
}

bool NogFormat::characters(const QString & str)
{
    m_currentText += str;
    return true;
}

bool NogFormat::fatalError(const QXmlParseException & exception)
{
    //TODO: do something more approbriate.
    kDebug() << "Got an exception: " << exception.message() << "\nline: "<< exception.lineNumber() << " column:" << exception.columnNumber();
    return false;
}




