/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "io/nop_format.h"

#include <KDebug>

#include "io/nog_format.h"
#include "io/nog_out.h"

#include "io/nop_out.h"

NopFormat::NopFormat(Package * package, QObject *parent) : m_package(package), parent(parent) {
    connect(this, SIGNAL(dataRead(Data *)), this, SLOT(gameRead(Data *)));
}


Package * NopFormat::readPackageFile(const QString & fileName)
{
    kDebug() << "Reading from " << fileName;

    QFile file(fileName);

    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        kDebug() << "Cannot read file: " << fileName << "\n" << file.errorString();
        return NULL;
    }

    QXmlInputSource xmlInputSource(&file);


    Package * readPackage = handleXmlPackage(xmlInputSource);
    readPackage->setFilename(fileName);
    return readPackage;
}

Package * NopFormat::readPackageData(const QByteArray & data)
{
    QXmlInputSource xmlInputSource;
    xmlInputSource.setData(data);
    return handleXmlPackage(xmlInputSource);
}

Package * NopFormat::handleXmlPackage(const QXmlInputSource & xmlInputSource )
{
    QXmlSimpleReader reader;
    reader.setContentHandler(this);
    reader.setErrorHandler(this);

    reader.parse(xmlInputSource);

    return m_package;
}


void NopFormat::dataRead(Data * data)
{
    m_package->append(data);
}

bool NopFormat::startElement(const QString &namespaceURI,
        const QString &localName, const QString &qName,
        const QXmlAttributes &attributes)
{

    /*let game-handler do it's magic*/
    if (NogFormat::startElement(namespaceURI, localName, qName, attributes) ) {
        return true;
    }

    /*Can't defer to game-handler*/
    bool retval = false;

    if (qName == NopOut::PACKAGE_TAG) {

        m_isPackage = true;
        retval = true;
    }

    return retval;
}

bool NopFormat::endElement(const QString &namespaceURI,
        const QString &localName, const QString &qName)
{
    /*let game-handler do it's magic*/
    if (NogFormat::endElement(namespaceURI, localName, qName) ) {
        return true;
    }

    /*only the ending part of the definition. make it quick and don't fiddle around*/
    if (qName == NopOut::PACKAGE_TAG) {

        m_isPackage = false;

        kDebug() << "Finished package definition. See ya.";

        //emit a signal?

        return true;
    }

    /*Can't defer to game-handler*/
    bool retval = false;


    /*This is where the actual values are set.*/
    if (qName == NogOut::NAME_TAG) {

        m_package->setName(m_currentText);
        retval = true;
    } else if (qName == NogOut::DESCRIPTION_TAG) {

        m_package->setDescription(m_currentText);
        retval = true;
    } else if (qName == NopOut::DEFAULT_DESCRIPTION_TAG) {
        m_package->setDefaultDescription(m_currentText);
        retval = true;
    }

    return retval;
}

void NopFormat::gameRead(Data * data)
{
    m_package->append(data);
}

