/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

/**
 * This class provides the ability to read game packages in NOP-format:
 * Each package consists of some actual games in the NOG-format
 * (@see io/nog_format.h) plus some package information.
 * The NOP-format is an XML-Format.
 *
 * Since most of the actual handling is performed for the contained games,
 * this implementation subclasses the NogFormat class.
 */
#ifndef NOP_FORMAT_H_
#define NOP_FORMAT_H_

#include "io/nog_format.h"

#include <QObject>
#include <QXmlAttributes>
#include <QString>
#include <QByteArray>

#include "ds/data.h"
#include "ds/package.h"

class NopFormat : public NogFormat
{
    Q_OBJECT

    public:

        NopFormat(Package * package = new Package(), QObject *parent=0);

        Package * readPackageFile(const QString & fileName);

        Package * readPackageData(const QByteArray & data);

        bool startElement(const QString &namespaceURI, const QString &localName,
                          const QString &qName, const QXmlAttributes &attributes);
        bool endElement(const QString &namespaceURI, const QString &localName,
                        const QString &qName);

    public slots:
        void gameRead(Data * data);

    private:
        Package * handleXmlPackage(const QXmlInputSource & xmlInputSource ) ;
        void dataRead(Data * data);

        Package * const m_package;
        bool m_isPackage;
        QObject *parent;

//        NogFormat * m_nogreader;

//        QString m_currentText;

};

#endif /*NOP_FORMAT_H_*/
