/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "pic_format.h"

#include <QDataStream>
#include <QVector>
#include <QFile>

#include "ds/data.h"

Data * PicFormat::readFile(const QString & fileName)
{
    QFile file(fileName);
    if (file.open(QIODevice::ReadOnly) ) {
        /*This comes from zack and jeremy's version: kpicross.*/
        QDataStream stream( &file);

        QString title;
        int rows, cols;

        stream >> title >> rows >> cols;

        QVector< QVector<bool> > picture = QVector< QVector<bool> >(rows,
                QVector<bool>(cols, false) );

        for (int i = 0; i < rows; ++i) {
            for (int j = 0; j < cols; ++j) {
                qint8 field;
                stream >> field;
                picture[i][j] = static_cast<bool>(field );
            }
        }

        return picture2Data(picture, rows, cols, title);
    }

    return NULL;
}

Data * PicFormat::readData(const QByteArray &)
{
    return NULL;
}

Data * PicFormat::picture2Data(const QVector< QVector<bool> > & picture,
        int rows, int cols, const QString & title)
{
    Data * data = new Data();

    getrows(picture, rows, cols, data);
    getcols(picture, rows, cols, data);

    data->description()->setHint(title);
    data->update();

    return data;
}

void PicFormat::getrows(const QVector< QVector<bool> > & picture, int rows,
        int cols, Data * data)
{

    for (int y = 0; y < rows; y++) {

        int counter = 0;
        for (int x = 0; x < cols; x++) {

            if (picture[x][y]) {
                counter ++;
            }
            else
                if (counter != 0) {
                    Data::HintEntry * hint = new Data::HintEntry();
                    hint->amount = counter;
                    hint->hintColor = Qt::black;

                    data->rowHints(y)->append(hint);
                    counter = 0;
                }
        }

        if (counter != 0) {
            //the last row contains something
            Data::HintEntry * hint = new Data::HintEntry();
            hint->amount = counter;
            hint->hintColor = Qt::black;

            data->rowHints(y)->append(hint);
        }
    }
}

void PicFormat::getcols(const QVector< QVector<bool> > & picture, int rows,
        int cols, Data * data)
{

    for (int x = 0; x < cols; x++) {

        int counter = 0;
        for (int y = 0; y < rows; y++) {

            if (picture[x][y]) {
                counter ++;
            }
            else
                if (counter != 0) {
                    Data::HintEntry * hint = new Data::HintEntry();
                    hint->amount = counter;
                    hint->hintColor = Qt::black;

                    data->columnHints(x)->append(hint);
                    counter = 0;
                }
        }
        if (counter != 0) {
            //the last column contains something
            Data::HintEntry * hint = new Data::HintEntry();
            hint->amount = counter;
            hint->hintColor = Qt::black;

            data->columnHints(x)->append(hint);
        }
    }
}

