/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "xbm_format.h"

#include <QImage>
#include <QString>
#include <KDebug>
#include <QColor>
#include <Qt>

#include "ds/data.h"
#include "ds/description.h"

Data * XbmFormat::readFile(const QString & fileName)
{
    kDebug() << "Loading file [" << fileName << "]";

    QImage image;
    if (!image.load(fileName) ) {
        kDebug() << "FAILURE: could not load [" << fileName << "]";
        return 0;
    }

    kDebug() << "Loaded [" << fileName << "]: [" << image.width() << "/" << image.height() << "]";

	/*Do some magic to extract a nice title from the filename*/
    QString name = fileName.toLower();
    name.remove(0, name.lastIndexOf("/")+1);
    name.remove(name.lastIndexOf("."), name.size());
	QStringList nameList = name.split(QRegExp("[\\s_\\.]+"), QString::SkipEmptyParts);
	name = "";
	for (int i = 0; i < nameList.size(); i++) {
		QString s = nameList.at(i);
		s[0] = s[0].toUpper();
		name.append(s+' ');
	}
	name.remove(name.length()-1, name.length());

	/*Wrap it up to a nice description*/
    Description * desc = new Description();
    desc->setName(name);

    Data * retval = image2Data(image);
    retval->setDescription(desc);

    return retval;

}

Data * XbmFormat::readData(const QByteArray & data)
{
    QImage image;

    if (!image.loadFromData(data) ) {
        kDebug() << "FAILURE: could not load from data";
        return NULL;
    }

    kDebug() << "Loaded from data: [" << image.width() << "/" << image.height() << "]";

    return image2Data(image);
}

Data * XbmFormat::image2Data(const QImage & image)
{

    Data * data = new Data();

    QList<QList<Data::HintEntry*> *> * rowHints = new QList<QList<Data::HintEntry*> *> ();
    for (int y = 0; y < image.height(); y++) {

        QList<Data::HintEntry*> * currentRow = new QList<Data::HintEntry*>();

        int counter = 0;
        QColor previousColor = Qt::transparent;
        for (int x = 0; x < image.width(); x++) {

            /*color of current pixel*/
            QColor color = QColor::fromRgba(image.pixel(x, y));

            if (color != previousColor) {

                if (counter != 0 && previousColor != Qt::white) {

                    Data::HintEntry * hint = new Data::HintEntry();
                    hint->amount = counter;
                    hint->hintColor = previousColor;
                    hint->hintColor.setAlpha(255);

                    currentRow->append(hint);
                }

                counter = 0;
            }


            counter++;
            previousColor = color;
        }

        if (counter != 0 && previousColor != Qt::white) {
            Data::HintEntry * hint = new Data::HintEntry();
            hint->amount = counter;
            hint->hintColor = previousColor;
            hint->hintColor.setAlpha(255);

            currentRow->append(hint);
        }
        rowHints->append(currentRow);
    }

    data->setRowHints(rowHints);

    QList<QList<Data::HintEntry*> *> * colHints = new QList<QList<Data::HintEntry*> *> ();
    for (int x = 0; x < image.width(); x++) {

        QList<Data::HintEntry*> * currentCol = new QList<Data::HintEntry*>();

        int counter = 0;
        QColor previousColor = Qt::transparent;
        for (int y = 0; y < image.height(); y++) {

            QColor color = QColor::fromRgba(image.pixel(x, y));
            if (color != previousColor) {

                if (counter != 0 && previousColor != Qt::white) {
                    Data::HintEntry * hint = new Data::HintEntry();
                    hint->amount = counter;
                    hint->hintColor = previousColor;
                    hint->hintColor.setAlpha(255);

                    currentCol->append(hint);
                }

                counter = 0;
            }

            counter++;
            previousColor = color;
        }
        if (counter != 0  && previousColor != Qt::white) {
            Data::HintEntry * hint = new Data::HintEntry();
            hint->amount = counter;
            hint->hintColor = previousColor;
            hint->hintColor.setAlpha(255);

            currentCol->append(hint);
        }
        colHints->append(currentCol);
    }
    data->setColumnHints(colHints);

    data->setDescription(NULL);

    data->update();

    return data;
}
