/* objectitem.cpp

    Parsek - free client for playing 4X space strategy games
    Copyright (C) 2006-2007  Jure Repinc <jlp@holodeck1.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "objectitem.h"

ObjectItem::ObjectItem(ObjectItem *parent)
{
    m_parent = parent;
}

ObjectItem::~ObjectItem()
{
    qDeleteAll(children);
}

void ObjectItem::appendChild(ObjectItem *child)
{
    children.append(child);
}

ObjectItem* ObjectItem::child(int row)
{
    return children.value(row);
}

int ObjectItem::childrenCount() const
{
    if (children.isEmpty())
        return 0;
    else
        return children.count();
}

QString ObjectItem::typestring() const
{
    switch(m_type)
    {
        case Universe:
            return QString("Universe");
        case Galaxy:
            return QString("Galaxy");
        case StarSystem:
            return QString("Star System");
        case Planet:
            return QString("Planet");
        case Fleet:
            return QString("Fleet");
        default:
            return QString("Unknown");
    }
}

int ObjectItem::row() const
{
    if(m_parent)
        return m_parent->children.indexOf(const_cast<ObjectItem*>(this));
    return 0;
}

ObjectItem *ObjectItem::parent()
{
    return m_parent;
}

quint32 ObjectItem::id() const
{
    return m_id;
}

void ObjectItem::setId(quint32 id)
{
    m_id = id;
}

ObjectItem::ObjectType ObjectItem::type() const
{
    return m_type;
}

void ObjectItem::setType(ObjectType type)
{
    m_type = type;
}

QString ObjectItem::name() const
{
    return m_name;
}

void ObjectItem::setName(const QString &name)
{
    m_name = name;
}
QString ObjectItem::description() const
{
    return m_description;
}

void ObjectItem::setDescription(const QString &desc)
{
    m_description = desc;
}

quint64 ObjectItem::modTime() const
{
    return m_modTime;
}

void ObjectItem::setModTime(quint64 time)
{
    m_modTime = time;
}

quint32 ObjectItem::parentid() const
{
    return m_parentid;
}
void ObjectItem::setParentId(quint32 id)
{
    m_parentid = id;
}
void ObjectItem::setParameters(std::list<ObjectParameterGroup*> parameters)
{
    m_parameters = QList<ObjectParameterGroup*>::fromStdList(parameters);
}

QList<ObjectParameterGroup*> ObjectItem::parameterslist() const
{
    return m_parameters;
}
